/*
    SCRIPT CREACIN TABLAS BASE DE DATOS
    Fecha: 11/04/2011
    Autor: Francisco Amador Risquez Teva
    Proyecto: gPV
*/


--
--CAMBIAMOS AL ESQUEMA DE BASE DE DATOS: BD
--
ALTER SESSION SET CURRENT_SCHEMA = BD;


--
--TABLA ELECTRODOMSTICOS
--
CREATE TABLE ELECTRODOMESTICOS (
  IDELECTRODOMESTICO  NUMBER                    NOT NULL,
  DSELECTRODOMESTICO  VARCHAR2(30)              NOT NULL,
  FXALTA              DATE                      DEFAULT SYSDATE               NOT NULL,
  FXBAJA              DATE
);

--
--CLAVE PRIMARIA, INDICES Y RELACIONES
--
ALTER TABLE ELECTRODOMESTICOS ADD (
  CONSTRAINT ELECTRODOMESTICOS_PK PRIMARY KEY(IDELECTRODOMESTICO),
  CONSTRAINT ELECTRODOMESTICOS_U01 UNIQUE (DSELECTRODOMESTICO)
);


--
--TABLA ESTADOS
--
CREATE TABLE ESTADOS (
  IDESTADO            CHAR(1)                   NOT NULL,
  DSESTADO            VARCHAR2(15)              NOT NULL
);

--CLAVE PRIMARIA, INDICES Y RELACIONES
ALTER TABLE ESTADOS ADD (
  CONSTRAINT ESTADOS_PK PRIMARY KEY(IDESTADO),
  CONSTRAINT ESTADOS_U01 UNIQUE (DSESTADO)
);

-- INSERTAMOS VALORES DE LOS ESTADOS
INSERT INTO ESTADOS VALUES ('P', 'PENDIENTE');
INSERT INTO ESTADOS VALUES ('A', 'ANULADA');
INSERT INTO ESTADOS VALUES ('T', 'ASIGNADA');
INSERT INTO ESTADOS VALUES ('C', 'CERRADA');


--
--TABLA LOG
--
CREATE TABLE LOG (
  IDLOG            NUMBER                    NOT NULL,
  PROCEDIMIENTO    VARCHAR2(50)              NOT NULL,
  PARAMIN          VARCHAR2(1000),
  PARAMOUT         VARCHAR2(1000),
  FXIN             DATE                      NOT NULL,
  FXOUT            DATE                      NOT NULL,
  RESULTADO        VARCHAR2(255)             NOT NULL
);

--CLAVE PRIMARIA, INDICES Y RELACIONES
ALTER TABLE LOG ADD (
  CONSTRAINT LOG_PK PRIMARY KEY(IDLOG)
);


--
--TABLA PROVINCIAS
--
CREATE TABLE PROVINCIAS (
  COPROVINCIA      CHAR(2)                   NOT NULL,
  DSPROVINCIA      VARCHAR2(30)              NOT NULL
);

--CLAVE PRIMARIA, INDICES Y RELACIONES
ALTER TABLE PROVINCIAS ADD (
  CONSTRAINT PROVINCIAS_PK PRIMARY KEY(COPROVINCIA),
  CONSTRAINT PROVINCIAS_U01 UNIQUE (DSPROVINCIA)
);


--
--TABLA POBLACIONES
--
CREATE TABLE POBLACIONES (
  CODPOSTAL        CHAR(5)                   NOT NULL,
  DSPOBLACION      VARCHAR2(50)              NOT NULL,
  COPROVINCIA      CHAR(2)                   NOT NULL
);

--CLAVE PRIMARIA, INDICES Y RELACIONES
ALTER TABLE POBLACIONES ADD (
  CONSTRAINT POBLACIONES_PK PRIMARY KEY(CODPOSTAL),
  CONSTRAINT PPOBLACIONES_U01 UNIQUE (DSPOBLACION),
  CONSTRAINT POBLACIONES_R01 FOREIGN KEY(COPROVINCIA) REFERENCES PROVINCIAS(COPROVINCIA)
);


--
--TABLA SERVICIOS
--
CREATE TABLE SERVICIOS (
  IDSERVICIO       NUMBER                   NOT NULL,
  DSSERVICIO       VARCHAR2(30)             NOT NULL,
  FXALTA           DATE                     DEFAULT SYSDATE               NOT NULL,
  FXBAJA           DATE
);

--CLAVE PRIMARIA, INDICES Y RELACIONES
ALTER TABLE SERVICIOS ADD (
  CONSTRAINT SERVICIOS_PK PRIMARY KEY(IDSERVICIO),
  CONSTRAINT SERVICIOS_U01 UNIQUE (DSSERVICIO)
);


--
--TABLA TECNICOS
--
CREATE TABLE TECNICOS (
  IDTECNICO           NUMBER                    NOT NULL,
  NOMBRE              VARCHAR2(30)              NOT NULL,
  APELLIDOS           VARCHAR2(50)              NOT NULL,
  NIF                 VARCHAR2(15)              NOT NULL,
  TELMOVIL            VARCHAR2(15)              NOT NULL,
  FXALTA              DATE                      DEFAULT SYSDATE               NOT NULL,
  FXBAJA              DATE
);

--CLAVE PRIMARIA, INDICES Y RELACIONES
ALTER TABLE TECNICOS ADD (
  CONSTRAINT TECNICOS_PK PRIMARY KEY(IDTECNICO),
  CONSTRAINT TECNICOS_U01 UNIQUE (NIF)
);


--
--TABLA TECNICOSELECTRODOMESTICOS
--
CREATE TABLE TECNICOSELECTRODOMESTICOS (
  IDTECNICO           NUMBER                    NOT NULL,
  IDELECTRODOMESTICO  NUMBER                    NOT NULL
);

--CLAVE PRIMARIA, INDICES Y RELACIONES
ALTER TABLE TECNICOSELECTRODOMESTICOS ADD (
  CONSTRAINT TECNICOSELECTRODOMESTICOS_PK PRIMARY KEY(IDTECNICO, IDELECTRODOMESTICO),
  CONSTRAINT TECNICOSELECTRODOMESTICOS_R01 FOREIGN KEY(IDTECNICO) REFERENCES TECNICOS(IDTECNICO),
  CONSTRAINT TECNICOSELECTRODOMESTICOS_R02 FOREIGN KEY(IDELECTRODOMESTICO) REFERENCES ELECTRODOMESTICOS(IDELECTRODOMESTICO)
);


--
--TABLA TECNICOSSERVICIOS
--
CREATE TABLE TECNICOSSERVICIOS (
  IDTECNICO           NUMBER                    NOT NULL,
  IDSERVICIO          NUMBER                    NOT NULL
);

--CLAVE PRIMARIA, INDICES Y RELACIONES
ALTER TABLE TECNICOSSERVICIOS ADD (
  CONSTRAINT TECNICOSSERVICIOS_PK PRIMARY KEY(IDTECNICO, IDSERVICIO),
  CONSTRAINT TECNICOSSERVICIOS_R01 FOREIGN KEY(IDTECNICO) REFERENCES TECNICOS(IDTECNICO),
  CONSTRAINT TECNICOSSERVICIOS_R02 FOREIGN KEY(IDSERVICIO) REFERENCES SERVICIOS(IDSERVICIO)
);


--
--TABLA TELEOPERADORES
--
CREATE TABLE TELEOPERADORES (
  IDTELEOPERADOR      NUMBER                    NOT NULL,
  NOMBRE              VARCHAR2(30)              NOT NULL,
  APELLIDOS           VARCHAR2(50)              NOT NULL,
  NIF                 VARCHAR2(15)              NOT NULL,
  FXALTA              DATE                      DEFAULT SYSDATE               NOT NULL,
  FXBAJA              DATE
);

--CLAVE PRIMARIA, INDICES Y RELACIONES
ALTER TABLE TELEOPERADORES ADD (
  CONSTRAINT TELEOPERADORES_PK PRIMARY KEY(IDTELEOPERADOR),
  CONSTRAINT TELEOPERADORES_U01 UNIQUE (NIF)
);


--
--TABLA ZONAS
--
CREATE TABLE ZONAS (
  IDZONA           NUMBER                    NOT NULL,
  DSZONA           VARCHAR2(30)              NOT NULL
);

--CLAVE PRIMARIA, INDICES Y RELACIONES
ALTER TABLE ZONAS ADD (
  CONSTRAINT ZONAS_PK PRIMARY KEY(IDZONA),
  CONSTRAINT ZONAS_U01 UNIQUE (DSZONA)
);


--
--TABLA ZONASPOBLACIONES
--
CREATE TABLE ZONASPOBLACIONES (
  IDZONA           NUMBER                    NOT NULL,
  CODPOSTAL        CHAR(5)                   NOT NULL
);

--CLAVE PRIMARIA, INDICES Y RELACIONES
ALTER TABLE ZONASPOBLACIONES ADD (
  CONSTRAINT ZONASPOBLACIONES_PK PRIMARY KEY(IDZONA, CODPOSTAL),
  CONSTRAINT ZONASPOBLACIONES_R01 FOREIGN KEY(IDZONA) REFERENCES ZONAS(IDZONA),
  CONSTRAINT ZONASPOBLACIONES_R02 FOREIGN KEY(CODPOSTAL) REFERENCES POBLACIONES(CODPOSTAL)
);


--
--TABLA TECNICOSZONAS
--
CREATE TABLE TECNICOSZONAS (
  IDTECNICO           NUMBER                    NOT NULL,
  IDZONA              NUMBER                    NOT NULL
);

--CLAVE PRIMARIA, INDICES Y RELACIONES
ALTER TABLE TECNICOSZONAS ADD (
  CONSTRAINT TECNICOSZONAS_PK PRIMARY KEY(IDTECNICO, IDZONA),
  CONSTRAINT TECNICOSZONAS_R01 FOREIGN KEY(IDTECNICO) REFERENCES TECNICOS(IDTECNICO),
  CONSTRAINT TECNICOSZONAS_R02 FOREIGN KEY(IDZONA) REFERENCES ZONAS(IDZONA)
);


--
--TABLA CLIENTES
--
CREATE TABLE CLIENTES (
  IDCLIENTE           NUMBER                    NOT NULL,
  NOMBRE              VARCHAR2(30)              NOT NULL,
  APELLIDOS           VARCHAR2(50)              NOT NULL,
  NIF                 VARCHAR2(15)              NOT NULL,
  DIRECCION           VARCHAR2(100)             NOT NULL,
  CODPOSTAL           CHAR(5)                   NOT NULL,
  TELFIJO             VARCHAR2(15)              NOT NULL,
  TELMOVIL            VARCHAR2(15)              NOT NULL,
  EMAIL               VARCHAR2(50)              NOT NULL,
  FXALTA              DATE                      DEFAULT SYSDATE               NOT NULL
);

--CLAVE PRIMARIA, INDICES Y RELACIONES
ALTER TABLE CLIENTES ADD (
  CONSTRAINT CLIENTES_PK PRIMARY KEY(IDCLIENTE),
  CONSTRAINT CLIENTES_U01 UNIQUE (NIF),
  CONSTRAINT CLIENTES_R01 FOREIGN KEY(CODPOSTAL) REFERENCES POBLACIONES(CODPOSTAL)
);


--
--TABLA PRODUCTOS
--
CREATE TABLE PRODUCTOS (
  IDPRODUCTO          NUMBER                    NOT NULL,
  IDELECTRODOMESTICO  NUMBER                    NOT NULL,
  MARCA               VARCHAR2(30)              NOT NULL,
  MODELO              VARCHAR2(50)              NOT NULL,
  NUMSERIE            VARCHAR2(15)              NOT NULL,
  FXCOMPRA            DATE                      NOT NULL,
  DIRECCION           VARCHAR2(100)             NOT NULL,
  CODPOSTAL           CHAR(5)                   NOT NULL,
  IDCLIENTE           NUMBER                    NOT NULL
);

--CLAVE PRIMARIA, INDICES Y RELACIONES
ALTER TABLE PRODUCTOS ADD (
  CONSTRAINT PRODUCTOS_PK PRIMARY KEY(IDPRODUCTO),
  CONSTRAINT PRODUCTOS_U01 UNIQUE (NUMSERIE),
  CONSTRAINT PRODUCTOS_R01 FOREIGN KEY(IDELECTRODOMESTICO) REFERENCES ELECTRODOMESTICOS(IDELECTRODOMESTICO),
  CONSTRAINT PRODUCTOS_R02 FOREIGN KEY(CODPOSTAL) REFERENCES POBLACIONES(CODPOSTAL),
  CONSTRAINT PRODUCTOS_R03 FOREIGN KEY(IDCLIENTE) REFERENCES CLIENTES(IDCLIENTE)
);


--
--TABLA PETICIONES
--
CREATE TABLE PETICIONES (
  IDPETICION          NUMBER                    NOT NULL,
  IDPRODUCTO          NUMBER                    NOT NULL,
  IDSERVICIO          NUMBER                    NOT NULL,
  IDESTADO            CHAR(1)                   DEFAULT 'P'                   NOT NULL,
  IDTELEOPERADOR      NUMBER                    NOT NULL,
  IDTECNICO           NUMBER,
  FXRECLAMACION       DATE                      DEFAULT SYSDATE               NOT NULL,
  FXCIERRE            DATE
);

--CLAVE PRIMARIA, INDICES Y RELACIONES
ALTER TABLE PETICIONES ADD (
  CONSTRAINT PETICIONES_PK PRIMARY KEY(IDPETICION),
  CONSTRAINT PETICIONES_R01 FOREIGN KEY(IDPRODUCTO) REFERENCES PRODUCTOS(IDPRODUCTO),
  CONSTRAINT PETICIONES_R02 FOREIGN KEY(IDSERVICIO) REFERENCES SERVICIOS(IDSERVICIO),
  CONSTRAINT PETICIONES_R03 FOREIGN KEY(IDESTADO) REFERENCES ESTADOS(IDESTADO),
  CONSTRAINT PETICIONES_R04 FOREIGN KEY(IDTELEOPERADOR) REFERENCES TELEOPERADORES(IDTELEOPERADOR),
  CONSTRAINT PETICIONES_R05 FOREIGN KEY(IDTECNICO) REFERENCES TECNICOS(IDTECNICO)
);


--
--TABLA TRABAJOS
--
CREATE TABLE TRABAJOS (
  IDTRABAJO           NUMBER                    NOT NULL,
  IDPETICION          NUMBER                    NOT NULL,
  IDTECNICO           NUMBER                    NOT NULL,
  FXTRABAJO           DATE                      DEFAULT SYSDATE               NOT NULL,
  DSTRABAJO           VARCHAR2(255)             NOT NULL,
  IMPORTE             NUMBER                    DEFAULT 0                     NOT NULL,
  GARANTIA            NUMBER                    DEFAULT 0                     NOT NULL
);

--CLAVE PRIMARIA, INDICES Y RELACIONES
ALTER TABLE TRABAJOS ADD (
  CONSTRAINT TRABAJOS_PK PRIMARY KEY(IDTRABAJO),
  CONSTRAINT TRABAJOS_R01 FOREIGN KEY(IDPETICION) REFERENCES PETICIONES(IDPETICION),
  CONSTRAINT TRABAJOS_R02 FOREIGN KEY(IDTECNICO) REFERENCES TECNICOS(IDTECNICO),
  CONSTRAINT TRABAJOS_C01 CHECK (IMPORTE >= 0),
  CONSTRAINT TRABAJOS_C02 CHECK (GARANTIA IN (0,1))
);


--
--TABLA ENCUESTAS
--
CREATE TABLE ENCUESTAS (
  IDENCUESTA          NUMBER                    NOT NULL,
  IDPETICION          NUMBER                    NOT NULL,
  IDTELEOPERADOR      NUMBER                    NOT NULL,
  FXENCUESTA          DATE                      DEFAULT SYSDATE               NOT NULL,
  VALORACION          NUMBER                    DEFAULT 0                     NOT NULL,
  COMENTARIO          VARCHAR2(255)             
);

--CLAVE PRIMARIA, INDICES Y RELACIONES
ALTER TABLE ENCUESTAS ADD (
  CONSTRAINT ENCUESTAS_PK PRIMARY KEY(IDENCUESTA),
  CONSTRAINT ENCUESTAS_R01 FOREIGN KEY(IDPETICION) REFERENCES PETICIONES(IDPETICION),
  CONSTRAINT ENCUESTAS_R02 FOREIGN KEY(IDTELEOPERADOR) REFERENCES TELEOPERADORES(IDTELEOPERADOR),
  CONSTRAINT ENCUESTAS_C01 CHECK (VALORACION BETWEEN 0 AND 9)
);

