/*
    SCRIPT CREACIN TABLAS ALMACEN DE DATOS
    Fecha: 15/04/2011
    Autor: Francisco Amador Risquez Teva
    Proyecto: gPV
*/


--
--CAMBIAMOS AL ESQUEMA DE ALMACEN DE DATOS: DW
--
ALTER SESSION SET CURRENT_SCHEMA = DW;


--
--TABLA D_ELECTRODOMSTICOS
--
CREATE TABLE D_ELECTRODOMESTICOS (
  IDELECTRODOMESTICO  NUMBER                    NOT NULL,
  DSELECTRODOMESTICO  VARCHAR2(30)              NOT NULL
);

--
--CLAVE PRIMARIA, INDICES Y RELACIONES
--
ALTER TABLE D_ELECTRODOMESTICOS ADD (
  CONSTRAINT D_ELECTRODOMESTICOS_PK PRIMARY KEY(IDELECTRODOMESTICO)
);


--
--TABLA D_LOCALIZACION
--
CREATE TABLE D_LOCALIZACION (
  IDLOCALIZACION   VARCHAR2(15)              NOT NULL,
  DSPOBLACION      VARCHAR2(50)              NOT NULL,
  DSPROVINCIA      VARCHAR2(30)              NOT NULL,
  DSZONA           VARCHAR2(30)              NOT NULL
);

--CLAVE PRIMARIA, INDICES Y RELACIONES
ALTER TABLE D_LOCALIZACION ADD (
  CONSTRAINT D_LOCALIZACION_PK PRIMARY KEY(IDLOCALIZACION)
);


--
--TABLA D_SERVICIOS
--
CREATE TABLE D_SERVICIOS (
  IDSERVICIO       NUMBER                   NOT NULL,
  DSSERVICIO       VARCHAR2(30)             NOT NULL
);

--CLAVE PRIMARIA, INDICES Y RELACIONES
ALTER TABLE D_SERVICIOS ADD (
  CONSTRAINT D_SERVICIOS_PK PRIMARY KEY(IDSERVICIO)
);


--
--TABLA D_TECNICOS
--
CREATE TABLE D_TECNICOS (
  IDTECNICO        NUMBER                    NOT NULL,
  NOMBRE           VARCHAR2(80)              NOT NULL
);

--CLAVE PRIMARIA, INDICES Y RELACIONES
ALTER TABLE D_TECNICOS ADD (
  CONSTRAINT D_TECNICOS_PK PRIMARY KEY(IDTECNICO)
);


--
--TABLA D_TELEOPERADORES
--
CREATE TABLE D_TELEOPERADORES (
  IDTELEOPERADOR   NUMBER                    NOT NULL,
  NOMBRE           VARCHAR2(80)              NOT NULL
);

--CLAVE PRIMARIA, INDICES Y RELACIONES
ALTER TABLE D_TELEOPERADORES ADD (
  CONSTRAINT D_TELEOPERADORES_PK PRIMARY KEY(IDTELEOPERADOR)
);


--
--TABLA D_TEMPORAL
--
CREATE TABLE D_TEMPORAL (
  IDTEMPORAL       CHAR(6)                   NOT NULL,
  MES              CHAR(2)                   NOT NULL,
  ANO              CHAR(4)                   NOT NULL
);

--CLAVE PRIMARIA, INDICES Y RELACIONES
ALTER TABLE D_TEMPORAL ADD (
  CONSTRAINT D_TEMPORAL_PK PRIMARY KEY(IDTEMPORAL)
);


--
--TABLA H_PETICIONES
--
CREATE TABLE H_PETICIONES (
  IDTEMPORAL          CHAR(6)                   NOT NULL,
  IDLOCALIZACION      VARCHAR2(15)              NOT NULL,
  IDSERVICIO          NUMBER                    NOT NULL,
  IDELECTRODOMESTICO  NUMBER                    NOT NULL,
  NUMPETICIONES       NUMBER                    DEFAULT 0                     NOT NULL,
  IMPORTE             NUMBER                    DEFAULT 0                     NOT NULL,
  IMPGARANTIA         NUMBER                    DEFAULT 0                     NOT NULL,
  VALORACIONMEDIA     NUMBER                    DEFAULT 0                     NOT NULL
);


--CLAVE PRIMARIA, INDICES Y RELACIONES
ALTER TABLE H_PETICIONES ADD (
  CONSTRAINT H_PETICIONES_PK PRIMARY KEY(IDTEMPORAL, IDLOCALIZACION, IDSERVICIO, IDELECTRODOMESTICO),
  CONSTRAINT H_PETICIONES_R01 FOREIGN KEY(IDTEMPORAL) REFERENCES D_TEMPORAL(IDTEMPORAL),
  CONSTRAINT H_PETICIONES_R02 FOREIGN KEY(IDLOCALIZACION) REFERENCES D_LOCALIZACION(IDLOCALIZACION),
  CONSTRAINT H_PETICIONES_R03 FOREIGN KEY(IDSERVICIO) REFERENCES D_SERVICIOS(IDSERVICIO),
  CONSTRAINT H_PETICIONES_R04 FOREIGN KEY(IDELECTRODOMESTICO) REFERENCES D_ELECTRODOMESTICOS(IDELECTRODOMESTICO)
);


--
--TABLA H_TECNICOS
--
CREATE TABLE H_TECNICOS (
  IDTEMPORAL          CHAR(6)                   NOT NULL,
  IDTECNICO           NUMBER                    NOT NULL,
  IDSERVICIO          NUMBER                    NOT NULL,
  IDELECTRODOMESTICO  NUMBER                    NOT NULL,
  NUMTRABAJOS         NUMBER                    DEFAULT 0                     NOT NULL,
  IMPORTE             NUMBER                    DEFAULT 0                     NOT NULL,
  IMPGARANTIA         NUMBER                    DEFAULT 0                     NOT NULL
);


--CLAVE PRIMARIA, INDICES Y RELACIONES
ALTER TABLE H_TECNICOS ADD (
  CONSTRAINT H_TECNICOS_PK PRIMARY KEY(IDTEMPORAL, IDTECNICO, IDSERVICIO, IDELECTRODOMESTICO),
  CONSTRAINT H_TECNICOS_R01 FOREIGN KEY(IDTEMPORAL) REFERENCES D_TEMPORAL(IDTEMPORAL),
  CONSTRAINT H_TECNICOS_R02 FOREIGN KEY(IDTECNICO) REFERENCES D_TECNICOS(IDTECNICO),
  CONSTRAINT H_TECNICOS_R03 FOREIGN KEY(IDSERVICIO) REFERENCES D_SERVICIOS(IDSERVICIO),
  CONSTRAINT H_TECNICOS_R04 FOREIGN KEY(IDELECTRODOMESTICO) REFERENCES D_ELECTRODOMESTICOS(IDELECTRODOMESTICO)
);


--
--TABLA H_TELEOPERADORES
--
CREATE TABLE H_TELEOPERADORES (
  IDTEMPORAL          CHAR(6)                   NOT NULL,
  IDTELEOPERADOR      NUMBER                    NOT NULL,
  IDSERVICIO          NUMBER                    NOT NULL,
  IDELECTRODOMESTICO  NUMBER                    NOT NULL,
  NUMPETICIONES       NUMBER                    DEFAULT 0                     NOT NULL,
  NUMENCUESTAS        NUMBER                    DEFAULT 0                     NOT NULL
);


--CLAVE PRIMARIA, INDICES Y RELACIONES
ALTER TABLE H_TELEOPERADORES ADD (
  CONSTRAINT H_TELEOPERADORES_PK PRIMARY KEY(IDTEMPORAL, IDTELEOPERADOR, IDSERVICIO, IDELECTRODOMESTICO),
  CONSTRAINT H_TELEOPERADORES_R01 FOREIGN KEY(IDTEMPORAL) REFERENCES D_TEMPORAL(IDTEMPORAL),
  CONSTRAINT H_TELEOPERADORES_R02 FOREIGN KEY(IDTELEOPERADOR) REFERENCES D_TELEOPERADORES(IDTELEOPERADOR),
  CONSTRAINT H_TELEOPERADORES_R03 FOREIGN KEY(IDSERVICIO) REFERENCES D_SERVICIOS(IDSERVICIO),
  CONSTRAINT H_TELEOPERADORES_R04 FOREIGN KEY(IDELECTRODOMESTICO) REFERENCES D_ELECTRODOMESTICOS(IDELECTRODOMESTICO)
);

