/*
    PACKAGE CONTROL
    Fecha: 18/04/2011
    Autor: Francisco Amador Risquez Teva
    Proyecto: gPV
*/


/*
    ESPECIFICACIN DEL PAQUETE
    SUBSISTEMA CONFIGURACION
*/
CREATE OR REPLACE PACKAGE BD.CONTROL AS

    /*
        LOG
    */   
    --RF69
    PROCEDURE GRABA_LOG(procedimiento IN BD.LOG.PROCEDIMIENTO%TYPE,
                        paramIN IN BD.LOG.PARAMIN%TYPE,
                        paramOUT IN BD.LOG.PARAMOUT%TYPE,
                        fxInicio IN BD.LOG.FXIN%TYPE,
                        fxFin IN BD.LOG.FXOUT%TYPE,
                        pResultado IN BD.LOG.RESULTADO%TYPE);
 
    --RF70
    PROCEDURE GET_LOG( pFxInicio IN BD.LOG.FXIN%TYPE
                     , pFxFin IN BD.LOG.FXOUT%TYPE
                     , rs OUT SYS_REFCURSOR);
   
END CONTROL;


/*
    CUERPO DEL PAQUETE
    SUBSISTEMA CONTROL
*/
CREATE OR REPLACE PACKAGE BODY BD.CONTROL AS

/*
    LOG
*/   

    --RF69
    PROCEDURE GRABA_LOG(procedimiento IN BD.LOG.PROCEDIMIENTO%TYPE,
                        paramIN IN BD.LOG.PARAMIN%TYPE,
                        paramOUT IN BD.LOG.PARAMOUT%TYPE,
                        fxInicio IN BD.LOG.FXIN%TYPE,
                        fxFin IN BD.LOG.FXOUT%TYPE,
                        pResultado IN BD.LOG.RESULTADO%TYPE) IS
    /*
        RFxx - CASO DE USO: GRABAR LOG
        PARAMETROS ENTRADA:
            procedimiento:  nombre procedimiento
            paramIN:        parmetros de entrada
            paramOUT:       parmetros de salida
            fxInicio:       fecha/hora de incio del procedimiento
            fxFin:          fecha/hora de fin del procedimiento
            pResultado:      resultado del procedimiento ejecutado
        PARAMETROS SALIDA:
            ninguno
    */
        --Hacemos la transaccin autonoma (el commit slo le afecta a ella)
        PRAGMA AUTONOMOUS_TRANSACTION;
        
    BEGIN
    

        INSERT INTO LOG( PROCEDIMIENTO
                       , PARAMIN
                       , PARAMOUT
                       , FXIN
                       , FXOUT
                       , RESULTADO
                       )
        VALUES( procedimiento
              , paramIN
              , paramOUT
              , fxInicio
              , fxFin
              , pResultado);
              
        -- Transaccin autnoma
        COMMIT;

    END GRABA_LOG;


    -- RF70
    PROCEDURE GET_LOG( pFxInicio IN BD.LOG.FXIN%TYPE
                     , pFxFin IN BD.LOG.FXOUT%TYPE
                     , rs OUT SYS_REFCURSOR)
    IS
    /*
        RF70 - CASO DE USO: CONSULTA LOG
        PARAMETROS ENTRADA:
            pFxInicio: fecha de incio
            pFxFin: fecha de fin
        PARAMETROS SALIDA:
            rs: cursor con el/los registro obtenido
    */
        procedimiento BD.LOG.PROCEDIMIENTO%TYPE;
        paramIN       BD.LOG.PARAMIN%TYPE;
        paramOUT      BD.LOG.PARAMOUT%TYPE;
        dateIN        BD.LOG.FXIN%TYPE;
        dateOUT       BD.LOG.FXOUT%TYPE;
        resultado     BD.LOG.RESULTADO%TYPE;
    BEGIN
    
        -- Asignamos valores entrada
        procedimiento := $$PLSQL_UNIT || '.GET_LOG';
        paramIN       := 'pFxInicio = ' || pFxInicio 
                      || ' ,pFxFin = ' || pFxFin;
        dateIN        := SYSDATE;
        resultado     := 'OK. Los movimientos del Log se han consultado con xito';

        -- Realizamos la consulta
        OPEN rs FOR
        SELECT * 
        FROM LOG
        WHERE fxIn >= pFxInicio
          AND fxOut <= pFxFin;
        
        -- Asignamos valores salida
        paramOUT      := 'rs = Cursor de movimientos del LOG.';
        dateOUT       := SYSDATE;
        
        -- Grabamos en el LOG
        CONTROL.GRABA_LOG( procedimiento
                         , paramIN
                         , paramOUT 
                         , dateIN
                         , dateOUT
                         , resultado );
         
         EXCEPTION
            WHEN OTHERS THEN
                CASE (SQLCODE)
                    WHEN 100 THEN
                        -- Registro no encontrado
                        resultado := 'ERROR: Registro(s) no encontrado(s).';
                    ELSE
                        -- Error no controlado
                        resultado := 'ERROR: ' ||  SQLERRM;
                END CASE;            

                -- Generamos un cursor vacio
                OPEN rs FOR
                SELECT * FROM DUAL WHERE DUMMY IS NULL;
                
                -- Asignamos valores salida
                paramOUT      := 'rs = Cursor vaco.';
                dateOUT       := SYSDATE;
                
                -- Mensaje de error
                DBMS_OUTPUT.put_line (resultado);

                -- Grabamos en el LOG
                CONTROL.GRABA_LOG( procedimiento
                                 , paramIN
                                 , paramOUT 
                                 , dateIN
                                 , dateOUT
                                 , resultado );
    END GET_LOG;

  
END CONTROL;
