/*
    PACKAGE CONFIGURACION
    Fecha: 20/04/2011
    Autor: Francisco Amador Risquez Teva
    Proyecto: gPV
*/



/*
    ESPECIFICACIN DEL PAQUETE
    SUBSISTEMA CONFIGURACION
*/
CREATE OR REPLACE PACKAGE BD.CONFIGURACION AS


    /*
        SERVICIOS
    */     
    --RF01
    PROCEDURE INS_SERVICIO( ds IN BD.SERVICIOS.DSSERVICIO%TYPE 
                          , resultado OUT BD.LOG.RESULTADO%TYPE );
    
    --RF02
    PROCEDURE DEL_SERVICIO( id IN BD.SERVICIOS.IDSERVICIO%TYPE 
                          , resultado OUT BD.LOG.RESULTADO%TYPE ); 

    --RF03
    PROCEDURE RES_SERVICIO( id IN BD.SERVICIOS.IDSERVICIO%TYPE 
                          , resultado OUT BD.LOG.RESULTADO%TYPE );
    
    --RF04
    PROCEDURE UPD_SERVICIO( id IN BD.SERVICIOS.IDSERVICIO%TYPE
                          , ds IN BD.SERVICIOS.DSSERVICIO%TYPE
                          , resultado OUT BD.LOG.RESULTADO%TYPE ); 
    
    --RF05
    PROCEDURE GET_SERVICIO( id IN BD.SERVICIOS.IDSERVICIO%TYPE
                          , rs OUT SYS_REFCURSOR);

    /*
        ELECTRODOMSTICOS
    */     
    --RF06
    PROCEDURE INS_ELECTRODOMESTICO( ds IN BD.ELECTRODOMESTICOS.DSELECTRODOMESTICO%TYPE 
                                  , resultado OUT BD.LOG.RESULTADO%TYPE );

    --RF07
    PROCEDURE DEL_ELECTRODOMESTICO( id IN BD.ELECTRODOMESTICOS.IDELECTRODOMESTICO%TYPE 
                                  , resultado OUT BD.LOG.RESULTADO%TYPE );

    --RF08
    PROCEDURE RES_ELECTRODOMESTICO( id IN BD.ELECTRODOMESTICOS.IDELECTRODOMESTICO%TYPE 
                                  , resultado OUT BD.LOG.RESULTADO%TYPE );

    --RF09
    PROCEDURE UPD_ELECTRODOMESTICO( id IN BD.ELECTRODOMESTICOS.IDELECTRODOMESTICO%TYPE
                                  , ds IN BD.ELECTRODOMESTICOS.DSELECTRODOMESTICO%TYPE
                                  , resultado OUT BD.LOG.RESULTADO%TYPE ); 

    --RF10
    PROCEDURE GET_ELECTRODOMESTICO( id IN BD.ELECTRODOMESTICOS.IDELECTRODOMESTICO%TYPE
                                  , rs OUT SYS_REFCURSOR);


    /*
        PROVINCIAS
    */     
    --RF11
    PROCEDURE INS_PROVINCIA( cod IN BD.PROVINCIAS.COPROVINCIA%TYPE
                           , prov IN BD.PROVINCIAS.DSPROVINCIA%TYPE
                           , resultado OUT BD.LOG.RESULTADO%TYPE );

   --RF12
    PROCEDURE DEL_PROVINCIA( cod IN BD.PROVINCIAS.COPROVINCIA%TYPE 
                           , resultado OUT BD.LOG.RESULTADO%TYPE );

    --RF13
    PROCEDURE UPD_PROVINCIA( cod IN BD.PROVINCIAS.COPROVINCIA%TYPE
                           , prov IN BD.PROVINCIAS.DSPROVINCIA%TYPE
                           , resultado OUT BD.LOG.RESULTADO%TYPE ); 

    --RF14
    PROCEDURE GET_PROVINCIA( cod IN BD.PROVINCIAS.COPROVINCIA%TYPE
                           , rs OUT SYS_REFCURSOR);

    /*
        POBLACIONES
    */     
    --RF15
    PROCEDURE INS_POBLACION( cPostal IN BD.POBLACIONES.CODPOSTAL%TYPE
                           , poblacion IN BD.POBLACIONES.DSPOBLACION%TYPE
                           , codProv IN BD.POBLACIONES.COPROVINCIA%TYPE
                           , resultado OUT BD.LOG.RESULTADO%TYPE );

   --RF16
    PROCEDURE DEL_POBLACION( cPostal IN BD.POBLACIONES.CODPOSTAL%TYPE
                           , resultado OUT BD.LOG.RESULTADO%TYPE );

    --RF17
    PROCEDURE UPD_POBLACION( cPostal IN BD.POBLACIONES.CODPOSTAL%TYPE
                           , poblacion IN BD.POBLACIONES.DSPOBLACION%TYPE
                           , codProv IN BD.POBLACIONES.COPROVINCIA%TYPE
                           , resultado OUT BD.LOG.RESULTADO%TYPE ); 

    --RF18
    PROCEDURE GET_POBLACION( cPostal IN BD.POBLACIONES.CODPOSTAL%TYPE
                           , rs OUT SYS_REFCURSOR);


    /*
        ZONAS
    */     
    --RF19
    PROCEDURE INS_ZONA( ds IN BD.ZONAS.DSZONA%TYPE
                      , resultado OUT BD.LOG.RESULTADO%TYPE );

   --RF20
    PROCEDURE DEL_ZONA( id IN BD.ZONAS.IDZONA%TYPE
                      , resultado OUT BD.LOG.RESULTADO%TYPE );

    --RF21
    PROCEDURE UPD_ZONA( id IN BD.ZONAS.IDZONA%TYPE
                      , ds IN BD.ZONAS.DSZONA%TYPE
                      , resultado OUT BD.LOG.RESULTADO%TYPE ); 

    --RF22
    PROCEDURE GET_ZONA( id IN BD.ZONAS.IDZONA%TYPE
                      , rs OUT SYS_REFCURSOR);
                      
    --RF23
    PROCEDURE ASG_PROVINCIAZONA( coProv IN BD.PROVINCIAS.COPROVINCIA%TYPE
                               , zona IN BD.ZONAS.IDZONA%TYPE
                               , resultado OUT BD.LOG.RESULTADO%TYPE ); 
                      
    --RF24
    PROCEDURE ASG_POBLACIONZONA( cp IN BD.POBLACIONES.CODPOSTAL%TYPE
                               , zona IN BD.ZONAS.IDZONA%TYPE
                               , resultado OUT BD.LOG.RESULTADO%TYPE ); 
                      

    --RF25
    PROCEDURE DSG_PROVINCIAZONA( coProv IN BD.PROVINCIAS.COPROVINCIA%TYPE
                               , zona IN BD.ZONAS.IDZONA%TYPE
                               , resultado OUT BD.LOG.RESULTADO%TYPE ); 
                      
    --RF26
    PROCEDURE DSG_POBLACIONZONA( cp IN BD.POBLACIONES.CODPOSTAL%TYPE
                               , zona IN BD.ZONAS.IDZONA%TYPE
                               , resultado OUT BD.LOG.RESULTADO%TYPE ); 

    --RF27
    PROCEDURE GET_POBLACIONESZONA( id IN BD.ZONAS.IDZONA%TYPE
                                 , rs OUT SYS_REFCURSOR);
                      

END CONFIGURACION;



/*
    CUERPO DEL PAQUETE
    SUBSISTEMA CONFIGURACION
*/
CREATE OR REPLACE PACKAGE BODY BD.CONFIGURACION AS

/*
    SERVICIOS
*/     
    -- RF01
    PROCEDURE INS_SERVICIO( ds IN BD.SERVICIOS.DSSERVICIO%TYPE
                          , resultado OUT BD.LOG.RESULTADO%TYPE ) 
    IS
    /*
        RF01 - CASO DE USO: CREAR SERVICIO
        PARAMETROS ENTRADA:
            ds: Descripcin del tipo de servicio
        PARAMETROS SALIDA:
            resultado: Informacin del resultado de la operacin.
    */
        procedimiento BD.LOG.PROCEDIMIENTO%TYPE;
        paramIN       BD.LOG.PARAMIN%TYPE;
        paramOUT      BD.LOG.PARAMOUT%TYPE;
        dateIN        BD.LOG.FXIN%TYPE;
        dateOUT       BD.LOG.FXOUT%TYPE;
    BEGIN
    
        -- Asignamos valores entrada
        procedimiento := $$PLSQL_UNIT|| '.INS_SERVICIO';
        paramIN       := 'dsServicio = ' || ds;
        dateIN        := SYSDATE;
        resultado     := 'OK. El servicio se ha creado con xito';

        INSERT INTO SERVICIOS(DSSERVICIO) VALUES(UPPER(ds));
        
        -- Asignamos valores salida
        paramOUT      := 'resultado = ' || resultado;
        dateOUT       := SYSDATE;
        
        -- Grabamos en el LOG
        CONTROL.GRABA_LOG( procedimiento
                         , paramIN
                         , paramOUT 
                         , dateIN
                         , dateOUT
                         , resultado );
         
         EXCEPTION
            WHEN OTHERS THEN
                CASE (SQLCODE)
                    WHEN -1 THEN
                        -- Registro duplicado
                        resultado := 'ERROR: La descripcin de servicio ya existe.';
                    WHEN -1400 THEN
                        -- Valor nulo
                        resultado := 'ERROR: Se ha intentado almacenar un valor nulo.' || SUBSTR(SQLERRM,INSTR(SQLERRM,'('));
                    ELSE
                        -- Error no controlado
                        resultado := 'ERROR: ' || SQLCODE || ' - ' || SQLERRM;
                END CASE;            

                -- Asignamos valores salida
                paramOUT      := 'resultado = ' || resultado;
                dateOUT       := SYSDATE;
                
                -- Mensaje de error
                DBMS_OUTPUT.put_line (resultado);

                -- Grabamos en el LOG
                CONTROL.GRABA_LOG( procedimiento
                                 , paramIN
                                 , paramOUT 
                                 , dateIN
                                 , dateOUT
                                 , resultado );
    END INS_SERVICIO;


    -- RF02
    PROCEDURE DEL_SERVICIO( id IN BD.SERVICIOS.IDSERVICIO%TYPE 
                          , resultado OUT BD.LOG.RESULTADO%TYPE ) 
    IS
    /*
        RF02 - CASO DE USO: BAJA SERVICIO
        PARAMETROS ENTRADA:
            id: Identificador del tipo de electrodomstico
        PARAMETROS SALIDA:
            resultado: Informacin del resultado de la operacin.
    */
        procedimiento BD.LOG.PROCEDIMIENTO%TYPE;
        paramIN       BD.LOG.PARAMIN%TYPE;
        paramOUT      BD.LOG.PARAMOUT%TYPE;
        dateIN        BD.LOG.FXIN%TYPE;
        dateOUT       BD.LOG.FXOUT%TYPE;
        fechaBaja     BD.SERVICIOS.FXBAJA%TYPE;
    BEGIN
    
        -- Asignamos valores entrada
        procedimiento := $$PLSQL_UNIT|| '.DEL_SERVICIO';
        paramIN       := 'idServicio =' || id;
        dateIN        := SYSDATE;
        resultado     := 'OK. El servicio se ha dado de baja con xito';

        -- Miramos que exista y que no est de baja
        SELECT fxBaja INTO fechaBaja 
        FROM SERVICIOS 
        WHERE idServicio=id;
        
        IF (fechaBaja IS NOT NULL) THEN
            RAISE_APPLICATION_ERROR(-20000,'');        
        END IF;
        
        -- Damos de baja el servicio con fecha del sistema
        UPDATE SERVICIOS 
        SET fxBaja = SYSDATE
        WHERE idServicio = id;
        
        -- Asignamos valores salida
        paramOUT      := 'resultado = ' || resultado;
        dateOUT       := SYSDATE;
        
        -- Grabamos en el LOG
        CONTROL.GRABA_LOG( procedimiento
                         , paramIN
                         , paramOUT 
                         , dateIN
                         , dateOUT
                         , resultado );
         
         EXCEPTION
            WHEN OTHERS THEN
                CASE (SQLCODE)
                    WHEN 100 THEN
                        -- Registro no encontrado
                        resultado := 'ERROR: El identificador de servicio no existe.';
                    WHEN -20000 THEN
                        -- Servicio dado de baja
                        resultado := 'ERROR: El servicio ya est de baja.';
                    ELSE
                        -- Error no controlado
                        resultado := 'ERROR: ' ||  SQLERRM;
                END CASE;            

                -- Asignamos valores salida
                paramOUT      := 'resultado = ' || resultado;
                dateOUT       := SYSDATE;
                
                -- Mensaje de error
                DBMS_OUTPUT.put_line (resultado);

                -- Grabamos en el LOG
                CONTROL.GRABA_LOG( procedimiento
                                 , paramIN
                                 , paramOUT 
                                 , dateIN
                                 , dateOUT
                                 , resultado );
    END DEL_SERVICIO;


    -- RF03
    PROCEDURE RES_SERVICIO( id IN BD.SERVICIOS.IDSERVICIO%TYPE 
                          , resultado OUT BD.LOG.RESULTADO%TYPE ) 
    IS
    /*
        RF03 - CASO DE USO: RECUPERAR SERVICIO
        PARAMETROS ENTRADA:
            id: Identificador del tipo de electrodomstico
        PARAMETROS SALIDA:
            resultado: Informacin del resultado de la operacin.
    */
        procedimiento BD.LOG.PROCEDIMIENTO%TYPE;
        paramIN       BD.LOG.PARAMIN%TYPE;
        paramOUT      BD.LOG.PARAMOUT%TYPE;
        dateIN        BD.LOG.FXIN%TYPE;
        dateOUT       BD.LOG.FXOUT%TYPE;
        fechaBaja     BD.SERVICIOS.FXBAJA%TYPE;
    BEGIN
    
        -- Asignamos valores entrada
        procedimiento := $$PLSQL_UNIT || '.RES_SERVICIO';
        paramIN       := 'idServicio =' || id;
        dateIN        := SYSDATE;
        resultado     := 'OK. El servicio se ha recuperado con xito';

        -- Miramos que exista y que est de baja
        SELECT fxBaja INTO fechaBaja 
        FROM SERVICIOS 
        WHERE idServicio=id;
        
        IF (fechaBaja IS NULL) THEN
            RAISE_APPLICATION_ERROR(-20000,'');        
        END IF;
        
        --  Recuperamos el servicio con fecha del sistema
        UPDATE SERVICIOS 
        SET fxBaja = NULL
        WHERE idServicio = id;
        
        -- Asignamos valores salida
        paramOUT      := 'resultado = ' || resultado;
        dateOUT       := SYSDATE;
        
        -- Grabamos en el LOG
        CONTROL.GRABA_LOG( procedimiento
                         , paramIN
                         , paramOUT 
                         , dateIN
                         , dateOUT
                         , resultado );
         
         EXCEPTION
            WHEN OTHERS THEN
                CASE (SQLCODE)
                    WHEN 100 THEN
                        -- Registro no encontrado
                        resultado := 'ERROR: El identificador de servicio no existe.';
                    WHEN -20000 THEN
                        -- Servicio dado de baja
                        resultado := 'ERROR: El servicio no est de baja.';
                    ELSE
                        -- Error no controlado
                        resultado := 'ERROR: ' ||  SQLERRM;
                END CASE;            

                -- Asignamos valores salida
                paramOUT      := 'resultado = ' || resultado;
                dateOUT       := SYSDATE;
                
                -- Mensaje de error
                DBMS_OUTPUT.put_line (resultado);

                -- Grabamos en el LOG
                CONTROL.GRABA_LOG( procedimiento
                                 , paramIN
                                 , paramOUT 
                                 , dateIN
                                 , dateOUT
                                 , resultado );
    END RES_SERVICIO;


    -- RF04
    PROCEDURE UPD_SERVICIO( id IN BD.SERVICIOS.IDSERVICIO%TYPE
                          , ds IN BD.SERVICIOS.DSSERVICIO%TYPE 
                          , resultado OUT BD.LOG.RESULTADO%TYPE ) 
    IS
    /*
        RF04 - CASO DE USO: MODIFICAR SERVICIO
        PARAMETROS ENTRADA:
            id: identificador del servicio
            ds: nueva descripcin del servicio
        PARAMETROS SALIDA:
            ninguno
    */
        procedimiento BD.LOG.PROCEDIMIENTO%TYPE;
        paramIN       BD.LOG.PARAMIN%TYPE;
        paramOUT      BD.LOG.PARAMOUT%TYPE;
        dateIN        BD.LOG.FXIN%TYPE;
        dateOUT       BD.LOG.FXOUT%TYPE;
        fechaBaja     BD.SERVICIOS.FXBAJA%TYPE;
    BEGIN
    
        -- Asignamos valores entrada
        procedimiento := $$PLSQL_UNIT || '.UPD_SERVICIO';
        paramIN       := 'idServicio = ' || id
                      || ' ,dsServicio = ' || ds;
        dateIN        := SYSDATE;
        resultado     := 'OK. El servicio se ha modificado con xito';

        -- Miramos que exista y que no est de baja
        SELECT fxBaja INTO fechaBaja 
        FROM SERVICIOS 
        WHERE idServicio=id;
        
        IF (fechaBaja IS NOT NULL) THEN
            RAISE_APPLICATION_ERROR(-20000,'');        
        END IF;
        
        --  Modidicamos los datos de servicio (dsServicio)
        UPDATE SERVICIOS 
        SET dsServicio = UPPER(ds)
        WHERE idServicio = id;
        
        -- Asignamos valores salida
        paramOUT      := 'resultado = ' || resultado;
        dateOUT       := SYSDATE;
        
        -- Grabamos en el LOG
        CONTROL.GRABA_LOG( procedimiento
                         , paramIN
                         , paramOUT 
                         , dateIN
                         , dateOUT
                         , resultado );
         
         EXCEPTION
            WHEN OTHERS THEN
                CASE (SQLCODE)
                    WHEN -1 THEN
                        -- Registro duplicado
                        resultado := 'ERROR: La descripcin de servicio ya existe.';
                    WHEN 100 THEN
                        -- Registro no encontrado
                        resultado := 'ERROR: El identificador de servicio no existe.';
                    WHEN -1407 THEN
                        -- Valor nulo
                        resultado := 'ERROR: Se ha intentado actualizar a un valor nulo ' || SUBSTR(SQLERRM,INSTR(SQLERRM,'('));
                    WHEN -20000 THEN
                        -- Servicio dado de baja
                        resultado := 'ERROR: El servicio est de baja.';
                    ELSE
                        -- Error no controlado
                        resultado := 'ERROR: ' ||  SQLERRM;
                END CASE;            

                -- Asignamos valores salida
                paramOUT      := 'resultado = ' || resultado;
                dateOUT       := SYSDATE;
                
                -- Mensaje de error
                DBMS_OUTPUT.put_line (resultado);

                -- Grabamos en el LOG
                CONTROL.GRABA_LOG( procedimiento
                                 , paramIN
                                 , paramOUT 
                                 , dateIN
                                 , dateOUT
                                 , resultado );
    END UPD_SERVICIO;


    -- RF05
    PROCEDURE GET_SERVICIO( id IN BD.SERVICIOS.IDSERVICIO%TYPE
                          , rs OUT SYS_REFCURSOR ) 
    IS
    /*
        RF05 - CASO DE USO: CONSULTAR SERVICIO
        PARAMETROS ENTRADA:
            id: identificador del servicio
                valor nulo devolver todos los registros.   
        PARAMETROS SALIDA:
            rs: cursor con el/los registro obtenido
    */
        procedimiento BD.LOG.PROCEDIMIENTO%TYPE;
        paramIN       BD.LOG.PARAMIN%TYPE;
        paramOUT      BD.LOG.PARAMOUT%TYPE;
        dateIN        BD.LOG.FXIN%TYPE;
        dateOUT       BD.LOG.FXOUT%TYPE;
        resultado     BD.LOG.RESULTADO%TYPE;
    BEGIN
    
        -- Asignamos valores entrada
        procedimiento := $$PLSQL_UNIT || '.GET_SERVICIO';
        paramIN       := 'idServicio = ' || id;
        dateIN        := SYSDATE;
        resultado     := 'OK. El(Los) servicio(s) se ha(n) consultado con xito';

        -- Realizamos la consulta en funcin del parmetro de entrada
        IF (id IS NULL) THEN
            OPEN rs FOR
            SELECT * FROM SERVICIOS;
        ELSE
            OPEN rs FOR
            SELECT * FROM SERVICIOS WHERE idServicio = id;
        END IF; 
        
        -- Asignamos valores salida
        paramOUT      := 'rs = Cursor de Servicios.';
        dateOUT       := SYSDATE;
        
        -- Grabamos en el LOG
        CONTROL.GRABA_LOG( procedimiento
                         , paramIN
                         , paramOUT 
                         , dateIN
                         , dateOUT
                         , resultado );
         
         EXCEPTION
            WHEN OTHERS THEN
                CASE (SQLCODE)
                    WHEN 100 THEN
                        -- Registro no encontrado
                        resultado := 'ERROR: Registro(s) no encontrado(s).';
                    ELSE
                        -- Error no controlado
                        resultado := 'ERROR: ' ||  SQLERRM;
                END CASE;            

                -- Generamos un cursor vacio
                OPEN rs FOR
                SELECT * FROM DUAL WHERE DUMMY IS NULL;
                
                -- Asignamos valores salida
                paramOUT      := 'rs = Cursor vaco.';
                dateOUT       := SYSDATE;
                
                -- Mensaje de error
                DBMS_OUTPUT.put_line (resultado);

                -- Grabamos en el LOG
                CONTROL.GRABA_LOG( procedimiento
                                 , paramIN
                                 , paramOUT 
                                 , dateIN
                                 , dateOUT
                                 , resultado );
    END GET_SERVICIO;


/*
    ELECTRODOMESTICOS
*/     
    -- RF06
    PROCEDURE INS_ELECTRODOMESTICO( ds IN BD.ELECTRODOMESTICOS.DSELECTRODOMESTICO%TYPE
                                  , resultado OUT BD.LOG.RESULTADO%TYPE ) 
    IS
    /*
        RF06 - CASO DE USO: CREAR ELECTRODOMESTICO
        PARAMETROS ENTRADA:
            ds: Descripcin del tipo de electrodomstico
        PARAMETROS SALIDA:
            resultado: Informacin del resultado de la operacin.
    */
        procedimiento BD.LOG.PROCEDIMIENTO%TYPE;
        paramIN       BD.LOG.PARAMIN%TYPE;
        paramOUT      BD.LOG.PARAMOUT%TYPE;
        dateIN        BD.LOG.FXIN%TYPE;
        dateOUT       BD.LOG.FXOUT%TYPE;
    BEGIN
    
        -- Asignamos valores entrada
        procedimiento := $$PLSQL_UNIT|| '.INS_ELECTRODOMESTICO';
        paramIN       := 'dsElectrodomestico = ' || ds;
        dateIN        := SYSDATE;
        resultado     := 'OK. El electrodomstico se ha creado con xito';

        INSERT INTO ELECTRODOMESTICOS(DSELECTRODOMESTICO) VALUES(UPPER(ds));
        
        -- Asignamos valores salida
        paramOUT      := 'resultado = ' || resultado;
        dateOUT       := SYSDATE;
        
        -- Grabamos en el LOG
        CONTROL.GRABA_LOG( procedimiento
                         , paramIN
                         , paramOUT 
                         , dateIN
                         , dateOUT
                         , resultado );
         
         EXCEPTION
            WHEN OTHERS THEN
                CASE (SQLCODE)
                    WHEN -1 THEN
                        -- Registro duplicado
                        resultado := 'ERROR: La descripcin de electrodomstico ya existe.';
                    WHEN -1400 THEN
                        -- Valor nulo
                        resultado := 'ERROR: Se ha intentado almacenar un valor nulo.' || SUBSTR(SQLERRM,INSTR(SQLERRM,'('));
                    ELSE
                        -- Error no controlado
                        resultado := 'ERROR: ' || SQLCODE || ' - ' || SQLERRM;
                END CASE;            

                -- Asignamos valores salida
                paramOUT      := 'resultado = ' || resultado;
                dateOUT       := SYSDATE;
                
                -- Mensaje de error
                DBMS_OUTPUT.put_line (resultado);

                -- Grabamos en el LOG
                CONTROL.GRABA_LOG( procedimiento
                                 , paramIN
                                 , paramOUT 
                                 , dateIN
                                 , dateOUT
                                 , resultado );
    END INS_ELECTRODOMESTICO;


    -- RF07
    PROCEDURE DEL_ELECTRODOMESTICO( id IN BD.ELECTRODOMESTICOS.IDELECTRODOMESTICO%TYPE 
                                  , resultado OUT BD.LOG.RESULTADO%TYPE ) 
    IS
    /*
        RF07 - CASO DE USO: BAJA ELECTRODOMESTICO
        PARAMETROS ENTRADA:
            id: Identificador del tipo de electrodomstico
        PARAMETROS SALIDA:
            resultado: Informacin del resultado de la operacin.
    */
        procedimiento BD.LOG.PROCEDIMIENTO%TYPE;
        paramIN       BD.LOG.PARAMIN%TYPE;
        paramOUT      BD.LOG.PARAMOUT%TYPE;
        dateIN        BD.LOG.FXIN%TYPE;
        dateOUT       BD.LOG.FXOUT%TYPE;
        fechaBaja     BD.SERVICIOS.FXBAJA%TYPE;
    BEGIN
    
        -- Asignamos valores entrada
        procedimiento := $$PLSQL_UNIT|| '.DEL_ELECTRODOMESTICO';
        paramIN       := 'idElectrodomestico =' || id;
        dateIN        := SYSDATE;
        resultado     := 'OK. El electrodomstico se ha dado de baja con xito';

        -- Miramos que exista y que no est de baja
        SELECT fxBaja INTO fechaBaja 
        FROM ELECTRODOMESTICOS 
        WHERE idElectrodomestico=id;
        
        IF (fechaBaja IS NOT NULL) THEN
            RAISE_APPLICATION_ERROR(-20000,'');        
        END IF;
        
        -- Damos de baja el electrodomestico con fecha del sistema
        UPDATE ELECTRODOMESTICOS 
        SET fxBaja = SYSDATE
        WHERE idElectrodomestico = id;
        
        -- Asignamos valores salida
        paramOUT      := 'resultado = ' || resultado;
        dateOUT       := SYSDATE;
        
        -- Grabamos en el LOG
        CONTROL.GRABA_LOG( procedimiento
                         , paramIN
                         , paramOUT 
                         , dateIN
                         , dateOUT
                         , resultado );
         
         EXCEPTION
            WHEN OTHERS THEN
                CASE (SQLCODE)
                    WHEN 100 THEN
                        -- Registro no encontrado
                        resultado := 'ERROR: El identificador de electrodomstico no existe.';
                    WHEN -20000 THEN
                        -- Electrodomstico dado de baja
                        resultado := 'ERROR: El electrodomstico ya est de baja.';
                    ELSE
                        -- Error no controlado
                        resultado := 'ERROR: ' ||  SQLERRM;
                END CASE;            

                -- Asignamos valores salida
                paramOUT      := 'resultado = ' || resultado;
                dateOUT       := SYSDATE;
                
                -- Mensaje de error
                DBMS_OUTPUT.put_line (resultado);

                -- Grabamos en el LOG
                CONTROL.GRABA_LOG( procedimiento
                                 , paramIN
                                 , paramOUT 
                                 , dateIN
                                 , dateOUT
                                 , resultado );
    END DEL_ELECTRODOMESTICO;
    

    -- RF08
    PROCEDURE RES_ELECTRODOMESTICO( id IN BD.ELECTRODOMESTICOS.IDELECTRODOMESTICO%TYPE 
                                  , resultado OUT BD.LOG.RESULTADO%TYPE ) 
    IS
    /*
        RF08 - CASO DE USO: RECUPERAR ELECTRODOMESTICO
        PARAMETROS ENTRADA:
            id: Identificador del tipo de electrodomstico
        PARAMETROS SALIDA:
            resultado: Informacin del resultado de la operacin.
    */
        procedimiento BD.LOG.PROCEDIMIENTO%TYPE;
        paramIN       BD.LOG.PARAMIN%TYPE;
        paramOUT      BD.LOG.PARAMOUT%TYPE;
        dateIN        BD.LOG.FXIN%TYPE;
        dateOUT       BD.LOG.FXOUT%TYPE;
        fechaBaja     BD.SERVICIOS.FXBAJA%TYPE;
    BEGIN
    
        -- Asignamos valores entrada
        procedimiento := $$PLSQL_UNIT || '.RES_ELECTRODOMESTICO';
        paramIN       := 'idElectrodomestico =' || id;
        dateIN        := SYSDATE;
        resultado     := 'OK. El electrodomstico se ha recuperado con xito';

        -- Miramos que exista y que est de baja
        SELECT fxBaja INTO fechaBaja 
        FROM ELECTRODOMESTICOS 
        WHERE idElectrodomestico=id;
        
        IF (fechaBaja IS NULL) THEN
            RAISE_APPLICATION_ERROR(-20000,'');        
        END IF;
        
        --  Recuperamos el electrodomstico con fecha del sistema
        UPDATE ELECTRODOMESTICOS 
        SET fxBaja = NULL
        WHERE idElectrodomestico = id;
        
        -- Asignamos valores salida
        paramOUT      := 'resultado = ' || resultado;
        dateOUT       := SYSDATE;
        
        -- Grabamos en el LOG
        CONTROL.GRABA_LOG( procedimiento
                         , paramIN
                         , paramOUT 
                         , dateIN
                         , dateOUT
                         , resultado );
         
         EXCEPTION
            WHEN OTHERS THEN
                CASE (SQLCODE)
                    WHEN 100 THEN
                        -- Registro no encontrado
                        resultado := 'ERROR: El identificador de electrodomstico no existe.';
                    WHEN -20000 THEN
                        -- Electrodomstico dado de baja
                        resultado := 'ERROR: El electrodomstico no est de baja.';
                    ELSE
                        -- Error no controlado
                        resultado := 'ERROR: ' ||  SQLERRM;
                END CASE;            

                -- Asignamos valores salida
                paramOUT      := 'resultado = ' || resultado;
                dateOUT       := SYSDATE;
                
                -- Mensaje de error
                DBMS_OUTPUT.put_line (resultado);

                -- Grabamos en el LOG
                CONTROL.GRABA_LOG( procedimiento
                                 , paramIN
                                 , paramOUT 
                                 , dateIN
                                 , dateOUT
                                 , resultado );
    END RES_ELECTRODOMESTICO;


    -- RF09
    PROCEDURE UPD_ELECTRODOMESTICO( id IN BD.ELECTRODOMESTICOS.IDELECTRODOMESTICO%TYPE
                                  , ds IN BD.ELECTRODOMESTICOS.DSELECTRODOMESTICO%TYPE 
                                  , resultado OUT BD.LOG.RESULTADO%TYPE ) 
    IS
    /*
        RF09 - CASO DE USO: MODIFICAR ELECTRODOMESTICO
        PARAMETROS ENTRADA:
            id: identificador del electrodomstico
            ds: nueva descripcin del electrodomstico
        PARAMETROS SALIDA:
            ninguno
    */
        procedimiento BD.LOG.PROCEDIMIENTO%TYPE;
        paramIN       BD.LOG.PARAMIN%TYPE;
        paramOUT      BD.LOG.PARAMOUT%TYPE;
        dateIN        BD.LOG.FXIN%TYPE;
        dateOUT       BD.LOG.FXOUT%TYPE;
        fechaBaja     BD.SERVICIOS.FXBAJA%TYPE;
    BEGIN
    
        -- Asignamos valores entrada
        procedimiento := $$PLSQL_UNIT || '.UPD_ELECTRODOMESTICO';
        paramIN       := 'idElectrodomestico = ' || id
                      || ' ,dsElectrodomestico = ' || ds;
        dateIN        := SYSDATE;
        resultado     := 'OK. El electrodomstico se ha modificado con xito';

        -- Miramos que exista y que no est de baja
        SELECT fxBaja INTO fechaBaja 
        FROM ELECTRODOMESTICOS 
        WHERE idElectrodomestico = id;
        
        IF (fechaBaja IS NOT NULL) THEN
            RAISE_APPLICATION_ERROR(-20000,'');        
        END IF;
        
        --  Modidicamos los datos del electrodomstico (dsElectrodomestico)
        UPDATE ELECTRODOMESTICOS 
        SET dsElectrodomestico = UPPER(ds)
        WHERE idElectrodomestico = id;
        
        -- Asignamos valores salida
        paramOUT      := 'resultado = ' || resultado;
        dateOUT       := SYSDATE;
        
        -- Grabamos en el LOG
        CONTROL.GRABA_LOG( procedimiento
                         , paramIN
                         , paramOUT 
                         , dateIN
                         , dateOUT
                         , resultado );
         
         EXCEPTION
            WHEN OTHERS THEN
                CASE (SQLCODE)
                    WHEN -1 THEN
                        -- Registro duplicado
                        resultado := 'ERROR: La descripcin de electrodomstico ya existe.';
                    WHEN 100 THEN
                        -- Registro no encontrado
                        resultado := 'ERROR: El identificador de electrodomstico no existe.';
                    WHEN -1407 THEN
                        -- Valor nulo
                        resultado := 'ERROR: Se ha intentado actualizar a un valor nulo ' || SUBSTR(SQLERRM,INSTR(SQLERRM,'('));
                    WHEN -20000 THEN
                        -- Electrodomstico dado de baja
                        resultado := 'ERROR: El electrodomstico est de baja.';
                    ELSE
                        -- Error no controlado
                        resultado := 'ERROR: ' ||  SQLERRM;
                END CASE;            

                -- Asignamos valores salida
                paramOUT      := 'resultado = ' || resultado;
                dateOUT       := SYSDATE;
                
                -- Mensaje de error
                DBMS_OUTPUT.put_line (resultado);

                -- Grabamos en el LOG
                CONTROL.GRABA_LOG( procedimiento
                                 , paramIN
                                 , paramOUT 
                                 , dateIN
                                 , dateOUT
                                 , resultado );
    END UPD_ELECTRODOMESTICO;


    -- RF10
    PROCEDURE GET_ELECTRODOMESTICO( id IN BD.ELECTRODOMESTICOS.IDELECTRODOMESTICO%TYPE
                                  , rs OUT SYS_REFCURSOR ) 
    IS
    /*
        RF10 - CASO DE USO: CONSULTAR ELECTRODOMESTICO
        PARAMETROS ENTRADA:
            id: identificador del electrodomstico
                valor nulo devolver todos los registros.   
        PARAMETROS SALIDA:
            rs: cursor con el/los registro obtenido
    */
        procedimiento BD.LOG.PROCEDIMIENTO%TYPE;
        paramIN       BD.LOG.PARAMIN%TYPE;
        paramOUT      BD.LOG.PARAMOUT%TYPE;
        dateIN        BD.LOG.FXIN%TYPE;
        dateOUT       BD.LOG.FXOUT%TYPE;
        resultado     BD.LOG.RESULTADO%TYPE;
    BEGIN
    
        -- Asignamos valores entrada
        procedimiento := $$PLSQL_UNIT || '.GET_ELECTRODOMESTICO';
        paramIN       := 'idElectrodomestico = ' || id;
        dateIN        := SYSDATE;
        resultado     := 'OK. El(Los) electrodomstico(s) se ha(n) consultado con xito';

        -- Realizamos la consulta en funcin del parmetro de entrada
        IF (id IS NULL) THEN
            OPEN rs FOR
            SELECT * FROM ELECTRODOMESTICOS;
        ELSE
            OPEN rs FOR
            SELECT * FROM ELECTRODOMESTICOS WHERE idElectrodomestico = id;
        END IF; 
        
        -- Asignamos valores salida
        paramOUT      := 'rs = Cursor de Electrodomsticos.';
        dateOUT       := SYSDATE;
        
        -- Grabamos en el LOG
        CONTROL.GRABA_LOG( procedimiento
                         , paramIN
                         , paramOUT 
                         , dateIN
                         , dateOUT
                         , resultado );
         
         EXCEPTION
            WHEN OTHERS THEN
                CASE (SQLCODE)
                    WHEN 100 THEN
                        -- Registro no encontrado
                        resultado := 'ERROR: Registro(s) no encontrado(s).';
                    ELSE
                        -- Error no controlado
                        resultado := 'ERROR: ' ||  SQLERRM;
                END CASE;            

                -- Generamos un cursor vacio
                OPEN rs FOR
                SELECT * FROM DUAL WHERE DUMMY IS NULL;
                
                -- Asignamos valores salida
                paramOUT      := 'rs = Cursor vaco.';
                dateOUT       := SYSDATE;
                
                -- Mensaje de error
                DBMS_OUTPUT.put_line (resultado);

                -- Grabamos en el LOG
                CONTROL.GRABA_LOG( procedimiento
                                 , paramIN
                                 , paramOUT 
                                 , dateIN
                                 , dateOUT
                                 , resultado );
    END GET_ELECTRODOMESTICO;


/*
    PROVINCIAS
*/     
    -- RF11
    PROCEDURE INS_PROVINCIA( cod IN BD.PROVINCIAS.COPROVINCIA%TYPE
                           , prov IN BD.PROVINCIAS.DSPROVINCIA%TYPE
                           , resultado OUT BD.LOG.RESULTADO%TYPE )
    IS
    /*
        RF11 - CASO DE USO: CREAR PROVINCIA
        PARAMETROS ENTRADA:
            cod: Cdigo de provincia
            prov: Nombre de la provincia
        PARAMETROS SALIDA:
            resultado: Informacin del resultado de la operacin.
    */
        procedimiento BD.LOG.PROCEDIMIENTO%TYPE;
        paramIN       BD.LOG.PARAMIN%TYPE;
        paramOUT      BD.LOG.PARAMOUT%TYPE;
        dateIN        BD.LOG.FXIN%TYPE;
        dateOUT       BD.LOG.FXOUT%TYPE;
    BEGIN
    
        -- Asignamos valores entrada
        procedimiento := $$PLSQL_UNIT|| '.INS_PROVINCIA';
        paramIN       := 'coProvincia = ' || cod 
                      || ' ,dsProvincia = ' || prov; 
        dateIN        := SYSDATE;
        resultado     := 'OK. La provincia se ha creado con xito';

        INSERT INTO PROVINCIAS(COPROVINCIA, DSPROVINCIA) VALUES(cod, UPPER(prov));
        
        -- Asignamos valores salida
        paramOUT      := 'resultado = ' || resultado;
        dateOUT       := SYSDATE;
        
        -- Grabamos en el LOG
        CONTROL.GRABA_LOG( procedimiento
                         , paramIN
                         , paramOUT 
                         , dateIN
                         , dateOUT
                         , resultado );
         
         EXCEPTION
            WHEN OTHERS THEN
                CASE (SQLCODE)
                    WHEN -1 THEN
                        -- Registro duplicado
                        -- Miramos si el error es en clave primaria o clave nica
                        IF INSTR(SQLERRM,'PK')>0 THEN
                            resultado := 'ERROR: El cdigo de provincia ya existe.';
                        ELSE 
                            resultado := 'ERROR: El nombre de provincia ya existe.';
                        END IF;                        
                    WHEN -1400 THEN
                        -- Valor nulo
                        resultado := 'ERROR: Se ha intentado almacenar un valor nulo.' || SUBSTR(SQLERRM,INSTR(SQLERRM,'('));
                    ELSE
                        -- Error no controlado
                        resultado := 'ERROR: ' || SQLCODE || ' - ' || SQLERRM;
                END CASE;            

                -- Asignamos valores salida
                paramOUT      := 'resultado = ' || resultado;
                dateOUT       := SYSDATE;
                
                -- Mensaje de error
                DBMS_OUTPUT.put_line (resultado);

                -- Grabamos en el LOG
                CONTROL.GRABA_LOG( procedimiento
                                 , paramIN
                                 , paramOUT 
                                 , dateIN
                                 , dateOUT
                                 , resultado );
    END INS_PROVINCIA;


    -- RF12
    PROCEDURE DEL_PROVINCIA( cod IN BD.PROVINCIAS.COPROVINCIA%TYPE 
                           , resultado OUT BD.LOG.RESULTADO%TYPE ) 
    IS
    /*
        RF12 - CASO DE USO: BORRAR PROVINCIA
        PARAMETROS ENTRADA:
            cod: Cdigo de provincia
        PARAMETROS SALIDA:
            resultado: Informacin del resultado de la operacin.
    */
        procedimiento BD.LOG.PROCEDIMIENTO%TYPE;
        paramIN       BD.LOG.PARAMIN%TYPE;
        paramOUT      BD.LOG.PARAMOUT%TYPE;
        dateIN        BD.LOG.FXIN%TYPE;
        dateOUT       BD.LOG.FXOUT%TYPE;
        codigo        BD.PROVINCIAS.COPROVINCIA%TYPE;
    BEGIN
    
        -- Asignamos valores entrada
        procedimiento := $$PLSQL_UNIT|| '.DEL_PROVINCIA';
        paramIN       := 'coProvincia =' || cod;
        dateIN        := SYSDATE;
        resultado     := 'OK. La provincia se ha dado de baja con xito';

        -- Comprobamos si existe la provincia
        SELECT coProvincia INTO codigo FROM PROVINCIAS 
        WHERE coProvincia = cod;
        
        -- Borramos la provincia
        DELETE FROM PROVINCIAS
        WHERE coProvincia = cod;
        
        -- Asignamos valores salida
        paramOUT      := 'resultado = ' || resultado;
        dateOUT       := SYSDATE;
        
        -- Grabamos en el LOG
        CONTROL.GRABA_LOG( procedimiento
                         , paramIN
                         , paramOUT 
                         , dateIN
                         , dateOUT
                         , resultado );
         
         EXCEPTION
            WHEN OTHERS THEN
                CASE (SQLCODE)
                    WHEN 100 THEN
                        -- Registro no encontrado
                        resultado := 'ERROR: El cdigo de provincia no existe.';
                    WHEN -2292 THEN
                        -- Existen poblaciones asociadas
                        resultado := 'ERROR: La provincia tiene poblaciones asignadas.';
                    ELSE
                        -- Error no controlado
                        resultado := 'ERROR: ' ||  SQLERRM;
                END CASE;            

                -- Asignamos valores salida
                paramOUT      := 'resultado = ' || resultado;
                dateOUT       := SYSDATE;
                
                -- Mensaje de error
                DBMS_OUTPUT.put_line (resultado);

                -- Grabamos en el LOG
                CONTROL.GRABA_LOG( procedimiento
                                 , paramIN
                                 , paramOUT 
                                 , dateIN
                                 , dateOUT
                                 , resultado );
    END DEL_PROVINCIA;


    -- RF13
    PROCEDURE UPD_PROVINCIA( cod IN BD.PROVINCIAS.COPROVINCIA%TYPE
                           , prov IN BD.PROVINCIAS.DSPROVINCIA%TYPE
                           , resultado OUT BD.LOG.RESULTADO%TYPE ) 
    IS
    /*
        RF13 - CASO DE USO: MODIFICAR PROVINCIA
        PARAMETROS ENTRADA:
            cod cdigo de provincia
            ds: nuevo nombre de provincia
        PARAMETROS SALIDA:
            ninguno
    */
        procedimiento BD.LOG.PROCEDIMIENTO%TYPE;
        paramIN       BD.LOG.PARAMIN%TYPE;
        paramOUT      BD.LOG.PARAMOUT%TYPE;
        dateIN        BD.LOG.FXIN%TYPE;
        dateOUT       BD.LOG.FXOUT%TYPE;
        codigo        BD.PROVINCIAS.COPROVINCIA%TYPE;
    BEGIN
    
        -- Asignamos valores entrada
        procedimiento := $$PLSQL_UNIT || '.UPD_PROVINCIA';
        paramIN       := 'coProvincia = ' || cod
                      || ' ,dsProvincia = ' || prov;
        dateIN        := SYSDATE;
        resultado     := 'OK. La provincia se ha modificado con xito';

        
        -- Comprobamos si existe la provincia
        SELECT coProvincia INTO codigo FROM PROVINCIAS 
        WHERE coProvincia = cod;
        
        --  Modidicamos los datos de la provincia (dsProvincia)
        UPDATE PROVINCIAS 
        SET dsProvincia = UPPER(prov)
        WHERE coProvincia = cod;
        
        -- Asignamos valores salida
        paramOUT      := 'resultado = ' || resultado;
        dateOUT       := SYSDATE;
        
        -- Grabamos en el LOG
        CONTROL.GRABA_LOG( procedimiento
                         , paramIN
                         , paramOUT 
                         , dateIN
                         , dateOUT
                         , resultado );
         
         EXCEPTION
            WHEN OTHERS THEN
                CASE (SQLCODE)
                    WHEN -1 THEN
                        -- Registro duplicado
                        resultado := 'ERROR: El nombre de la provincia ya existe.';
                    WHEN 100 THEN
                        -- Registro no encontrado
                        resultado := 'ERROR: El cdigo de provincia no existe.';
                    WHEN -1407 THEN
                        -- Valor nulo
                        resultado := 'ERROR: Se ha intentado actualizar a un valor nulo ' || SUBSTR(SQLERRM,INSTR(SQLERRM,'('));
                    ELSE
                        -- Error no controlado
                        resultado := 'ERROR: ' ||  SQLERRM;
                END CASE;            

                -- Asignamos valores salida
                paramOUT      := 'resultado = ' || resultado;
                dateOUT       := SYSDATE;
                
                -- Mensaje de error
                DBMS_OUTPUT.put_line (resultado);

                -- Grabamos en el LOG
                CONTROL.GRABA_LOG( procedimiento
                                 , paramIN
                                 , paramOUT 
                                 , dateIN
                                 , dateOUT
                                 , resultado );
    END UPD_PROVINCIA;


    -- RF14
    PROCEDURE GET_PROVINCIA( cod IN BD.PROVINCIAS.COPROVINCIA%TYPE
                           , rs OUT SYS_REFCURSOR)
    IS
    /*
        RF14 - CASO DE USO: CONSULTAR PROVINCIA
        PARAMETROS ENTRADA:
            cod cdigo de provincia
                valor nulo devolver todos los registros.   
        PARAMETROS SALIDA:
            rs: cursor con el/los registro obtenido
    */
        procedimiento BD.LOG.PROCEDIMIENTO%TYPE;
        paramIN       BD.LOG.PARAMIN%TYPE;
        paramOUT      BD.LOG.PARAMOUT%TYPE;
        dateIN        BD.LOG.FXIN%TYPE;
        dateOUT       BD.LOG.FXOUT%TYPE;
        resultado     BD.LOG.RESULTADO%TYPE;
    BEGIN
    
        -- Asignamos valores entrada
        procedimiento := $$PLSQL_UNIT || '.GET_PROVINCIA';
        paramIN       := 'coProvincia = ' || cod;
        dateIN        := SYSDATE;
        resultado     := 'OK. La(s) provincia(s) se ha(n) consultado con xito';

        -- Realizamos la consulta en funcin del parmetro de entrada
        IF (cod IS NULL) THEN
            OPEN rs FOR
            SELECT * FROM PROVINCIAS;
        ELSE
            OPEN rs FOR
            SELECT * FROM PROVINCIAS WHERE coProvincia = cod;
        END IF; 
        
        -- Asignamos valores salida
        paramOUT      := 'rs = Cursor de Provincias.';
        dateOUT       := SYSDATE;
        
        -- Grabamos en el LOG
        CONTROL.GRABA_LOG( procedimiento
                         , paramIN
                         , paramOUT 
                         , dateIN
                         , dateOUT
                         , resultado );
         
         EXCEPTION
            WHEN OTHERS THEN
                CASE (SQLCODE)
                    WHEN 100 THEN
                        -- Registro no encontrado
                        resultado := 'ERROR: Registro(s) no encontrado(s).';
                    ELSE
                        -- Error no controlado
                        resultado := 'ERROR: ' ||  SQLERRM;
                END CASE;            

                -- Generamos un cursor vacio
                OPEN rs FOR
                SELECT * FROM DUAL WHERE DUMMY IS NULL;
                
                -- Asignamos valores salida
                paramOUT      := 'rs = Cursor vaco.';
                dateOUT       := SYSDATE;
                
                -- Mensaje de error
                DBMS_OUTPUT.put_line (resultado);

                -- Grabamos en el LOG
                CONTROL.GRABA_LOG( procedimiento
                                 , paramIN
                                 , paramOUT 
                                 , dateIN
                                 , dateOUT
                                 , resultado );
    END GET_PROVINCIA;


/*
    POBLACIONES
*/     
    -- RF15
    PROCEDURE INS_POBLACION( cPostal IN BD.POBLACIONES.CODPOSTAL%TYPE
                           , poblacion IN BD.POBLACIONES.DSPOBLACION%TYPE
                           , codProv IN BD.POBLACIONES.COPROVINCIA%TYPE
                           , resultado OUT BD.LOG.RESULTADO%TYPE )
    IS
    /*
        RF15 - CASO DE USO: CREAR POBLACION
        PARAMETROS ENTRADA:
            cPostal: Cdigo postal de la poblacion
            poblacion: Nombre de la poblacion
            codProv: Cdigo de la provincia a la que pertenece
        PARAMETROS SALIDA:
            resultado: Informacin del resultado de la operacin.
    */
        procedimiento BD.LOG.PROCEDIMIENTO%TYPE;
        paramIN       BD.LOG.PARAMIN%TYPE;
        paramOUT      BD.LOG.PARAMOUT%TYPE;
        dateIN        BD.LOG.FXIN%TYPE;
        dateOUT       BD.LOG.FXOUT%TYPE;
    BEGIN
    
        -- Asignamos valores entrada
        procedimiento := $$PLSQL_UNIT|| '.INS_POBLACION';
        paramIN       := 'codPostal = ' || cPostal 
                      || ' ,dsPoblacion = ' || poblacion 
                      || ' ,coProvincia = ' || codProv; 
        dateIN        := SYSDATE;
        resultado     := 'OK. La poblacin se ha creado con xito';

        INSERT INTO POBLACIONES(CODPOSTAL, DSPOBLACION, COPROVINCIA) VALUES(cPostal, UPPER(poblacion), codProv);
        
        -- Asignamos valores salida
        paramOUT      := 'resultado = ' || resultado;
        dateOUT       := SYSDATE;
        
        -- Grabamos en el LOG
        CONTROL.GRABA_LOG( procedimiento
                         , paramIN
                         , paramOUT 
                         , dateIN
                         , dateOUT
                         , resultado );
         
         EXCEPTION
            WHEN OTHERS THEN
                CASE (SQLCODE)
                    WHEN -1 THEN
                        -- Registro duplicado
                        IF INSTR(SQLERRM,'PK')>0 THEN
                            resultado := 'ERROR: El cdigo postal ya existe.';
                        ELSE 
                            resultado := 'ERROR: El nombre de la poblacin ya existe.';
                        END IF;                        
                    WHEN -1400 THEN
                        -- Valor nulo
                        resultado := 'ERROR: Se ha intentado almacenar un valor nulo.' || SUBSTR(SQLERRM,INSTR(SQLERRM,'('));
                    WHEN -2291 THEN
                        -- Restriccin de integridad
                        resultado := 'ERROR: El cdigo de provincia no existe.';
                    ELSE
                        -- Error no controlado
                        resultado := 'ERROR: ' || SQLCODE || ' - ' || SQLERRM;
                END CASE;            

                -- Asignamos valores salida
                paramOUT      := 'resultado = ' || resultado;
                dateOUT       := SYSDATE;
                
                -- Mensaje de error
                DBMS_OUTPUT.put_line (resultado);

                -- Grabamos en el LOG
                CONTROL.GRABA_LOG( procedimiento
                                 , paramIN
                                 , paramOUT 
                                 , dateIN
                                 , dateOUT
                                 , resultado );
    END INS_POBLACION;


    -- RF16
    PROCEDURE DEL_POBLACION( cPostal IN BD.POBLACIONES.CODPOSTAL%TYPE
                           , resultado OUT BD.LOG.RESULTADO%TYPE )
    IS
    /*
        RF16 - CASO DE USO: BORRAR POBLACION
        PARAMETROS ENTRADA:
            cPostal: Cdigo postal de la poblacin
        PARAMETROS SALIDA:
            resultado: Informacin del resultado de la operacin.
    */
        procedimiento BD.LOG.PROCEDIMIENTO%TYPE;
        paramIN       BD.LOG.PARAMIN%TYPE;
        paramOUT      BD.LOG.PARAMOUT%TYPE;
        dateIN        BD.LOG.FXIN%TYPE;
        dateOUT       BD.LOG.FXOUT%TYPE;
        codigo        BD.POBLACIONES.CODPOSTAL%TYPE;
    BEGIN
    
        -- Asignamos valores entrada
        procedimiento := $$PLSQL_UNIT|| '.DEL_POBLACION';
        paramIN       := 'codPostal =' || cPostal;
        dateIN        := SYSDATE;
        resultado     := 'OK. La poblacin se ha dado de baja con xito';

        -- Comprobamos si existe la provincia
        SELECT codPostal INTO codigo FROM POBLACIONES 
        WHERE codPostal = cPostal;
        
        -- Borramos la poblacion
        DELETE FROM POBLACIONES
        WHERE codPostal = cPostal;
        
        -- Asignamos valores salida
        paramOUT      := 'resultado = ' || resultado;
        dateOUT       := SYSDATE;
        
        -- Grabamos en el LOG
        CONTROL.GRABA_LOG( procedimiento
                         , paramIN
                         , paramOUT 
                         , dateIN
                         , dateOUT
                         , resultado );
         
         EXCEPTION
            WHEN OTHERS THEN
                CASE (SQLCODE)
                    WHEN 100 THEN
                        -- Registro no encontrado
                        resultado := 'ERROR: El cdigo postal no existe.';
                    WHEN -2292 THEN
                        -- Existen poblaciones asociadas
                        resultado := 'ERROR: La poblacin est asignada a alguna zona.';
                    ELSE
                        -- Error no controlado
                        resultado := 'ERROR: ' ||  SQLERRM;
                END CASE;            

                -- Asignamos valores salida
                paramOUT      := 'resultado = ' || resultado;
                dateOUT       := SYSDATE;
                
                -- Mensaje de error
                DBMS_OUTPUT.put_line (resultado);

                -- Grabamos en el LOG
                CONTROL.GRABA_LOG( procedimiento
                                 , paramIN
                                 , paramOUT 
                                 , dateIN
                                 , dateOUT
                                 , resultado );
    END DEL_POBLACION;


    -- RF17
    PROCEDURE UPD_POBLACION( cPostal IN BD.POBLACIONES.CODPOSTAL%TYPE
                           , poblacion IN BD.POBLACIONES.DSPOBLACION%TYPE
                           , codProv IN BD.POBLACIONES.COPROVINCIA%TYPE
                           , resultado OUT BD.LOG.RESULTADO%TYPE )
    IS
    /*
        RF17 - CASO DE USO: MODIFICAR POBLACION
        PARAMETROS ENTRADA:
            cPostal: Cdigo postal de la poblacion
            poblacion: Nombre de la poblacion
            codProv: Cdigo de la provincia a la que pertenece
        PARAMETROS SALIDA:
            resultado: Informacin del resultado de la operacin.
    */
        procedimiento BD.LOG.PROCEDIMIENTO%TYPE;
        paramIN       BD.LOG.PARAMIN%TYPE;
        paramOUT      BD.LOG.PARAMOUT%TYPE;
        dateIN        BD.LOG.FXIN%TYPE;
        dateOUT       BD.LOG.FXOUT%TYPE;
        codigo        BD.POBLACIONES.CODPOSTAL%TYPE;
    BEGIN
    
        -- Asignamos valores entrada
        procedimiento := $$PLSQL_UNIT || '.UPD_POBLACION';
        paramIN       := 'codPostal = ' || cPostal 
                      || ' ,dsPoblacion = ' || poblacion 
                      || ' ,coProvincia = ' || codProv; 
        dateIN        := SYSDATE;
        resultado     := 'OK. La poblacin se ha modificado con xito';

        
        -- Comprobamos si existe la poblacin
        SELECT codPostal INTO codigo FROM POBLACIONES 
        WHERE codPostal = cPostal;
        
        --  Modidicamos los datos de la poblacin (dsPoblacion, coProvincia)
        UPDATE POBLACIONES 
        SET dsPoblacion = UPPER(poblacion)
          , coProvincia = codProv
        WHERE codPostal = cPostal;
        
        -- Asignamos valores salida
        paramOUT      := 'resultado = ' || resultado;
        dateOUT       := SYSDATE;
        
        -- Grabamos en el LOG
        CONTROL.GRABA_LOG( procedimiento
                         , paramIN
                         , paramOUT 
                         , dateIN
                         , dateOUT
                         , resultado );
         
         EXCEPTION
            WHEN OTHERS THEN
                CASE (SQLCODE)
                    WHEN -1 THEN
                        -- Registro duplicado
                        resultado := 'ERROR: El nombre de la poblacin ya existe.';
                    WHEN 100 THEN
                        -- Registro no encontrado
                        resultado := 'ERROR: El cdigo postal no existe.';
                    WHEN -1407 THEN
                        -- Valor nulo
                        resultado := 'ERROR: Se ha intentado actualizar a un valor nulo ' || SUBSTR(SQLERRM,INSTR(SQLERRM,'('));
                    WHEN -2291 THEN
                        -- Restriccin de integridad
                        resultado := 'ERROR: El cdigo de provincia no existe.';
                    ELSE
                        -- Error no controlado
                        resultado := 'ERROR: ' ||  SQLERRM;
                END CASE;            

                -- Asignamos valores salida
                paramOUT      := 'resultado = ' || resultado;
                dateOUT       := SYSDATE;
                
                -- Mensaje de error
                DBMS_OUTPUT.put_line (resultado);

                -- Grabamos en el LOG
                CONTROL.GRABA_LOG( procedimiento
                                 , paramIN
                                 , paramOUT 
                                 , dateIN
                                 , dateOUT
                                 , resultado );
    END UPD_POBLACION;


    -- RF18
    PROCEDURE GET_POBLACION( cPostal IN BD.POBLACIONES.CODPOSTAL%TYPE
                           , rs OUT SYS_REFCURSOR)
    IS
    /*
        RF18 - CASO DE USO: CONSULTAR POBLACION
        PARAMETROS ENTRADA:
            cPostal cdigo postal de la poblacin
                    valor nulo devolver todos los registros.   
        PARAMETROS SALIDA:
            rs: cursor con el/los registro obtenido
    */
        procedimiento BD.LOG.PROCEDIMIENTO%TYPE;
        paramIN       BD.LOG.PARAMIN%TYPE;
        paramOUT      BD.LOG.PARAMOUT%TYPE;
        dateIN        BD.LOG.FXIN%TYPE;
        dateOUT       BD.LOG.FXOUT%TYPE;
        resultado     BD.LOG.RESULTADO%TYPE;
    BEGIN
    
        -- Asignamos valores entrada
        procedimiento := $$PLSQL_UNIT || '.GET_POBLACION';
        paramIN       := 'codPostal = ' || cPostal;
        dateIN        := SYSDATE;
        resultado     := 'OK. La(s) poblacin(es) se ha(n) consultado con xito';

        -- Realizamos la consulta en funcin del parmetro de entrada
        IF (cPostal IS NULL) THEN
            OPEN rs FOR
            SELECT codPostal
                 , dsPoblacion
                 , PROV.coProvincia
                 , dsProvincia
            FROM POBLACIONES POB INNER JOIN PROVINCIAS PROV ON POB.coProvincia = PROV.coProvincia;
        ELSE
            OPEN rs FOR
            SELECT codPostal
                 , dsPoblacion
                 , PROV.coProvincia
                 , dsProvincia
            FROM POBLACIONES POB INNER JOIN PROVINCIAS PROV ON POB.coProvincia = PROV.coProvincia
            WHERE codPostal = cPostal;
        END IF; 
        
        -- Asignamos valores salida
        paramOUT      := 'rs = Cursor de Poblaciones.';
        dateOUT       := SYSDATE;
        
        -- Grabamos en el LOG
        CONTROL.GRABA_LOG( procedimiento
                         , paramIN
                         , paramOUT 
                         , dateIN
                         , dateOUT
                         , resultado );
         
         EXCEPTION
            WHEN OTHERS THEN
                CASE (SQLCODE)
                    WHEN 100 THEN
                        -- Registro no encontrado
                        resultado := 'ERROR: Registro(s) no encontrado(s).';
                    ELSE
                        -- Error no controlado
                        resultado := 'ERROR: ' ||  SQLERRM;
                END CASE;            

                -- Generamos un cursor vacio
                OPEN rs FOR
                SELECT * FROM DUAL WHERE DUMMY IS NULL;
                
                -- Asignamos valores salida
                paramOUT      := 'rs = Cursor vaco.';
                dateOUT       := SYSDATE;
                
                -- Mensaje de error
                DBMS_OUTPUT.put_line (resultado);

                -- Grabamos en el LOG
                CONTROL.GRABA_LOG( procedimiento
                                 , paramIN
                                 , paramOUT 
                                 , dateIN
                                 , dateOUT
                                 , resultado );
    END GET_POBLACION;


/*
    ZONAS
*/     
    -- RF19
    PROCEDURE INS_ZONA( ds IN BD.ZONAS.DSZONA%TYPE
                      , resultado OUT BD.LOG.RESULTADO%TYPE ) 
    IS
    /*
        RF19 - CASO DE USO: CREAR ZONA
        PARAMETROS ENTRADA:
            ds: Descripcin de la zona
        PARAMETROS SALIDA:
            resultado: Informacin del resultado de la operacin.
    */
        procedimiento BD.LOG.PROCEDIMIENTO%TYPE;
        paramIN       BD.LOG.PARAMIN%TYPE;
        paramOUT      BD.LOG.PARAMOUT%TYPE;
        dateIN        BD.LOG.FXIN%TYPE;
        dateOUT       BD.LOG.FXOUT%TYPE;
    BEGIN
    
        -- Asignamos valores entrada
        procedimiento := $$PLSQL_UNIT|| '.INS_ZONA';
        paramIN       := 'dsZona = ' || ds;
        dateIN        := SYSDATE;
        resultado     := 'OK. La zona se ha creado con xito';

        -- Insertamos la zona
        INSERT INTO ZONAS(DSZONA) VALUES(UPPER(ds));
        
        -- Asignamos valores salida
        paramOUT      := 'resultado = ' || resultado;
        dateOUT       := SYSDATE;
        
        -- Grabamos en el LOG
        CONTROL.GRABA_LOG( procedimiento
                         , paramIN
                         , paramOUT 
                         , dateIN
                         , dateOUT
                         , resultado );
         
         EXCEPTION
            WHEN OTHERS THEN
                CASE (SQLCODE)
                    WHEN -1 THEN
                        -- Registro duplicado
                        resultado := 'ERROR: La descripcin de la zona ya existe.';
                    WHEN -1400 THEN
                        -- Valor nulo
                        resultado := 'ERROR: Se ha intentado almacenar un valor nulo.' || SUBSTR(SQLERRM,INSTR(SQLERRM,'('));
                    ELSE
                        -- Error no controlado
                        resultado := 'ERROR: ' || SQLCODE || ' - ' || SQLERRM;
                END CASE;            

                -- Asignamos valores salida
                paramOUT      := 'resultado = ' || resultado;
                dateOUT       := SYSDATE;
                
                -- Mensaje de error
                DBMS_OUTPUT.put_line (resultado);

                -- Grabamos en el LOG
                CONTROL.GRABA_LOG( procedimiento
                                 , paramIN
                                 , paramOUT 
                                 , dateIN
                                 , dateOUT
                                 , resultado );
    END INS_ZONA;


    -- RF20
    PROCEDURE DEL_ZONA( id IN BD.ZONAS.IDZONA%TYPE 
                      , resultado OUT BD.LOG.RESULTADO%TYPE ) 
    IS
    /*
        RF20 - CASO DE USO: BORRAR ZONA
        PARAMETROS ENTRADA:
            id: Identificador de la zona
        PARAMETROS SALIDA:
            resultado: Informacin del resultado de la operacin.
    */
        procedimiento BD.LOG.PROCEDIMIENTO%TYPE;
        paramIN       BD.LOG.PARAMIN%TYPE;
        paramOUT      BD.LOG.PARAMOUT%TYPE;
        dateIN        BD.LOG.FXIN%TYPE;
        dateOUT       BD.LOG.FXOUT%TYPE;
        identificador BD.ZONAS.IDZONA%TYPE;
    BEGIN
    
        -- Asignamos valores entrada
        procedimiento := $$PLSQL_UNIT|| '.DEL_ZONA';
        paramIN       := 'idZona =' || id;
        dateIN        := SYSDATE;
        resultado     := 'OK. La zona se ha dado de baja con xito';

        -- Comprobamos si existe la zona
        SELECT idZona INTO identificador FROM ZONAS 
        WHERE idZona = id;
        
        -- Damos de baja la zona
        DELETE FROM ZONAS 
        WHERE idZona = id;
        
        -- Asignamos valores salida
        paramOUT      := 'resultado = ' || resultado;
        dateOUT       := SYSDATE;
        
        -- Grabamos en el LOG
        CONTROL.GRABA_LOG( procedimiento
                         , paramIN
                         , paramOUT 
                         , dateIN
                         , dateOUT
                         , resultado );
         
         EXCEPTION
            WHEN OTHERS THEN
                CASE (SQLCODE)
                    WHEN 100 THEN
                        -- Registro no encontrado
                        resultado := 'ERROR: El identificador de zona no existe.';
                    WHEN -2292 THEN
                        -- Existen poblaciones asociadas
                        resultado := 'ERROR: La zona tiene poblaciones asignadas.';
                    ELSE
                        -- Error no controlado
                        resultado := 'ERROR: ' ||  SQLERRM;
                END CASE;            

                -- Asignamos valores salida
                paramOUT      := 'resultado = ' || resultado;
                dateOUT       := SYSDATE;
                
                -- Mensaje de error
                DBMS_OUTPUT.put_line (resultado);

                -- Grabamos en el LOG
                CONTROL.GRABA_LOG( procedimiento
                                 , paramIN
                                 , paramOUT 
                                 , dateIN
                                 , dateOUT
                                 , resultado );
    END DEL_ZONA;


    -- RF21
    PROCEDURE UPD_ZONA( id IN BD.ZONAS.IDZONA%TYPE
                      , ds IN BD.ZONAS.DSZONA%TYPE 
                      , resultado OUT BD.LOG.RESULTADO%TYPE ) 
    IS
    /*
        RF21 - CASO DE USO: MODIFICAR ZONA
        PARAMETROS ENTRADA:
            id: identificador de la zona
            ds: nueva descripcin de la zona
        PARAMETROS SALIDA:
            ninguno
    */
        procedimiento BD.LOG.PROCEDIMIENTO%TYPE;
        paramIN       BD.LOG.PARAMIN%TYPE;
        paramOUT      BD.LOG.PARAMOUT%TYPE;
        dateIN        BD.LOG.FXIN%TYPE;
        dateOUT       BD.LOG.FXOUT%TYPE;
        identificador BD.ZONAS.IDZONA%TYPE;
    BEGIN
    
        -- Asignamos valores entrada
        procedimiento := $$PLSQL_UNIT || '.UPD_ZONA';
        paramIN       := 'idZona = ' || id
                      || ' ,dsZona = ' || ds;
        dateIN        := SYSDATE;
        resultado     := 'OK. La zona se ha modificado con xito';

        -- Comprobamos si existe la zona
        SELECT idZona INTO identificador FROM ZONAS 
        WHERE idZona = id;
        
        --  Modidicamos los datos de la zona (dsZona)
        UPDATE ZONAS 
        SET dsZona = UPPER(ds)
        WHERE idZona = id;
        
        -- Asignamos valores salida
        paramOUT      := 'resultado = ' || resultado;
        dateOUT       := SYSDATE;
        
        -- Grabamos en el LOG
        CONTROL.GRABA_LOG( procedimiento
                         , paramIN
                         , paramOUT 
                         , dateIN
                         , dateOUT
                         , resultado );
         
         EXCEPTION
            WHEN OTHERS THEN
                CASE (SQLCODE)
                    WHEN -1 THEN
                        -- Registro duplicado
                        resultado := 'ERROR: La descripcin de zona ya existe.';
                    WHEN 100 THEN
                        -- Registro no encontrado
                        resultado := 'ERROR: El identificador de zona no existe.';
                    WHEN -1407 THEN
                        -- Valor nulo
                        resultado := 'ERROR: Se ha intentado actualizar a un valor nulo ' || SUBSTR(SQLERRM,INSTR(SQLERRM,'('));
                    ELSE
                        -- Error no controlado
                        resultado := 'ERROR: ' ||  SQLERRM;
                END CASE;            

                -- Asignamos valores salida
                paramOUT      := 'resultado = ' || resultado;
                dateOUT       := SYSDATE;
                
                -- Mensaje de error
                DBMS_OUTPUT.put_line (resultado);

                -- Grabamos en el LOG
                CONTROL.GRABA_LOG( procedimiento
                                 , paramIN
                                 , paramOUT 
                                 , dateIN
                                 , dateOUT
                                 , resultado );
    END UPD_ZONA;


    -- RF22
    PROCEDURE GET_ZONA( id IN BD.ZONAS.IDZONA%TYPE
                      , rs OUT SYS_REFCURSOR ) 
    IS
    /*
        RF22 - CASO DE USO: CONSULTAR ZONA
        PARAMETROS ENTRADA:
            id: identificador de la zona
                valor nulo devolver todos los registros.   
        PARAMETROS SALIDA:
            rs: cursor con el/los registro obtenido
    */
        procedimiento BD.LOG.PROCEDIMIENTO%TYPE;
        paramIN       BD.LOG.PARAMIN%TYPE;
        paramOUT      BD.LOG.PARAMOUT%TYPE;
        dateIN        BD.LOG.FXIN%TYPE;
        dateOUT       BD.LOG.FXOUT%TYPE;
        resultado     BD.LOG.RESULTADO%TYPE;
    BEGIN
    
        -- Asignamos valores entrada
        procedimiento := $$PLSQL_UNIT || '.GET_ZONA';
        paramIN       := 'idZona = ' || id;
        dateIN        := SYSDATE;
        resultado     := 'OK. La(s) zona(s) se ha(n) consultado con xito';

        -- Realizamos la consulta en funcin del parmetro de entrada
        IF (id IS NULL) THEN
            OPEN rs FOR
            SELECT * FROM ZONAS;
        ELSE
            OPEN rs FOR
            SELECT * FROM ZONAS WHERE idZona = id;
        END IF; 
        
        -- Asignamos valores salida
        paramOUT      := 'rs = Cursor de Zonas.';
        dateOUT       := SYSDATE;
        
        -- Grabamos en el LOG
        CONTROL.GRABA_LOG( procedimiento
                         , paramIN
                         , paramOUT 
                         , dateIN
                         , dateOUT
                         , resultado );
         
         EXCEPTION
            WHEN OTHERS THEN
                CASE (SQLCODE)
                    WHEN 100 THEN
                        -- Registro no encontrado
                        resultado := 'ERROR: Registro(s) no encontrado(s).';
                    ELSE
                        -- Error no controlado
                        resultado := 'ERROR: ' ||  SQLERRM;
                END CASE;            

                -- Generamos un cursor vacio
                OPEN rs FOR
                SELECT * FROM DUAL WHERE DUMMY IS NULL;
                
                -- Asignamos valores salida
                paramOUT      := 'rs = Cursor vaco.';
                dateOUT       := SYSDATE;
                
                -- Mensaje de error
                DBMS_OUTPUT.put_line (resultado);

                -- Grabamos en el LOG
                CONTROL.GRABA_LOG( procedimiento
                                 , paramIN
                                 , paramOUT 
                                 , dateIN
                                 , dateOUT
                                 , resultado );
    END GET_ZONA;


    -- RF23
    PROCEDURE ASG_PROVINCIAZONA( coProv IN BD.PROVINCIAS.COPROVINCIA%TYPE
                               , zona IN BD.ZONAS.IDZONA%TYPE
                               , resultado OUT BD.LOG.RESULTADO%TYPE )
    IS
    /*
        RF23 - CASO DE USO: ASIGNAR PROVINCA A ZONA
        PARAMETROS ENTRADA:
            coProv: cdigo de la provincia
            zona: identificador de la zona
        PARAMETROS SALIDA:
            ninguno
    */
        procedimiento BD.LOG.PROCEDIMIENTO%TYPE;
        paramIN       BD.LOG.PARAMIN%TYPE;
        paramOUT      BD.LOG.PARAMOUT%TYPE;
        dateIN        BD.LOG.FXIN%TYPE;
        dateOUT       BD.LOG.FXOUT%TYPE;
        codigo        BD.PROVINCIAS.COPROVINCIA%TYPE;
        izona         BD.ZONAS.idZona%TYPE;
    BEGIN
    
        -- Asignamos valores entrada
        procedimiento := $$PLSQL_UNIT || '.ASG_PROVINCIAZONA';
        paramIN       := 'coProvincia = ' || coProv
                      || ' ,idZona = ' || zona;
        dateIN        := SYSDATE;
        resultado     := 'OK. La asignacin se ha realizado con xito';

        -- Comprobamos si existe la provincia
        BEGIN
            SELECT coProvincia INTO codigo FROM PROVINCIAS 
            WHERE coProvincia = coProv;
        EXCEPTION
            WHEN OTHERS THEN
                IF (SQLCODE = 100) THEN
                    -- Registro No encontrado
                    RAISE_APPLICATION_ERROR(-20000,'');
                END IF; 
        END;
        
        -- Comprobamos si existe la zona
        BEGIN
            SELECT idZona INTO iZona FROM ZONAS 
            WHERE idZona = zona;
        EXCEPTION
            WHEN OTHERS THEN
                IF (SQLCODE = 100) THEN
                    -- Registro No encontrado
                    RAISE_APPLICATION_ERROR(-20001,'');
                END IF; 
        END;

        --  Asignamos las poblaciones asignadas a la provincia a la zona
        INSERT INTO ZONASPOBLACIONES(idZona, codPostal)
        (SELECT zona, codPostal
         FROM POBLACIONES 
         WHERE coProvincia = coProv
         AND codPostal NOT IN (SELECT codPostal FROM ZONASPOBLACIONES WHERE idZona = zona)); 
        
        -- Asignamos valores salida
        paramOUT      := 'resultado = ' || resultado;
        dateOUT       := SYSDATE;
        
        -- Grabamos en el LOG
        CONTROL.GRABA_LOG( procedimiento
                         , paramIN
                         , paramOUT 
                         , dateIN
                         , dateOUT
                         , resultado );
         
         EXCEPTION
            WHEN OTHERS THEN
                CASE (SQLCODE)
                    WHEN -20000 THEN
                        -- Provincia no existe
                        resultado := 'ERROR: El cdigo de provincia no existe.';
                    WHEN -20001 THEN
                        -- Zona no existe
                        resultado := 'ERROR: El identificador de zona no existe.';
                    ELSE
                        -- Error no controlado
                        resultado := 'ERROR: ' ||  SQLERRM;
                END CASE;            

                -- Asignamos valores salida
                paramOUT      := 'resultado = ' || resultado;
                dateOUT       := SYSDATE;
                
                -- Mensaje de error
                DBMS_OUTPUT.put_line (resultado);

                -- Grabamos en el LOG
                CONTROL.GRABA_LOG( procedimiento
                                 , paramIN
                                 , paramOUT 
                                 , dateIN
                                 , dateOUT
                                 , resultado );
    END ASG_PROVINCIAZONA;

   
 -- RF24
    PROCEDURE ASG_POBLACIONZONA( cp IN BD.POBLACIONES.CODPOSTAL%TYPE
                               , zona IN BD.ZONAS.IDZONA%TYPE
                               , resultado OUT BD.LOG.RESULTADO%TYPE )
    IS
    /*
        RF24 - CASO DE USO: ASIGNAR POBLACION A ZONA
        PARAMETROS ENTRADA:
            cp: cdigo postal de la poblacin
            zona: identificador de la zona
        PARAMETROS SALIDA:
            ninguno
    */
        procedimiento BD.LOG.PROCEDIMIENTO%TYPE;
        paramIN       BD.LOG.PARAMIN%TYPE;
        paramOUT      BD.LOG.PARAMOUT%TYPE;
        dateIN        BD.LOG.FXIN%TYPE;
        dateOUT       BD.LOG.FXOUT%TYPE;
        codigo        BD.POBLACIONES.CODPOSTAL%TYPE;
        izona         BD.ZONAS.idZona%TYPE;
    BEGIN
    
        -- Asignamos valores entrada
        procedimiento := $$PLSQL_UNIT || '.ASG_POBLACIONZONA';
        paramIN       := 'codPostal = ' || cp
                      || ' ,idZona = ' || zona;
        dateIN        := SYSDATE;
        resultado     := 'OK. La asignacin se ha realizado con xito';

        -- Comprobamos si existe la poblacin
        BEGIN
            SELECT codPostal INTO codigo FROM POBLACIONES 
            WHERE codPostal = cp;
        EXCEPTION
            WHEN OTHERS THEN
                IF (SQLCODE = 100) THEN
                    -- Registro No encontrado
                    RAISE_APPLICATION_ERROR(-20000,'');
                END IF; 
        END;
        
        -- Comprobamos si existe la zona
        BEGIN
            SELECT idZona INTO iZona FROM ZONAS 
            WHERE idZona = zona;
        EXCEPTION
            WHEN OTHERS THEN
                IF (SQLCODE = 100) THEN
                    -- Registro No encontrado
                    RAISE_APPLICATION_ERROR(-20001,'');
                END IF; 
        END;
        
                
        --  Asignamos la poblacin a la zona
        INSERT INTO ZONASPOBLACIONES(idZona, codPostal)
        VALUES (zona, cp);
        
        -- Asignamos valores salida
        paramOUT      := 'resultado = ' || resultado;
        dateOUT       := SYSDATE;
        
        -- Grabamos en el LOG
        CONTROL.GRABA_LOG( procedimiento
                         , paramIN
                         , paramOUT 
                         , dateIN
                         , dateOUT
                         , resultado );
         
         EXCEPTION
            WHEN OTHERS THEN
                CASE (SQLCODE)
                    WHEN -1 THEN
                        -- Registro duplicado
                        resultado := 'ERROR: La poblacin ya est asociada a la zona.';
                    WHEN -20000 THEN
                        -- Poblacin no existe
                        resultado := 'ERROR: El cdigo postal no existe.';
                    WHEN -20001 THEN
                        -- Zona no existe
                        resultado := 'ERROR: El identificador de zona no existe.';
                    ELSE
                        -- Error no controlado
                        resultado := 'ERROR: ' ||  SQLERRM;
                END CASE;            

                -- Asignamos valores salida
                paramOUT      := 'resultado = ' || resultado;
                dateOUT       := SYSDATE;
                
                -- Mensaje de error
                DBMS_OUTPUT.put_line (resultado);

                -- Grabamos en el LOG
                CONTROL.GRABA_LOG( procedimiento
                                 , paramIN
                                 , paramOUT 
                                 , dateIN
                                 , dateOUT
                                 , resultado );
    END ASG_POBLACIONZONA;


    -- RF25
    PROCEDURE DSG_PROVINCIAZONA( coProv IN BD.PROVINCIAS.COPROVINCIA%TYPE
                               , zona IN BD.ZONAS.IDZONA%TYPE
                               , resultado OUT BD.LOG.RESULTADO%TYPE )
    IS
    /*
        RF25 - CASO DE USO: DESASIGNAR PROVINCA DE ZONA
        PARAMETROS ENTRADA:
            coProv: cdigo de la provincia
            zona: identificador de la zona
        PARAMETROS SALIDA:
            ninguno
    */
        procedimiento BD.LOG.PROCEDIMIENTO%TYPE;
        paramIN       BD.LOG.PARAMIN%TYPE;
        paramOUT      BD.LOG.PARAMOUT%TYPE;
        dateIN        BD.LOG.FXIN%TYPE;
        dateOUT       BD.LOG.FXOUT%TYPE;
        codigo        BD.PROVINCIAS.COPROVINCIA%TYPE;
        iZona         BD.ZONAS.IDZONA%TYPE;

    BEGIN
    
        -- Asignamos valores entrada
        procedimiento := $$PLSQL_UNIT || '.DSG_PROVINCIAZONA';
        paramIN       := 'coProvincia = ' || coProv
                      || ' ,idZona = ' || zona;
        dateIN        := SYSDATE;
        resultado     := 'OK. La desasignacin se ha realizado con xito';

        -- Comprobamos si existe la provincia
        BEGIN
            SELECT coProvincia INTO codigo FROM PROVINCIAS 
            WHERE coProvincia = coProv;
        EXCEPTION
            WHEN OTHERS THEN
                IF (SQLCODE = 100) THEN
                    -- Registro No encontrado
                    RAISE_APPLICATION_ERROR(-20000,'');
                END IF; 
        END;
        
        -- Comprobamos si existe la zona
        BEGIN
            SELECT idZona INTO iZona FROM ZONAS 
            WHERE idZona = zona;
        EXCEPTION
            WHEN OTHERS THEN
                IF (SQLCODE = 100) THEN
                    -- Registro No encontrado
                    RAISE_APPLICATION_ERROR(-20001,'');
                END IF; 
        END;

        --  Desasignamos las poblaciones asignadas a la provincia de la zona
        DELETE FROM ZONASPOBLACIONES
        WHERE idZona = zona
        AND codPostal IN (SELECT codPostal FROM POBLACIONES WHERE coProvincia = coProv);
        
        -- Asignamos valores salida
        paramOUT      := 'resultado = ' || resultado;
        dateOUT       := SYSDATE;
        
        -- Grabamos en el LOG
        CONTROL.GRABA_LOG( procedimiento
                         , paramIN
                         , paramOUT 
                         , dateIN
                         , dateOUT
                         , resultado );
         
         EXCEPTION
            WHEN OTHERS THEN
                CASE (SQLCODE)
                    WHEN -20000 THEN
                        -- Provincia no existe
                        resultado := 'ERROR: El cdigo de provincia no existe.';
                    WHEN -20001 THEN
                        -- Zona no existe
                        resultado := 'ERROR: El identificador de zona no existe.';
                    ELSE
                        -- Error no controlado
                        resultado := 'ERROR: ' ||  SQLERRM;
                END CASE;            

                -- Asignamos valores salida
                paramOUT      := 'resultado = ' || resultado;
                dateOUT       := SYSDATE;
                
                -- Mensaje de error
                DBMS_OUTPUT.put_line (resultado);

                -- Grabamos en el LOG
                CONTROL.GRABA_LOG( procedimiento
                                 , paramIN
                                 , paramOUT 
                                 , dateIN
                                 , dateOUT
                                 , resultado );
    END DSG_PROVINCIAZONA;

   
 -- RF26
    PROCEDURE DSG_POBLACIONZONA( cp IN BD.POBLACIONES.CODPOSTAL%TYPE
                               , zona IN BD.ZONAS.IDZONA%TYPE
                               , resultado OUT BD.LOG.RESULTADO%TYPE )
    IS
    /*
        RF26 - CASO DE USO: DESASIGNAR POBLACION DE ZONA
        PARAMETROS ENTRADA:
            cp: cdigo postal de la poblacin
            zona: identificador de la zona
        PARAMETROS SALIDA:
            ninguno
    */
        procedimiento BD.LOG.PROCEDIMIENTO%TYPE;
        paramIN       BD.LOG.PARAMIN%TYPE;
        paramOUT      BD.LOG.PARAMOUT%TYPE;
        dateIN        BD.LOG.FXIN%TYPE;
        dateOUT       BD.LOG.FXOUT%TYPE;
        codigo        BD.POBLACIONES.CODPOSTAL%TYPE;
        iZona         BD.ZONAS.IDZONA%TYPE;
    BEGIN
    
        -- Asignamos valores entrada
        procedimiento := $$PLSQL_UNIT || '.DSG_POBLACIONZONA';
        paramIN       := 'codPostal = ' || cp
                      || ' ,idZona = ' || zona;
        dateIN        := SYSDATE;
        resultado     := 'OK. La desasignacin se ha realizado con xito';

        -- Comprobamos si existe la poblacin
        BEGIN
            SELECT codPostal INTO codigo FROM POBLACIONES 
            WHERE codPostal = cp;
        EXCEPTION
            WHEN OTHERS THEN
                IF (SQLCODE = 100) THEN
                    -- Registro No encontrado
                    RAISE_APPLICATION_ERROR(-20000,'');
                END IF; 
        END;
        
        -- Comprobamos si existe la zona
        BEGIN
            SELECT idZona INTO iZona FROM ZONAS 
            WHERE idZona = zona;
        EXCEPTION
            WHEN OTHERS THEN
                IF (SQLCODE = 100) THEN
                    -- Registro No encontrado
                    RAISE_APPLICATION_ERROR(-20001,'');
                END IF; 
        END;

        --  Desasignamos la poblacin de la zona
        DELETE FROM ZONASPOBLACIONES 
        WHERE idZona = zona 
        AND codPostal = cp;
        
        -- Asignamos valores salida
        paramOUT      := 'resultado = ' || resultado;
        dateOUT       := SYSDATE;
        
        -- Grabamos en el LOG
        CONTROL.GRABA_LOG( procedimiento
                         , paramIN
                         , paramOUT 
                         , dateIN
                         , dateOUT
                         , resultado );
         
         EXCEPTION
            WHEN OTHERS THEN
                CASE (SQLCODE)
                    WHEN -20000 THEN
                        -- Poblacin no existe
                        resultado := 'ERROR: El cdigo postal no existe.';
                    WHEN -20001 THEN
                        -- Zona no existe
                        resultado := 'ERROR: El identificador de zona no existe.';
                    ELSE
                        -- Error no controlado
                        resultado := 'ERROR: ' ||  SQLERRM;
                END CASE;            

                -- Asignamos valores salida
                paramOUT      := 'resultado = ' || resultado;
                dateOUT       := SYSDATE;
                
                -- Mensaje de error
                DBMS_OUTPUT.put_line (resultado);

                -- Grabamos en el LOG
                CONTROL.GRABA_LOG( procedimiento
                                 , paramIN
                                 , paramOUT 
                                 , dateIN
                                 , dateOUT
                                 , resultado );
    END DSG_POBLACIONZONA;


    -- RF27
    PROCEDURE GET_POBLACIONESZONA( id IN BD.ZONAS.IDZONA%TYPE
                                 , rs OUT SYS_REFCURSOR ) 
    IS
    /*
        RF27 - CASO DE USO: CONSULTAR POBLACIONES DE ZONA
        PARAMETROS ENTRADA:
            id: identificador de la zona
                valor nulo devolver todos los registros.   
        PARAMETROS SALIDA:
            rs: cursor con el/los registro obtenido
    */
        procedimiento BD.LOG.PROCEDIMIENTO%TYPE;
        paramIN       BD.LOG.PARAMIN%TYPE;
        paramOUT      BD.LOG.PARAMOUT%TYPE;
        dateIN        BD.LOG.FXIN%TYPE;
        dateOUT       BD.LOG.FXOUT%TYPE;
        resultado     BD.LOG.RESULTADO%TYPE;
    BEGIN
    
        -- Asignamos valores entrada
        procedimiento := $$PLSQL_UNIT || '.GET_POBLACIONESZONA';
        paramIN       := 'idZona = ' || id;
        dateIN        := SYSDATE;
        resultado     := 'OK. Las poblaciones de la(s) zona(s) se ha(n) consultado con xito';

        -- Realizamos la consulta en funcin del parmetro de entrada
        IF (id IS NULL) THEN
            OPEN rs FOR
            SELECT Z.idZona
                 , Z.dsZona
                 , PB.codPostal
                 , PB.dsPoblacion
                 , PR.coProvincia
                 , PR.dsProvincia
            FROM ZONASPOBLACIONES ZP 
                 INNER JOIN ZONAS Z ON ZP.idZona = Z.idZona
                 INNER JOIN POBLACIONES PB ON ZP.codPostal = PB.codPostal
                 INNER JOIN PROVINCIAS PR ON PB.coProvincia = PR.coProvincia;
        ELSE
            OPEN rs FOR
            SELECT Z.idZona
                 , Z.dsZona
                 , PB.codPostal
                 , PB.dsPoblacion
                 , PR.coProvincia
                 , PR.dsProvincia
            FROM ZONASPOBLACIONES ZP 
                 INNER JOIN ZONAS Z ON ZP.idZona = Z.idZona
                 INNER JOIN POBLACIONES PB ON ZP.codPostal = PB.codPostal
                 INNER JOIN PROVINCIAS PR ON PB.coProvincia = PR.coProvincia
            WHERE ZP.idZona = id;
        END IF; 
        
        -- Asignamos valores salida
        paramOUT      := 'rs = Cursor de Poblaciones.';
        dateOUT       := SYSDATE;
        
        -- Grabamos en el LOG
        CONTROL.GRABA_LOG( procedimiento
                         , paramIN
                         , paramOUT 
                         , dateIN
                         , dateOUT
                         , resultado );
         
         EXCEPTION
            WHEN OTHERS THEN
                CASE (SQLCODE)
                    WHEN 100 THEN
                        -- Registro no encontrado
                        resultado := 'ERROR: Registro(s) no encontrado(s).';
                    ELSE
                        -- Error no controlado
                        resultado := 'ERROR: ' ||  SQLERRM;
                END CASE;            

                -- Generamos un cursor vacio
                OPEN rs FOR
                SELECT * FROM DUAL WHERE DUMMY IS NULL;
                
                -- Asignamos valores salida
                paramOUT      := 'rs = Cursor vaco.';
                dateOUT       := SYSDATE;
                
                -- Mensaje de error
                DBMS_OUTPUT.put_line (resultado);

                -- Grabamos en el LOG
                CONTROL.GRABA_LOG( procedimiento
                                 , paramIN
                                 , paramOUT 
                                 , dateIN
                                 , dateOUT
                                 , resultado );
    END GET_POBLACIONESZONA;


END CONFIGURACION;

