/*
    PACKAGE PETICION
    Fecha: 28/04/2011
    Autor: Francisco Amador Risquez Teva
    Proyecto: gPV
*/



/*
    ESPECIFICACIN DEL PAQUETE
    SUBSISTEMA PETICION
*/
CREATE OR REPLACE PACKAGE BD.PETICION AS


    /*
        CLIENTES
    */     
    --RF47
    PROCEDURE INS_CLIENTE( pNombre IN BD.CLIENTES.NOMBRE%TYPE
                         , pApellidos IN BD.CLIENTES.APELLIDOS%TYPE
                         , pNif IN BD.CLIENTES.NIF%TYPE
                         , pDireccion IN BD.CLIENTES.DIRECCION%TYPE
                         , pCodPostal IN BD.CLIENTES.CODPOSTAL%TYPE
                         , pTelFijo IN BD.CLIENTES.TELFIJO%TYPE
                         , pTelMovil IN BD.CLIENTES.TELMOVIL%TYPE
                         , pEMail IN BD.CLIENTES.EMAIL%TYPE
                         , resultado OUT BD.LOG.RESULTADO%TYPE );
    
    --RF48
    PROCEDURE UPD_CLIENTE( pId IN BD.CLIENTES.IDCLIENTE%TYPE
                         , pNombre IN BD.CLIENTES.NOMBRE%TYPE
                         , pApellidos IN BD.CLIENTES.APELLIDOS%TYPE
                         , pNif IN BD.CLIENTES.NIF%TYPE
                         , pDireccion IN BD.CLIENTES.DIRECCION%TYPE
                         , pCodPostal IN BD.CLIENTES.CODPOSTAL%TYPE
                         , pTelFijo IN BD.CLIENTES.TELFIJO%TYPE
                         , pTelMovil IN BD.CLIENTES.TELMOVIL%TYPE
                         , pEMail IN BD.CLIENTES.EMAIL%TYPE
                         , resultado OUT BD.LOG.RESULTADO%TYPE ); 
    
    --RF49
    PROCEDURE GET_CLIENTE( pId IN BD.CLIENTES.IDCLIENTE%TYPE
                         , rs OUT SYS_REFCURSOR);


    /*
        PRODUCTOS
    */     
    --RF50
    PROCEDURE INS_PRODUCTO( pIdElectrodomestico IN BD.PRODUCTOS.IDELECTRODOMESTICO%TYPE
                          , pMarca IN BD.PRODUCTOS.MARCA%TYPE
                          , pModelo IN BD.PRODUCTOS.MODELO%TYPE
                          , pNumSerie IN BD.PRODUCTOS.NUMSERIE%TYPE
                          , pFxCompra IN BD.PRODUCTOS.FXCOMPRA%TYPE
                          , pDireccion IN BD.PRODUCTOS.DIRECCION%TYPE
                          , pCodPostal IN BD.PRODUCTOS.CODPOSTAL%TYPE
                          , pIdCliente IN BD.PRODUCTOS.IDCLIENTE%TYPE
                          , resultado OUT BD.LOG.RESULTADO%TYPE );
    
    --RF51
    PROCEDURE UPD_PRODUCTO( pId IN BD.PRODUCTOS.IDPRODUCTO%TYPE
                          , pIdElectrodomestico IN BD.PRODUCTOS.IDELECTRODOMESTICO%TYPE
                          , pMarca IN BD.PRODUCTOS.MARCA%TYPE
                          , pModelo IN BD.PRODUCTOS.MODELO%TYPE
                          , pNumSerie IN BD.PRODUCTOS.NUMSERIE%TYPE
                          , pFxCompra IN BD.PRODUCTOS.FXCOMPRA%TYPE
                          , pDireccion IN BD.PRODUCTOS.DIRECCION%TYPE
                          , pCodPostal IN BD.PRODUCTOS.CODPOSTAL%TYPE
                          , pIdCliente IN BD.PRODUCTOS.IDCLIENTE%TYPE
                          , resultado OUT BD.LOG.RESULTADO%TYPE ); 
    
    --RF52
    PROCEDURE GET_PRODUCTO( pId IN BD.PRODUCTOS.IDPRODUCTO%TYPE
                          , rs OUT SYS_REFCURSOR);


    /*
        PETICION
    */     
    --RF53
    PROCEDURE INS_PETICION( pIdProducto IN BD.PETICIONES.IDPRODUCTO%TYPE
                          , pIdServicio IN BD.PETICIONES.IDSERVICIO%TYPE
                          , pIdTeleoperador IN BD.PETICIONES.IDTELEOPERADOR%TYPE
                          , resultado OUT BD.LOG.RESULTADO%TYPE );

    --RF54
    PROCEDURE CANCEL_PETICION( pId IN BD.PETICIONES.IDPETICION%TYPE
                             , resultado OUT BD.LOG.RESULTADO%TYPE );

    --RF55
    PROCEDURE GET_PETICION( pId IN BD.PETICIONES.IDPETICION%TYPE
                          , rs OUT SYS_REFCURSOR);



END PETICION;



/*
    CUERPO DEL PAQUETE
    SUBSISTEMA PETICION
*/
CREATE OR REPLACE PACKAGE BODY BD.PETICION AS

/*
    CLIENTES
*/     
    --RF47
    PROCEDURE INS_CLIENTE( pNombre IN BD.CLIENTES.NOMBRE%TYPE
                         , pApellidos IN BD.CLIENTES.APELLIDOS%TYPE
                         , pNif IN BD.CLIENTES.NIF%TYPE
                         , pDireccion IN BD.CLIENTES.DIRECCION%TYPE
                         , pCodPostal IN BD.CLIENTES.CODPOSTAL%TYPE
                         , pTelFijo IN BD.CLIENTES.TELFIJO%TYPE
                         , pTelMovil IN BD.CLIENTES.TELMOVIL%TYPE
                         , pEMail IN BD.CLIENTES.EMAIL%TYPE
                         , resultado OUT BD.LOG.RESULTADO%TYPE )
    IS
    /*
        RF47 - CASO DE USO: CREAR CLIENTE
        PARAMETROS ENTRADA:
            pNombre:    Nombre del Cliente
            pApellidos: Apellidos del Cliente
            pNif:       NIF del Cliente
            pDireccion: Direccin del Cliente
            pCodPostal: Cdigo Postal de la poblacin del Cliente
            pTelFijo:   Telfono Fijo del Cliente
            pTelMovil:  Telfono Mvil del Cliente
            pEMail:     EMail del Cliente
        PARAMETROS SALIDA:
            resultado: Informacin del resultado de la operacin.
    */
        procedimiento BD.LOG.PROCEDIMIENTO%TYPE;
        paramIN       BD.LOG.PARAMIN%TYPE;
        paramOUT      BD.LOG.PARAMOUT%TYPE;
        dateIN        BD.LOG.FXIN%TYPE;
        dateOUT       BD.LOG.FXOUT%TYPE;
    BEGIN
    
        -- Asignamos valores entrada
        procedimiento := $$PLSQL_UNIT|| '.INS_CLIENTE';
        paramIN       := 'pNombre = ' || pNombre
                      || ' ,pApellidos = ' || pApellidos
                      || ' ,pNif = ' || pNif
                      || ' ,pDireccion = ' || pDireccion
                      || ' ,pCodPostal = ' || pCodPostal
                      || ' ,pTelFijo = ' || pTelFijo
                      || ' ,pTelMovil = ' || pTelMovil
                      || ' ,pEMail = ' || pEMail; 
        dateIN        := SYSDATE;
        resultado     := 'OK. El cliente se ha creado con xito';

        -- Insertamos el Cliente
        INSERT INTO CLIENTES(nombre, apellidos, nif, direccion, codpostal, telfijo, telmovil, email) 
        VALUES(UPPER(pNombre), UPPER(pApellidos), UPPER(pNif), UPPER(pDireccion), pCodPostal, pTelFijo, pTelMovil, pEmail);
        
        -- Asignamos valores salida
        paramOUT      := 'resultado = ' || resultado;
        dateOUT       := SYSDATE;
        
        -- Grabamos en el LOG
        CONTROL.GRABA_LOG( procedimiento
                         , paramIN
                         , paramOUT 
                         , dateIN
                         , dateOUT
                         , resultado );
         
         EXCEPTION
            WHEN OTHERS THEN
                CASE (SQLCODE)
                    WHEN -1 THEN
                        -- Registro duplicado
                        resultado := 'ERROR: El nif del cliente ya existe.';
                    WHEN -1400 THEN
                        -- Valor nulo
                        resultado := 'ERROR: Se ha intentado almacenar un valor nulo.' || SUBSTR(SQLERRM,INSTR(SQLERRM,'('));
                    WHEN -2291 THEN
                        -- Restriccin de integridad
                        resultado := 'ERROR: El cdigo postal no existe';
                    ELSE
                        -- Error no controlado
                        resultado := 'ERROR: ' || SQLCODE || ' - ' || SQLERRM;
                END CASE;            

                -- Asignamos valores salida
                paramOUT      := 'resultado = ' || resultado;
                dateOUT       := SYSDATE;
                
                -- Mensaje de error
                DBMS_OUTPUT.put_line (resultado);

                -- Grabamos en el LOG
                CONTROL.GRABA_LOG( procedimiento
                                 , paramIN
                                 , paramOUT 
                                 , dateIN
                                 , dateOUT
                                 , resultado );
    END INS_CLIENTE;


    -- RF48
    PROCEDURE UPD_CLIENTE( pId IN BD.CLIENTES.IDCLIENTE%TYPE
                         , pNombre IN BD.CLIENTES.NOMBRE%TYPE
                         , pApellidos IN BD.CLIENTES.APELLIDOS%TYPE
                         , pNif IN BD.CLIENTES.NIF%TYPE
                         , pDireccion IN BD.CLIENTES.DIRECCION%TYPE
                         , pCodPostal IN BD.CLIENTES.CODPOSTAL%TYPE
                         , pTelFijo IN BD.CLIENTES.TELFIJO%TYPE
                         , pTelMovil IN BD.CLIENTES.TELMOVIL%TYPE
                         , pEMail IN BD.CLIENTES.EMAIL%TYPE
                         , resultado OUT BD.LOG.RESULTADO%TYPE )
    IS
    /*
        RF48 - CASO DE USO: MODIFICAR CLIENTE
        PARAMETROS ENTRADA:
            pId:        identificador del Cliente
            pNombre:    Nombre del Cliente
            pApellidos: Apellidos del Cliente
            pNif:       NIF del Cliente
            pDireccion: Direccin del Cliente
            pCodPostal: Cdigo Postal de la poblacin del Cliente
            pTelFijo:   Telfono Fijo del Cliente
            pTelMovil:  Telfono Mvil del Cliente
            pEMail:     EMail del Cliente
        PARAMETROS SALIDA:
            resultado: Informacin del resultado de la operacin.
    */
        procedimiento BD.LOG.PROCEDIMIENTO%TYPE;
        paramIN       BD.LOG.PARAMIN%TYPE;
        paramOUT      BD.LOG.PARAMOUT%TYPE;
        dateIN        BD.LOG.FXIN%TYPE;
        dateOUT       BD.LOG.FXOUT%TYPE;
        fechaAlta     BD.CLIENTES.FXALTA%TYPE;
    BEGIN
    
        -- Asignamos valores entrada
        procedimiento := $$PLSQL_UNIT || '.UPD_CLIENTE';
        paramIN       := 'pId = ' || pId
                      || ' ,pNombre = ' || pNombre
                      || ' ,pApellidos = ' || pApellidos
                      || ' ,pNif = ' || pNif
                      || ' ,pDireccion = ' || pDireccion
                      || ' ,pCodPostal = ' || pCodPostal
                      || ' ,pTelFijo = ' || pTelFijo
                      || ' ,pTelMovil = ' || pTelMovil
                      || ' ,pEMail = ' || pEMail; 
        dateIN        := SYSDATE;
        resultado     := 'OK. El cliente se ha modificado con xito';

        -- Miramos que exista 
        SELECT fxAlta INTO fechaAlta 
        FROM CLIENTES
        WHERE idCliente = pId;
        
        --  Modidicamos los datos del Cliente (nombre, apellidos, nif, direccion, codpostal, telfijo, telmovil, email)
        UPDATE CLIENTES
        SET nombre = UPPER(pNombre)
          , apellidos = UPPER(pApellidos)
          , nif = UPPER(pNif)
          , direccion = UPPER(pDireccion)
          , codpostal = pCodPostal
          , telFijo = pTelFijo
          , telMovil = pTelMovil
          , eMail = pEMail
        WHERE idCliente = pId;
        
        -- Asignamos valores salida
        paramOUT      := 'resultado = ' || resultado;
        dateOUT       := SYSDATE;
        
        -- Grabamos en el LOG
        CONTROL.GRABA_LOG( procedimiento
                         , paramIN
                         , paramOUT 
                         , dateIN
                         , dateOUT
                         , resultado );
         
         EXCEPTION
            WHEN OTHERS THEN
                CASE (SQLCODE)
                    WHEN -1 THEN
                        -- Registro duplicado
                        resultado := 'ERROR: El nif del cliente ya existe.';
                    WHEN 100 THEN
                        -- Registro no encontrado
                        resultado := 'ERROR: El identificador del cliente no existe.';
                    WHEN -1407 THEN
                        -- Valor nulo
                        resultado := 'ERROR: Se ha intentado actualizar a un valor nulo ' || SUBSTR(SQLERRM,INSTR(SQLERRM,'('));
                    WHEN -2291 THEN
                        -- Restriccin de integridad
                        resultado := 'ERROR: El cdigo postal no existe';
                    ELSE
                        -- Error no controlado
                        resultado := 'ERROR: ' ||  SQLERRM;
                END CASE;            

                -- Asignamos valores salida
                paramOUT      := 'resultado = ' || resultado;
                dateOUT       := SYSDATE;
                
                -- Mensaje de error
                DBMS_OUTPUT.put_line (resultado);

                -- Grabamos en el LOG
                CONTROL.GRABA_LOG( procedimiento
                                 , paramIN
                                 , paramOUT 
                                 , dateIN
                                 , dateOUT
                                 , resultado );
    END UPD_CLIENTE;


    -- RF49
    PROCEDURE GET_CLIENTE( pId IN BD.CLIENTES.IDCLIENTE%TYPE
                         , rs OUT SYS_REFCURSOR)
    IS
    /*
        RF49 - CASO DE USO: CONSULTAR CLIENTE
        PARAMETROS ENTRADA:
            pId: identificador del cliente
                valor nulo devolver todos los registros.   
        PARAMETROS SALIDA:
            rs: cursor con el/los registro obtenido
    */
        procedimiento BD.LOG.PROCEDIMIENTO%TYPE;
        paramIN       BD.LOG.PARAMIN%TYPE;
        paramOUT      BD.LOG.PARAMOUT%TYPE;
        dateIN        BD.LOG.FXIN%TYPE;
        dateOUT       BD.LOG.FXOUT%TYPE;
        resultado     BD.LOG.RESULTADO%TYPE;
    BEGIN
    
        -- Asignamos valores entrada
        procedimiento := $$PLSQL_UNIT || '.GET_CLIENTE';
        paramIN       := 'pId = ' || pId;
        dateIN        := SYSDATE;
        resultado     := 'OK. El(Los) cliente(s) se ha(n) consultado con xito';

        -- Realizamos la consulta en funcin del parmetro de entrada
        IF (pId IS NULL) THEN
            OPEN rs FOR
            SELECT * FROM CLIENTES;
        ELSE
            OPEN rs FOR
            SELECT * FROM CLIENTES WHERE idCliente = pId;
        END IF; 
        
        -- Asignamos valores salida
        paramOUT      := 'rs = Cursor de Clientes.';
        dateOUT       := SYSDATE;
        
        -- Grabamos en el LOG
        CONTROL.GRABA_LOG( procedimiento
                         , paramIN
                         , paramOUT 
                         , dateIN
                         , dateOUT
                         , resultado );
         
         EXCEPTION
            WHEN OTHERS THEN
                CASE (SQLCODE)
                    WHEN 100 THEN
                        -- Registro no encontrado
                        resultado := 'ERROR: Registro(s) no encontrado(s).';
                    ELSE
                        -- Error no controlado
                        resultado := 'ERROR: ' ||  SQLERRM;
                END CASE;            

                -- Generamos un cursor vacio
                OPEN rs FOR
                SELECT * FROM DUAL WHERE DUMMY IS NULL;
                
                -- Asignamos valores salida
                paramOUT      := 'rs = Cursor vaco.';
                dateOUT       := SYSDATE;
                
                -- Mensaje de error
                DBMS_OUTPUT.put_line (resultado);

                -- Grabamos en el LOG
                CONTROL.GRABA_LOG( procedimiento
                                 , paramIN
                                 , paramOUT 
                                 , dateIN
                                 , dateOUT
                                 , resultado );
    END GET_CLIENTE;


/*
    PRODUCTOS
*/     
    --RF50
    PROCEDURE INS_PRODUCTO( pIdElectrodomestico IN BD.PRODUCTOS.IDELECTRODOMESTICO%TYPE
                          , pMarca IN BD.PRODUCTOS.MARCA%TYPE
                          , pModelo IN BD.PRODUCTOS.MODELO%TYPE
                          , pNumSerie IN BD.PRODUCTOS.NUMSERIE%TYPE
                          , pFxCompra IN BD.PRODUCTOS.FXCOMPRA%TYPE
                          , pDireccion IN BD.PRODUCTOS.DIRECCION%TYPE
                          , pCodPostal IN BD.PRODUCTOS.CODPOSTAL%TYPE
                          , pIdCliente IN BD.PRODUCTOS.IDCLIENTE%TYPE
                          , resultado OUT BD.LOG.RESULTADO%TYPE )
    IS
    /*
        RF50 - CASO DE USO: CREAR PRODUCTO
        PARAMETROS ENTRADA:
            pIdElectrodomestico: Identidicador del tipo de electrodomstico del producto
            pMarca: Marca del producto
            pModelo: Modelo del producto
            pNumSerie: Nmero de serie del producto
            pFxCompra: Fecha de compra del producto
            pDireccin: Direccin en la que se encuentra el producto
            pCodPostal: Cdigo postal de la poblacin en la que se encuentra el producto
            pIdCliente: Identificador del propietario del producto
        PARAMETROS SALIDA:
            resultado: Informacin del resultado de la operacin.
    */
        procedimiento BD.LOG.PROCEDIMIENTO%TYPE;
        paramIN       BD.LOG.PARAMIN%TYPE;
        paramOUT      BD.LOG.PARAMOUT%TYPE;
        dateIN        BD.LOG.FXIN%TYPE;
        dateOUT       BD.LOG.FXOUT%TYPE;
        fechaBaja     BD.ELECTRODOMESTICOS.FXBAJA%TYPE;
    BEGIN
    
        -- Asignamos valores entrada
        procedimiento := $$PLSQL_UNIT|| '.INS_PRODUCTO';
        paramIN       := 'pIdElectrodomestico= ' || pIdElectrodomestico
                      || ' ,pMarca = ' || pMarca
                      || ' ,pModelo = ' || pModelo
                      || ' ,pNumSerie = ' || pNumSerie
                      || ' ,pFxCompra = ' || pFxCompra
                      || ' ,pDireccion = ' || pDireccion
                      || ' ,pCodPostal = ' || pCodPostal
                      || ' ,pIdCliente = ' || pIdCliente; 
        dateIN        := SYSDATE;
        resultado     := 'OK. El producto se ha creado con xito';

        -- Miramos si el electrodomstico exite y est de alta
        SELECT fxBaja INTO fechaBaja
        FROM ELECTRODOMESTICOS
        WHERE idElectrodomestico = pIdElectrodomestico;
        
        IF (fechaBaja IS NOT NULL) THEN
            RAISE_APPLICATION_ERROR(-20000,'');        
        END IF;

        -- Insertamos el Producto
        INSERT INTO PRODUCTOS(idElectrodomestico, marca, modelo, numSerie, fxCompra, direccion, codPostal, idCliente) 
        VALUES(pIdElectrodomestico, UPPER(pMarca), UPPER(pModelo), UPPER(pNumSerie), pFxCompra, UPPER(pDireccion), pCodPostal, pIdCliente);
        
        -- Asignamos valores salida
        paramOUT      := 'resultado = ' || resultado;
        dateOUT       := SYSDATE;
        
        -- Grabamos en el LOG
        CONTROL.GRABA_LOG( procedimiento
                         , paramIN
                         , paramOUT 
                         , dateIN
                         , dateOUT
                         , resultado );
         
         EXCEPTION
            WHEN OTHERS THEN
                CASE (SQLCODE)
                    WHEN -1 THEN
                        -- Registro duplicado
                        resultado := 'ERROR: El nmero de serie del producto ya existe.';
                    WHEN 100 THEN
                        -- Registro no encontrado
                        resultado := 'ERROR: El identificador de electrodomstico no existe.';
                    WHEN -20000 THEN
                        -- Electrodomstico dado de baja
                        resultado := 'ERROR: El electrodomstico est de baja.';
                    WHEN -1400 THEN
                        -- Valor nulo
                        resultado := 'ERROR: Se ha intentado almacenar un valor nulo.' || SUBSTR(SQLERRM,INSTR(SQLERRM,'('));
                    WHEN -2291 THEN
                        -- Restriccin de integridad
                        IF (INSTR(SQLERRM,'R02')>0) THEN
                            resultado := 'ERROR: El cdigo postal no existe.';
                        ELSE
                            resultado := 'ERROR: El identificador de cliente no existe';
                        END IF;
                    ELSE
                        -- Error no controlado
                        resultado := 'ERROR: ' || SQLCODE || ' - ' || SQLERRM;
                END CASE;            

                -- Asignamos valores salida
                paramOUT      := 'resultado = ' || resultado;
                dateOUT       := SYSDATE;
                
                -- Mensaje de error
                DBMS_OUTPUT.put_line (resultado);

                -- Grabamos en el LOG
                CONTROL.GRABA_LOG( procedimiento
                                 , paramIN
                                 , paramOUT 
                                 , dateIN
                                 , dateOUT
                                 , resultado );
    END INS_PRODUCTO;


    -- RF51
    PROCEDURE UPD_PRODUCTO( pId IN BD.PRODUCTOS.IDPRODUCTO%TYPE
                          , pIdElectrodomestico IN BD.PRODUCTOS.IDELECTRODOMESTICO%TYPE
                          , pMarca IN BD.PRODUCTOS.MARCA%TYPE
                          , pModelo IN BD.PRODUCTOS.MODELO%TYPE
                          , pNumSerie IN BD.PRODUCTOS.NUMSERIE%TYPE
                          , pFxCompra IN BD.PRODUCTOS.FXCOMPRA%TYPE
                          , pDireccion IN BD.PRODUCTOS.DIRECCION%TYPE
                          , pCodPostal IN BD.PRODUCTOS.CODPOSTAL%TYPE
                          , pIdCliente IN BD.PRODUCTOS.IDCLIENTE%TYPE
                          , resultado OUT BD.LOG.RESULTADO%TYPE )
 IS
    /*
        RF51 - CASO DE USO: MODIFICAR PRODUCTO
        PARAMETROS ENTRADA:
            pId:        identificador del Producto
            pIdElectrodomestico: Identidicador del tipo de electrodomstico del producto
            pMarca: Marca del producto
            pModelo: Modelo del producto
            pNumSerie: Nmero de serie del producto
            pFxCompra: Fecha de compra del producto
            pDireccin: Direccin en la que se encuentra el producto
            pCodPostal: Cdigo postal de la poblacin en la que se encuentra el producto
            pIdCliente: Identificador del propietario del producto
        PARAMETROS SALIDA:
            resultado: Informacin del resultado de la operacin.
    */
        procedimiento BD.LOG.PROCEDIMIENTO%TYPE;
        paramIN       BD.LOG.PARAMIN%TYPE;
        paramOUT      BD.LOG.PARAMOUT%TYPE;
        dateIN        BD.LOG.FXIN%TYPE;
        dateOUT       BD.LOG.FXOUT%TYPE;
        idElec        BD.PRODUCTOS.IDELECTRODOMESTICO%TYPE;
        fechaBaja     BD.ELECTRODOMESTICOS.FXBAJA%TYPE;
    BEGIN
    
        -- Asignamos valores entrada
        procedimiento := $$PLSQL_UNIT || '.UPD_CLIENTE';
        paramIN       := 'pId = ' || pId
                      || 'pIdElectrodomestico= ' || pIdElectrodomestico
                      || ' ,pMarca = ' || pMarca
                      || ' ,pModelo = ' || pModelo
                      || ' ,pNumSerie = ' || pNumSerie
                      || ' ,pFxCompra = ' || pFxCompra
                      || ' ,pDireccion = ' || pDireccion
                      || ' ,pCodPostal = ' || pCodPostal
                      || ' ,pIdCliente = ' || pIdCliente; 
        dateIN        := SYSDATE;
        resultado     := 'OK. El producto se ha modificado con xito';

        -- Miramos que exista el producto
        SELECT idElectrodomestico INTO idElec 
        FROM PRODUCTOS
        WHERE idProducto = pId;
        
        -- Miramos si el electrodomstico exite y est de alta
        BEGIN
            SELECT fxBaja INTO fechaBaja
            FROM ELECTRODOMESTICOS
            WHERE idElectrodomestico = pIdElectrodomestico;
        EXCEPTION
            WHEN OTHERS THEN
                IF (SQLCODE = 100) THEN
                    -- Registro No encontrado
                    RAISE_APPLICATION_ERROR(-20000,'');
                END IF; 
        END;
        
        IF (fechaBaja IS NOT NULL) THEN
            RAISE_APPLICATION_ERROR(-20001,'');        
        END IF;

        --  Modidicamos los datos del Producto (idElectrodomestico, marca, modelo, numSerie, fxCompra, direccion, codpostal, idCliente)
        UPDATE PRODUCTOS
        SET idElectrodomestico = pIdElectrodomestico
          , marca = UPPER(pMarca)
          , modelo = UPPER(pModelo)
          , numSerie = UPPER(pNumSerie)
          , fxCompra = pFxCompra
          , direccion = UPPER(pDireccion)
          , codpostal = pCodPostal
          , idCliente = pIdCliente
        WHERE idProducto = pId;
        
        -- Asignamos valores salida
        paramOUT      := 'resultado = ' || resultado;
        dateOUT       := SYSDATE;
        
        -- Grabamos en el LOG
        CONTROL.GRABA_LOG( procedimiento
                         , paramIN
                         , paramOUT 
                         , dateIN
                         , dateOUT
                         , resultado );
         
         EXCEPTION
            WHEN OTHERS THEN
                CASE (SQLCODE)
                    WHEN -1 THEN
                        -- Registro duplicado
                        resultado := 'ERROR: El nmero de serie del producto ya existe.';
                    WHEN 100 THEN
                        -- Registro no encontrado
                        resultado := 'ERROR: El identificador del producto no existe.';
                    WHEN -20000 THEN
                        -- Electrodomstico no encontrado
                        resultado := 'ERROR: El identificador de electrodomstico no existe.';
                    WHEN -20001 THEN
                        -- Electrodomstico dado de baja
                        resultado := 'ERROR: El electrodomstico est de baja.';
                    WHEN -1407 THEN
                        -- Valor nulo
                        resultado := 'ERROR: Se ha intentado actualizar a un valor nulo ' || SUBSTR(SQLERRM,INSTR(SQLERRM,'('));
                    WHEN -2291 THEN
                        -- Restriccin de integridad
                        IF (INSTR(SQLERRM,'R02')>0) THEN
                            resultado := 'ERROR: El cdigo postal no existe.';
                        ELSE
                            resultado := 'ERROR: El identificador de cliente no existe';
                        END IF;
                    ELSE
                        -- Error no controlado
                        resultado := 'ERROR: ' ||  SQLERRM;
                END CASE;            

                -- Asignamos valores salida
                paramOUT      := 'resultado = ' || resultado;
                dateOUT       := SYSDATE;
                
                -- Mensaje de error
                DBMS_OUTPUT.put_line (resultado);

                -- Grabamos en el LOG
                CONTROL.GRABA_LOG( procedimiento
                                 , paramIN
                                 , paramOUT 
                                 , dateIN
                                 , dateOUT
                                 , resultado );
    END UPD_PRODUCTO;


    -- RF52
    PROCEDURE GET_PRODUCTO( pId IN BD.PRODUCTOS.IDPRODUCTO%TYPE
                          , rs OUT SYS_REFCURSOR)
    IS
    /*
        RF52 - CASO DE USO: CONSULTAR PRODUCTO
        PARAMETROS ENTRADA:
            pId: identificador del producto
                valor nulo devolver todos los registros.   
        PARAMETROS SALIDA:
            rs: cursor con el/los registro obtenido
    */
        procedimiento BD.LOG.PROCEDIMIENTO%TYPE;
        paramIN       BD.LOG.PARAMIN%TYPE;
        paramOUT      BD.LOG.PARAMOUT%TYPE;
        dateIN        BD.LOG.FXIN%TYPE;
        dateOUT       BD.LOG.FXOUT%TYPE;
        resultado     BD.LOG.RESULTADO%TYPE;
    BEGIN
    
        -- Asignamos valores entrada
        procedimiento := $$PLSQL_UNIT || '.GET_PRODUCTO';
        paramIN       := 'pId = ' || pId;
        dateIN        := SYSDATE;
        resultado     := 'OK. El(Los) producto(s) se ha(n) consultado con xito';

        -- Realizamos la consulta en funcin del parmetro de entrada
        IF (pId IS NULL) THEN
            OPEN rs FOR
            SELECT * FROM PRODUCTOS;
        ELSE
            OPEN rs FOR
            SELECT * FROM PRODUCTOS WHERE idProducto = pId;
        END IF; 
        
        -- Asignamos valores salida
        paramOUT      := 'rs = Cursor de Productos.';
        dateOUT       := SYSDATE;
        
        -- Grabamos en el LOG
        CONTROL.GRABA_LOG( procedimiento
                         , paramIN
                         , paramOUT 
                         , dateIN
                         , dateOUT
                         , resultado );
         
         EXCEPTION
            WHEN OTHERS THEN
                CASE (SQLCODE)
                    WHEN 100 THEN
                        -- Registro no encontrado
                        resultado := 'ERROR: Registro(s) no encontrado(s).';
                    ELSE
                        -- Error no controlado
                        resultado := 'ERROR: ' ||  SQLERRM;
                END CASE;            

                -- Generamos un cursor vacio
                OPEN rs FOR
                SELECT * FROM DUAL WHERE DUMMY IS NULL;
                
                -- Asignamos valores salida
                paramOUT      := 'rs = Cursor vaco.';
                dateOUT       := SYSDATE;
                
                -- Mensaje de error
                DBMS_OUTPUT.put_line (resultado);

                -- Grabamos en el LOG
                CONTROL.GRABA_LOG( procedimiento
                                 , paramIN
                                 , paramOUT 
                                 , dateIN
                                 , dateOUT
                                 , resultado );
    END GET_PRODUCTO;


/*
    PETICIONES
*/     
    --RF53
    PROCEDURE INS_PETICION( pIdProducto IN BD.PETICIONES.IDPRODUCTO%TYPE
                          , pIdServicio IN BD.PETICIONES.IDSERVICIO%TYPE
                          , pIdTeleoperador IN BD.PETICIONES.IDTELEOPERADOR%TYPE
                          , resultado OUT BD.LOG.RESULTADO%TYPE )
    IS
    /*
        RF53 - CASO DE USO: CREAR PETICION
        PARAMETROS ENTRADA:
            pIdProducto:    Identificador del producto del que se graba la incidencia
            pIdServicio:    Identificador del tipo de servicio solicitado.
            pIdTeleoperador:Identificador del teleoperador que registra la peticin.
        PARAMETROS SALIDA:
            resultado: Informacin del resultado de la operacin.
    */
        procedimiento BD.LOG.PROCEDIMIENTO%TYPE;
        paramIN       BD.LOG.PARAMIN%TYPE;
        paramOUT      BD.LOG.PARAMOUT%TYPE;
        dateIN        BD.LOG.FXIN%TYPE;
        dateOUT       BD.LOG.FXOUT%TYPE;
        fxBajaSer     BD.SERVICIOS.FXBAJA%TYPE;
        fxBajaTel     BD.TELEOPERADORES.FXBAJA%TYPE;
    BEGIN
    
        -- Asignamos valores entrada
        procedimiento := $$PLSQL_UNIT|| '.INS_PETICION';
        paramIN       := 'pIdProducto = ' || pIdProducto
                      || ' ,pIdServicio = ' || pIdServicio
                      || ' ,pIdTeleoperador = ' || pIdTeleoperador;
        dateIN        := SYSDATE;
        resultado     := 'OK. La peticin se ha creado con xito';

        -- Miramos si el servicio exite y est de alta
        BEGIN
            SELECT fxBaja INTO fxBajaSer
            FROM SERVICIOS
            WHERE idServicio = pIdServicio;
        EXCEPTION
            WHEN OTHERS THEN
                IF (SQLCODE = 100) THEN
                    -- Registro No encontrado
                    RAISE_APPLICATION_ERROR(-20000,'');
                END IF; 
        END;        
        
        IF (fxBajaSer IS NOT NULL) THEN
            RAISE_APPLICATION_ERROR(-20001,'');        
        END IF;

        -- Miramos si el teleoperador exite y est de alta
        BEGIN
            SELECT fxBaja INTO fxBajaTel
            FROM TELEOPERADORES
            WHERE idTeleoperador = pIdTeleoperador;
        EXCEPTION
            WHEN OTHERS THEN
                IF (SQLCODE = 100) THEN
                    -- Registro No encontrado
                    RAISE_APPLICATION_ERROR(-20010,'');
                END IF; 
        END;        
        
        IF (fxBajaTel IS NOT NULL) THEN
            RAISE_APPLICATION_ERROR(-20011,'');        
        END IF;


        -- Insertamos la Peticin
        INSERT INTO PETICIONES(idProducto, idServicio, idTeleoperador) 
        VALUES(pIdProducto, pIdServicio, pIdTeleoperador);
        
        -- Asignamos valores salida
        paramOUT      := 'resultado = ' || resultado;
        dateOUT       := SYSDATE;
        
        -- Grabamos en el LOG
        CONTROL.GRABA_LOG( procedimiento
                         , paramIN
                         , paramOUT 
                         , dateIN
                         , dateOUT
                         , resultado );
         
         EXCEPTION
            WHEN OTHERS THEN
                CASE (SQLCODE)
                    WHEN -1400 THEN
                        -- Valor nulo
                        resultado := 'ERROR: Se ha intentado almacenar un valor nulo.' || SUBSTR(SQLERRM,INSTR(SQLERRM,'('));
                    WHEN -20000 THEN
                        -- Servicio no existe
                        resultado := 'ERROR: El identificador de servicio no existe.';
                    WHEN -20001 THEN
                        -- Servicio dado de baja
                        resultado := 'ERROR: El servicio est de baja.';
                    WHEN -20010 THEN
                        -- Teleoperador no existe
                        resultado := 'ERROR: El identificador de teleoperador no existe.';
                    WHEN -20011 THEN
                        -- Teleoperador dado de baja
                        resultado := 'ERROR: El teleoperador est de baja.';
                    WHEN -2291 THEN
                        -- Restriccin de integridad
                        IF (INSTR(SQLERRM,'R01')>0) THEN
                            resultado := 'ERROR: El identificador de producto no existe.';
                        ELSE
                            resultado := 'ERROR: Referencia no encontrada. ' || SQLERRM;
                        END IF;
                    ELSE
                        -- Error no controlado
                        resultado := 'ERROR: ' || SQLCODE || ' - ' || SQLERRM;
                END CASE;            

                -- Asignamos valores salida
                paramOUT      := 'resultado = ' || resultado;
                dateOUT       := SYSDATE;
                
                -- Mensaje de error
                DBMS_OUTPUT.put_line (resultado);

                -- Grabamos en el LOG
                CONTROL.GRABA_LOG( procedimiento
                                 , paramIN
                                 , paramOUT 
                                 , dateIN
                                 , dateOUT
                                 , resultado );
    END INS_PETICION;


    -- RF54
    PROCEDURE CANCEL_PETICION( pId IN BD.PETICIONES.IDPETICION%TYPE
                             , resultado OUT BD.LOG.RESULTADO%TYPE )
    IS
    /*
        RF54 - CASO DE USO: CANCELAR PETICION
        PARAMETROS ENTRADA:
            pId:        identificador de la peticin
        PARAMETROS SALIDA:
            resultado: Informacin del resultado de la operacin.
    */
        procedimiento BD.LOG.PROCEDIMIENTO%TYPE;
        paramIN       BD.LOG.PARAMIN%TYPE;
        paramOUT      BD.LOG.PARAMOUT%TYPE;
        dateIN        BD.LOG.FXIN%TYPE;
        dateOUT       BD.LOG.FXOUT%TYPE;
        estado        BD.PETICIONES.IDESTADO%TYPE;
        
    BEGIN
    
        -- Asignamos valores entrada
        procedimiento := $$PLSQL_UNIT || '.CANCEL_PETICION';
        paramIN       := 'pId = ' || pId; 
        dateIN        := SYSDATE;
        resultado     := 'OK. La peticin se ha cancelado con xito';

        -- Miramos que exista y est pendiente
        SELECT idEstado INTO estado 
        FROM PETICIONES
        WHERE idPeticion = pId;
        
        IF (estado!='P') THEN
            RAISE_APPLICATION_ERROR(-20000,'');        
        END IF;

        --  Cancelamos la peticin (idEstado = A(nualda), fxCierre = SYSDATE)
        UPDATE PETICIONES
        SET idEstado = 'A'
          , fxCierre = SYSDATE
        WHERE idPeticion = pId;
        
        -- Asignamos valores salida
        paramOUT      := 'resultado = ' || resultado;
        dateOUT       := SYSDATE;
        
        -- Grabamos en el LOG
        CONTROL.GRABA_LOG( procedimiento
                         , paramIN
                         , paramOUT 
                         , dateIN
                         , dateOUT
                         , resultado );
         
         EXCEPTION
            WHEN OTHERS THEN
                CASE (SQLCODE)
                    WHEN 100 THEN
                        -- Registro no encontrado
                        resultado := 'ERROR: El identificador de la peticin no existe.';
                    WHEN -20000 THEN
                        -- Peticin no est en estado pendiente
                        resultado := 'ERROR: La peticin no se encuentra en estado PENDIENTE.';
                    ELSE
                        -- Error no controlado
                        resultado := 'ERROR: ' ||  SQLERRM;
                END CASE;            

                -- Asignamos valores salida
                paramOUT      := 'resultado = ' || resultado;
                dateOUT       := SYSDATE;
                
                -- Mensaje de error
                DBMS_OUTPUT.put_line (resultado);

                -- Grabamos en el LOG
                CONTROL.GRABA_LOG( procedimiento
                                 , paramIN
                                 , paramOUT 
                                 , dateIN
                                 , dateOUT
                                 , resultado );
    END CANCEL_PETICION;


    -- RF55
    PROCEDURE GET_PETICION( pId IN BD.PETICIONES.IDPETICION%TYPE
                          , rs OUT SYS_REFCURSOR)
    IS
    /*
        RF55 - CASO DE USO: CONSULTAR PETICION
        PARAMETROS ENTRADA:
            pId: identificador de la peticin
                valor nulo devolver todos los registros.   
        PARAMETROS SALIDA:
            rs: cursor con el/los registro obtenido
    */
        procedimiento BD.LOG.PROCEDIMIENTO%TYPE;
        paramIN       BD.LOG.PARAMIN%TYPE;
        paramOUT      BD.LOG.PARAMOUT%TYPE;
        dateIN        BD.LOG.FXIN%TYPE;
        dateOUT       BD.LOG.FXOUT%TYPE;
        resultado     BD.LOG.RESULTADO%TYPE;
    BEGIN
    
        -- Asignamos valores entrada
        procedimiento := $$PLSQL_UNIT || '.GET_PETICION';
        paramIN       := 'pId = ' || pId;
        dateIN        := SYSDATE;
        resultado     := 'OK. La(s) peticin(es) se ha(n) consultado con xito';

        -- Realizamos la consulta en funcin del parmetro de entrada
        IF (pId IS NULL) THEN
            OPEN rs FOR
            SELECT * FROM PETICIONES;
        ELSE
            OPEN rs FOR
            SELECT * FROM PETICIONES WHERE idPeticion = pId;
        END IF; 
        
        -- Asignamos valores salida
        paramOUT      := 'rs = Cursor de Peticiones.';
        dateOUT       := SYSDATE;
        
        -- Grabamos en el LOG
        CONTROL.GRABA_LOG( procedimiento
                         , paramIN
                         , paramOUT 
                         , dateIN
                         , dateOUT
                         , resultado );
         
         EXCEPTION
            WHEN OTHERS THEN
                CASE (SQLCODE)
                    WHEN 100 THEN
                        -- Registro no encontrado
                        resultado := 'ERROR: Registro(s) no encontrado(s).';
                    ELSE
                        -- Error no controlado
                        resultado := 'ERROR: ' ||  SQLERRM;
                END CASE;            

                -- Generamos un cursor vacio
                OPEN rs FOR
                SELECT * FROM DUAL WHERE DUMMY IS NULL;
                
                -- Asignamos valores salida
                paramOUT      := 'rs = Cursor vaco.';
                dateOUT       := SYSDATE;
                
                -- Mensaje de error
                DBMS_OUTPUT.put_line (resultado);

                -- Grabamos en el LOG
                CONTROL.GRABA_LOG( procedimiento
                                 , paramIN
                                 , paramOUT 
                                 , dateIN
                                 , dateOUT
                                 , resultado );
    END GET_PETICION;


END PETICION;

