/*
    PACKAGE ATENCION
    Fecha: 02/05/2011
    Autor: Francisco Amador Risquez Teva
    Proyecto: gPV
*/



/*
    ESPECIFICACIN DEL PAQUETE
    SUBSISTEMA ATENCION
*/
CREATE OR REPLACE PACKAGE BD.ATENCION AS

    /*
        BUSQUEDAS Y ASIGNACIONES
    */     
    --RF56
    PROCEDURE GET_PETICIONESSERVICIOZONA( pIdServicio IN BD.SERVICIOS.IDSERVICIO%TYPE
                                        , pIdZona IN BD.ZONAS.IDZONA%TYPE
                                        , rs OUT SYS_REFCURSOR);
    
    --RF57
    PROCEDURE GET_PETICIONESTECNICO( pIdTecnico IN BD.TECNICOS.IDTECNICO%TYPE
                                   , rs OUT SYS_REFCURSOR);

    --RF58
    PROCEDURE ASG_TECNICO( pIdPeticion IN BD.PETICIONES.IDPETICION%TYPE
                         , pIdTecnico IN BD.PETICIONES.IDTECNICO%TYPE
                         , resultado OUT BD.LOG.RESULTADO%TYPE );

    --RF59
    PROCEDURE DSG_TECNICO( pIdPeticion IN BD.PETICIONES.IDPETICION%TYPE
                         , resultado OUT BD.LOG.RESULTADO%TYPE );

    --RF60
    PROCEDURE GET_TECNICOASIGNADO( pId IN BD.PETICIONES.IDPETICION%TYPE
                                 , rs OUT SYS_REFCURSOR);
    
    /*
        TRABAJOS
    */     
    --RF61
    PROCEDURE INS_TRABAJO( pIdPeticion IN BD.TRABAJOS.IDPETICION%TYPE
                         , pIdTecnico IN BD.TRABAJOS.IDTECNICO%TYPE
                         , pDsTrabajo IN BD.TRABAJOS.DSTRABAJO%TYPE
                         , pImporte IN BD.TRABAJOS.IMPORTE%TYPE
                         , pGarantia IN BD.TRABAJOS.GARANTIA%TYPE
                         , resultado OUT BD.LOG.RESULTADO%TYPE );
    

    --RF62
    PROCEDURE GET_TRABAJO( pId IN BD.TRABAJOS.IDTRABAJO%TYPE
                         , rs OUT SYS_REFCURSOR);


    --RF63
    PROCEDURE CLOSE_PETICION( pIdPeticion IN BD.PETICIONES.IDPETICION%TYPE
                            , resultado OUT BD.LOG.RESULTADO%TYPE );



    /*
        ENCUESTA
    */     
    --RF64
    PROCEDURE INS_ENCUESTA( pIdPeticion IN BD.ENCUESTAS.IDPETICION%TYPE
                          , pIdTeleoperador IN BD.ENCUESTAS.IDTELEOPERADOR%TYPE
                          , pValoracion IN BD.ENCUESTAS.VALORACION%TYPE
                          , pComentario IN BD.ENCUESTAS.COMENTARIO%TYPE
                          , resultado OUT BD.LOG.RESULTADO%TYPE );
    

    --RF65
    PROCEDURE GET_ENCUESTA( pId IN BD.ENCUESTAS.IDENCUESTA%TYPE
                          , rs OUT SYS_REFCURSOR);



END ATENCION;



/*
    CUERPO DEL PAQUETE
    SUBSISTEMA ATENCION
*/
CREATE OR REPLACE PACKAGE BODY BD.ATENCION AS

/*
        BUSQUEDAS Y ASIGNACIONES
*/     
    -- RF56
    PROCEDURE GET_PETICIONESSERVICIOZONA( pIdServicio IN BD.SERVICIOS.IDSERVICIO%TYPE
                                        , pIdZona IN BD.ZONAS.IDZONA%TYPE
                                        , rs OUT SYS_REFCURSOR)
    IS
    /*
        RF56 - CASO DE USO: CONSULTAR PETICIONES POR SERVICIO Y ZONA
        PARAMETROS ENTRADA:
            pIdServicio: identificador de servicio
                valor nulo devolver todos los servicios.   
            pIdZona: identificador de zona
                valor nulo devolver todas las zonas.   
        PARAMETROS SALIDA:
            rs: cursor con el/los registro obtenido
    */
        procedimiento BD.LOG.PROCEDIMIENTO%TYPE;
        paramIN       BD.LOG.PARAMIN%TYPE;
        paramOUT      BD.LOG.PARAMOUT%TYPE;
        dateIN        BD.LOG.FXIN%TYPE;
        dateOUT       BD.LOG.FXOUT%TYPE;
        resultado     BD.LOG.RESULTADO%TYPE;
        fxBaja        BD.SERVICIOS.fxBaja%TYPE;
    BEGIN
    
        -- Asignamos valores entrada
        procedimiento := $$PLSQL_UNIT || '.GET_PETICIONESSERVICIOZONA';
        paramIN       := 'pIdServicio = ' || pIdServicio
                      || ' ,pIdZona = ' || pIdZona;
        dateIN        := SYSDATE;
        resultado     := 'OK. La(s) peticin(es) se ha(n) consultado con xito';

        -- Realizamos la consulta en funcin del parmetro de entrada
        IF (pIdServicio IS NULL) THEN
            IF (pIdZona IS NULL) THEN
                OPEN rs FOR
                SELECT * FROM PETICIONES WHERE idEstado='P';
            ELSE
                OPEN rs FOR
                SELECT PET.* 
                FROM PETICIONES PET 
                     INNER JOIN (PRODUCTOS PRO INNER JOIN ZONASPOBLACIONES ZP ON ZP.codPostal = PRO.codPostal) ON PRO.idProducto = PET.idProducto               
                WHERE PET.idEstado='P'
                AND ZP.idZona = pIdZona;
            END IF;
        ELSE
            IF (pIdZona IS NULL) THEN
                OPEN rs FOR
                SELECT * FROM PETICIONES WHERE idEstado='P' AND idServicio = pIdServicio;
            ELSE
                OPEN rs FOR
                SELECT PET.* 
                FROM PETICIONES PET 
                     INNER JOIN (PRODUCTOS PRO INNER JOIN ZONASPOBLACIONES ZP ON ZP.codPostal = PRO.codPostal) ON PRO.idProducto = PET.idProducto               
                WHERE PET.idEstado='P'
                AND PET.idServicio = pIdServicio
                AND ZP.idZona = pIdZona;
            END IF;
        END IF; 
        
        -- Asignamos valores salida
        paramOUT      := 'rs = Cursor de Peticiones.';
        dateOUT       := SYSDATE;
        
        -- Grabamos en el LOG
        CONTROL.GRABA_LOG( procedimiento
                         , paramIN
                         , paramOUT 
                         , dateIN
                         , dateOUT
                         , resultado );
         
         EXCEPTION
            WHEN OTHERS THEN
                CASE (SQLCODE)
                    WHEN 100 THEN
                        -- Registro no encontrado
                        resultado := 'ERROR: Registro(s) no encontrado(s).';
                    ELSE
                        -- Error no controlado
                        resultado := 'ERROR: ' ||  SQLERRM;
                END CASE;            

                -- Generamos un cursor vacio
                OPEN rs FOR
                SELECT * FROM DUAL WHERE DUMMY IS NULL;
                
                -- Asignamos valores salida
                paramOUT      := 'rs = Cursor vaco.';
                dateOUT       := SYSDATE;
                
                -- Mensaje de error
                DBMS_OUTPUT.put_line (resultado);

                -- Grabamos en el LOG
                CONTROL.GRABA_LOG( procedimiento
                                 , paramIN
                                 , paramOUT 
                                 , dateIN
                                 , dateOUT
                                 , resultado );
    END GET_PETICIONESSERVICIOZONA;


    -- RF57
    PROCEDURE GET_PETICIONESTECNICO( pIdTecnico IN BD.TECNICOS.IDTECNICO%TYPE
                                   , rs OUT SYS_REFCURSOR)
    IS
    /*
        RF57 - CASO DE USO: CONSULTAR PETICIONES POR TECNICO
        PARAMETROS ENTRADA:
            pIdTecnico: identificador de tcnicoque puede atender las peticiones dadas
                valor nulo devolver todos las peticiones.   
        PARAMETROS SALIDA:
            rs: cursor con el/los registro obtenido
    */
        procedimiento BD.LOG.PROCEDIMIENTO%TYPE;
        paramIN       BD.LOG.PARAMIN%TYPE;
        paramOUT      BD.LOG.PARAMOUT%TYPE;
        dateIN        BD.LOG.FXIN%TYPE;
        dateOUT       BD.LOG.FXOUT%TYPE;
        resultado     BD.LOG.RESULTADO%TYPE;
    BEGIN
    
        -- Asignamos valores entrada
        procedimiento := $$PLSQL_UNIT || '.GET_PETICIONESTECNICO';
        paramIN       := 'pIdTecnico = ' || pIdTecnico;
        dateIN        := SYSDATE;
        resultado     := 'OK. La(s) peticin(es) se ha(n) consultado con xito';

        -- Realizamos la consulta en funcin del parmetro de entrada
        IF (pIdTecnico IS NULL) THEN
            OPEN rs FOR
            SELECT * FROM PETICIONES WHERE idEstado='P';
        ELSE
            OPEN rs FOR
            SELECT PET.*
            FROM PETICIONES PET
                 INNER JOIN (PRODUCTOS PRO INNER JOIN (ZONASPOBLACIONES ZP 
                                                      INNER JOIN TECNICOSZONAS TZ ON ZP.idZona = TZ.idZona) ON PRO.codPostal = ZP.codPostal
                                          INNER JOIN TECNICOSELECTRODOMESTICOS TE ON PRO.idElectrodomestico = TE.idElectrodomestico)                              
                 ON PET.idProducto = PRO.idProducto
                 INNER JOIN TECNICOSSERVICIOS TS ON PET.idServicio = TS.idServicio
            WHERE TZ.idTecnico = pIdTecnico
            AND TE.idTecnico = pIdTecnico
            AND TS.idTecnico = pIdTecnico
            AND PET.idEstado = 'P';
        END IF;
        
        -- Asignamos valores salida
        paramOUT      := 'rs = Cursor de Peticiones.';
        dateOUT       := SYSDATE;
        
        -- Grabamos en el LOG
        CONTROL.GRABA_LOG( procedimiento
                         , paramIN
                         , paramOUT 
                         , dateIN
                         , dateOUT
                         , resultado );
         
         EXCEPTION
            WHEN OTHERS THEN
                CASE (SQLCODE)
                    WHEN 100 THEN
                        -- Registro no encontrado
                        resultado := 'ERROR: Registro(s) no encontrado(s).';
                    ELSE
                        -- Error no controlado
                        resultado := 'ERROR: ' ||  SQLERRM;
                END CASE;            

                -- Generamos un cursor vacio
                OPEN rs FOR
                SELECT * FROM DUAL WHERE DUMMY IS NULL;
                
                -- Asignamos valores salida
                paramOUT      := 'rs = Cursor vaco.';
                dateOUT       := SYSDATE;
                
                -- Mensaje de error
                DBMS_OUTPUT.put_line (resultado);

                -- Grabamos en el LOG
                CONTROL.GRABA_LOG( procedimiento
                                 , paramIN
                                 , paramOUT 
                                 , dateIN
                                 , dateOUT
                                 , resultado );
    END GET_PETICIONESTECNICO;


    --RF58
    PROCEDURE ASG_TECNICO( pIdPeticion IN BD.PETICIONES.IDPETICION%TYPE
                         , pIdTecnico IN BD.PETICIONES.IDTECNICO%TYPE
                         , resultado OUT BD.LOG.RESULTADO%TYPE )
    IS
    /*
        RF58 - CASO DE USO: ASIGNAR TECNICO
        PARAMETROS ENTRADA:
            pIdPeticion:    Identificador de la peticin
            pIdTecnico:     Identificador de Tcnico
        PARAMETROS SALIDA:
            ninguno
    */
        procedimiento BD.LOG.PROCEDIMIENTO%TYPE;
        paramIN       BD.LOG.PARAMIN%TYPE;
        paramOUT      BD.LOG.PARAMOUT%TYPE;
        dateIN        BD.LOG.FXIN%TYPE;
        dateOUT       BD.LOG.FXOUT%TYPE;
        estado        BD.PETICIONES.IDESTADO%TYPE;
        fechaBaja     BD.TECNICOS.FXBAJA%TYPE;
    BEGIN
    
        -- Asignamos valores entrada
        procedimiento := $$PLSQL_UNIT || '.ASG_TECNICO';
        paramIN       := 'pIdPeticion = ' || pIdPeticion
                      || ' ,pIdTecnico = ' || pIdTecnico;
        dateIN        := SYSDATE;
        resultado     := 'OK. La asignacin se ha realizado con xito';

        -- Miramos que exista la peticin y est pendiente
        BEGIN
            SELECT idEstado INTO estado 
            FROM PETICIONES
            WHERE idPeticion = pIdPeticion;
        EXCEPTION
            WHEN OTHERS THEN
                IF (SQLCODE = 100) THEN
                    -- Registro No encontrado
                    RAISE_APPLICATION_ERROR(-20000,'');
                END IF; 
        END;
                
        IF (estado!='P') THEN
            RAISE_APPLICATION_ERROR(-20001,'');        
        END IF;
        
        
        -- Miramos que exista el tcnico y que est de alta
        BEGIN
            SELECT fxBaja INTO fechaBaja 
            FROM TECNICOS
            WHERE idTecnico = pIdTecnico;
        EXCEPTION
            WHEN OTHERS THEN
                IF (SQLCODE = 100) THEN
                    -- Registro No encontrado
                    RAISE_APPLICATION_ERROR(-20010,'');
                END IF; 
        END;
                
        IF (fechaBaja IS NOT NULL) THEN
            RAISE_APPLICATION_ERROR(-20011,'');        
        END IF;

        --  Asignamos el servicio al tcnico
        UPDATE PETICIONES
        SET idTecnico = pIdTecnico
          , idEstado = 'T'
        WHERE idPeticion = pIdPeticion;
        
        -- Asignamos valores salida
        paramOUT      := 'resultado = ' || resultado;
        dateOUT       := SYSDATE;
        
        -- Grabamos en el LOG
        CONTROL.GRABA_LOG( procedimiento
                         , paramIN
                         , paramOUT 
                         , dateIN
                         , dateOUT
                         , resultado );
         
         EXCEPTION
            WHEN OTHERS THEN
                CASE (SQLCODE)
                    WHEN -20000 THEN
                        -- Tcnico dado de baja
                        resultado := 'ERROR: El identificador de peticin no existe.';
                    WHEN -20001 THEN
                        -- Estado Errneo
                        resultado := 'ERROR: El peticin no se encuentra en estado PENDIENTE.';
                    WHEN -20010 THEN
                        -- Tcnico dado de baja
                        resultado := 'ERROR: El identificador de tcnico no existe.';
                    WHEN -20011 THEN
                        -- Tcnico dado de baja
                        resultado := 'ERROR: El tcnico est de baja.';
                    ELSE
                        -- Error no controlado
                        resultado := 'ERROR: ' ||  SQLERRM;
                END CASE;            

                -- Asignamos valores salida
                paramOUT      := 'resultado = ' || resultado;
                dateOUT       := SYSDATE;
                
                -- Mensaje de error
                DBMS_OUTPUT.put_line (resultado);

                -- Grabamos en el LOG
                CONTROL.GRABA_LOG( procedimiento
                                 , paramIN
                                 , paramOUT 
                                 , dateIN
                                 , dateOUT
                                 , resultado );
    END ASG_TECNICO;
                         

    --RF59
    PROCEDURE DSG_TECNICO( pIdPeticion IN BD.PETICIONES.IDPETICION%TYPE
                         , resultado OUT BD.LOG.RESULTADO%TYPE )
    IS
    /*
        RF59 - CASO DE USO: DESASIGNAR TECNICO
        PARAMETROS ENTRADA:
            pIdPeticion:    Identificador de la peticin
        PARAMETROS SALIDA:
            ninguno
    */
        procedimiento BD.LOG.PROCEDIMIENTO%TYPE;
        paramIN       BD.LOG.PARAMIN%TYPE;
        paramOUT      BD.LOG.PARAMOUT%TYPE;
        dateIN        BD.LOG.FXIN%TYPE;
        dateOUT       BD.LOG.FXOUT%TYPE;
        estado        BD.PETICIONES.IDESTADO%TYPE;
    BEGIN
    
        -- Asignamos valores entrada
        procedimiento := $$PLSQL_UNIT || '.DES_TECNICO';
        paramIN       := 'pIdPeticion = ' || pIdPeticion;
        dateIN        := SYSDATE;
        resultado     := 'OK. La desasignacin se ha realizado con xito';

        -- Miramos que exista la peticin y est asignada
        SELECT idEstado INTO estado 
        FROM PETICIONES
        WHERE idPeticion = pIdPeticion;
                
        IF (estado!='T') THEN
            RAISE_APPLICATION_ERROR(-20000,'');        
        END IF;
        
        
        --  Asignamos el servicio al tcnico
        UPDATE PETICIONES
        SET idTecnico = null
          , idEstado = 'P'
        WHERE idPeticion = pIdPeticion;
        
        -- Asignamos valores salida
        paramOUT      := 'resultado = ' || resultado;
        dateOUT       := SYSDATE;
        
        -- Grabamos en el LOG
        CONTROL.GRABA_LOG( procedimiento
                         , paramIN
                         , paramOUT 
                         , dateIN
                         , dateOUT
                         , resultado );
         
         EXCEPTION
            WHEN OTHERS THEN
                CASE (SQLCODE)
                    WHEN 100 THEN
                        -- Tcnico dado de baja
                        resultado := 'ERROR: El identificador de peticin no existe.';
                    WHEN -20000 THEN
                        -- Estdo errneo
                        resultado := 'ERROR: El peticin no se encuentra en estado ASIGNADA.';
                    ELSE
                        -- Error no controlado
                        resultado := 'ERROR: ' ||  SQLERRM;
                END CASE;            

                -- Asignamos valores salida
                paramOUT      := 'resultado = ' || resultado;
                dateOUT       := SYSDATE;
                
                -- Mensaje de error
                DBMS_OUTPUT.put_line (resultado);

                -- Grabamos en el LOG
                CONTROL.GRABA_LOG( procedimiento
                                 , paramIN
                                 , paramOUT 
                                 , dateIN
                                 , dateOUT
                                 , resultado );
    END DSG_TECNICO;


    --RF60
    PROCEDURE GET_TECNICOASIGNADO( pId IN BD.PETICIONES.IDPETICION%TYPE
                                 , rs OUT SYS_REFCURSOR)
    IS
    /*
        RF60 - CASO DE USO: CONSULTAR TECNICO ASIGNADO
        PARAMETROS ENTRADA:
            pId: identificador de la peticin
                valor nulo devolver todos los registros.   
        PARAMETROS SALIDA:
            rs: cursor con el/los registro obtenido
    */
        procedimiento BD.LOG.PROCEDIMIENTO%TYPE;
        paramIN       BD.LOG.PARAMIN%TYPE;
        paramOUT      BD.LOG.PARAMOUT%TYPE;
        dateIN        BD.LOG.FXIN%TYPE;
        dateOUT       BD.LOG.FXOUT%TYPE;
        resultado     BD.LOG.RESULTADO%TYPE;
    BEGIN
    
        -- Asignamos valores entrada
        procedimiento := $$PLSQL_UNIT || '.GET_TECNICOASIGNADO';
        paramIN       := 'pId = ' || pId;
        dateIN        := SYSDATE;
        resultado     := 'OK. El tcnico se ha consultado con xito';

        -- Realizamos la consulta
        OPEN rs FOR
        SELECT TEC.* 
        FROM TECNICOS TEC INNER JOIN PETICIONES PET ON PET.idTecnico = TEC.idTecnico
        WHERE PET.idPeticion = pId;
        
        -- Asignamos valores salida
        paramOUT      := 'rs = Cursor de Peticiones.';
        dateOUT       := SYSDATE;
        
        -- Grabamos en el LOG
        CONTROL.GRABA_LOG( procedimiento
                         , paramIN
                         , paramOUT 
                         , dateIN
                         , dateOUT
                         , resultado );
         
         EXCEPTION
            WHEN OTHERS THEN
                CASE (SQLCODE)
                    WHEN 100 THEN
                        -- Registro no encontrado
                        resultado := 'ERROR: Registro(s) no encontrado(s).';
                    ELSE
                        -- Error no controlado
                        resultado := 'ERROR: ' ||  SQLERRM;
                END CASE;            

                -- Generamos un cursor vacio
                OPEN rs FOR
                SELECT * FROM DUAL WHERE DUMMY IS NULL;
                
                -- Asignamos valores salida
                paramOUT      := 'rs = Cursor vaco.';
                dateOUT       := SYSDATE;
                
                -- Mensaje de error
                DBMS_OUTPUT.put_line (resultado);

                -- Grabamos en el LOG
                CONTROL.GRABA_LOG( procedimiento
                                 , paramIN
                                 , paramOUT 
                                 , dateIN
                                 , dateOUT
                                 , resultado );
    END GET_TECNICOASIGNADO;


/*
    TRABAJOS
*/     
    --RF61
    PROCEDURE INS_TRABAJO( pIdPeticion IN BD.TRABAJOS.IDPETICION%TYPE
                         , pIdTecnico IN BD.TRABAJOS.IDTECNICO%TYPE
                         , pDsTrabajo IN BD.TRABAJOS.DSTRABAJO%TYPE
                         , pImporte IN BD.TRABAJOS.IMPORTE%TYPE
                         , pGarantia IN BD.TRABAJOS.GARANTIA%TYPE
                         , resultado OUT BD.LOG.RESULTADO%TYPE )
    IS
    /*
        RF61 - CASO DE USO: CREAR TRABAJO
        PARAMETROS ENTRADA:
 	        pIdPeticion: identificador de la peticin sobre la que se realiza el trabajo.
 	        pIdTecnico: identificador del tcnico que realiza el trabajo.
 	        pDsTrabajo: descripcin del trabajo realizado.
 	        pImporte: importe del trabajo realizado.
 	        pGarantia: indica con un valor 0 o 1 si la garanta ampara el trabajo.
        PARAMETROS SALIDA:
            resultado: Informacin del resultado de la operacin.
    */
        procedimiento BD.LOG.PROCEDIMIENTO%TYPE;
        paramIN       BD.LOG.PARAMIN%TYPE;
        paramOUT      BD.LOG.PARAMOUT%TYPE;
        dateIN        BD.LOG.FXIN%TYPE;
        dateOUT       BD.LOG.FXOUT%TYPE;
        estado        BD.PETICIONES.IDESTADO%TYPE;
        tecnico       BD.PETICIONES.IDTECNICO%TYPE;
        fechaBaja     BD.TECNICOS.FXBAJA%TYPE;
    BEGIN
    
        -- Asignamos valores entrada
        procedimiento := $$PLSQL_UNIT|| '.INS_TRABAJO';
        paramIN       := 'pIdPeticion = ' || pIdPeticion
                      || ' ,pIdTecnico = ' || pIdTecnico
                      || ' ,pDsTrabajo = ' || pDsTrabajo
                      || ' ,pImporte = ' || pImporte
                      || ' ,pGarantia = ' || pGarantia;
        dateIN        := SYSDATE;
        resultado     := 'OK. El trabajo se ha creado con xito';

        -- Miramos que exista lel tcnico y est de alta
        BEGIN
            SELECT fxBaja INTO fechaBaja 
            FROM TECNICOS
            WHERE idTecnico = pIdTecnico;
        EXCEPTION
            WHEN OTHERS THEN
                IF (SQLCODE = 100) THEN
                    -- Registro No encontrado
                    RAISE_APPLICATION_ERROR(-20010,'');
                END IF; 
        END;
        
        IF (fechaBaja IS NOT NULL) THEN
            RAISE_APPLICATION_ERROR(-20011,'');        
        END IF;
        
        -- Miramos que exista la peticin y est asignada
        BEGIN
            SELECT idEstado, idTecnico INTO estado, tecnico 
            FROM PETICIONES
            WHERE idPeticion = pIdPeticion;
        EXCEPTION
            WHEN OTHERS THEN
                IF (SQLCODE = 100) THEN
                    -- Registro No encontrado
                    RAISE_APPLICATION_ERROR(-20000,'');
                END IF; 
        END;
        
        IF (estado!='T') THEN
            RAISE_APPLICATION_ERROR(-20001,'');        
        END IF;
        
        IF (tecnico!=pIdTecnico) THEN
            RAISE_APPLICATION_ERROR(-20002,'');        
        END IF;


        -- Insertamos la Peticin
        INSERT INTO TRABAJOS(idPeticion, idTecnico, dsTrabajo, importe, garantia) 
        VALUES(pIdPeticion, pIdTecnico, pDsTrabajo, pImporte, pGarantia);
        
        -- Asignamos valores salida
        paramOUT      := 'resultado = ' || resultado;
        dateOUT       := SYSDATE;
        
        -- Grabamos en el LOG
        CONTROL.GRABA_LOG( procedimiento
                         , paramIN
                         , paramOUT 
                         , dateIN
                         , dateOUT
                         , resultado );
         
         EXCEPTION
            WHEN OTHERS THEN
                CASE (SQLCODE)
                    WHEN -1400 THEN
                        -- Valor nulo
                        resultado := 'ERROR: Se ha intentado almacenar un valor nulo.' || SUBSTR(SQLERRM,INSTR(SQLERRM,'('));
                    WHEN -20000 THEN
                            resultado := 'ERROR: El identificador de peticin no existe.';
                    WHEN -20001 THEN
                            resultado := 'ERROR: La peticin no se encuentra en estado ASIGNADA.';
                    WHEN -20002 THEN
                            resultado := 'ERROR: La peticin est asignada a otro tcnico.';
                    WHEN -20010 THEN
                            resultado := 'ERROR: El identificador de tcnico no existe.';
                    WHEN -20011 THEN
                            resultado := 'ERROR: El tcnico est de baja.';
                    WHEN -2290 THEN
                        IF (INSTR(SQLERRM,'_C01')>0) THEN
                            resultado := 'ERROR: El valor de importe debe ser mayor que cero.';
                        ELSE
                            resultado := 'ERROR: El valor de garanta debe ser 0 o 1.';
                        END IF;    
                    ELSE
                        -- Error no controlado
                        resultado := 'ERROR: ' || SQLCODE || ' - ' || SQLERRM;
                END CASE;            

                -- Asignamos valores salida
                paramOUT      := 'resultado = ' || resultado;
                dateOUT       := SYSDATE;
                
                -- Mensaje de error
                DBMS_OUTPUT.put_line (resultado);

                -- Grabamos en el LOG
                CONTROL.GRABA_LOG( procedimiento
                                 , paramIN
                                 , paramOUT 
                                 , dateIN
                                 , dateOUT
                                 , resultado );
    END INS_TRABAJO;

    -- RF62
    PROCEDURE GET_TRABAJO( pId IN BD.TRABAJOS.IDTRABAJO%TYPE
                         , rs OUT SYS_REFCURSOR)
    IS
    /*
        RF62 - CASO DE USO: CONSULTAR TRABAJO
        PARAMETROS ENTRADA:
            pId: identificador del trabajo
                valor nulo devolver todos los registros.   
        PARAMETROS SALIDA:
            rs: cursor con el/los registro obtenido
    */
        procedimiento BD.LOG.PROCEDIMIENTO%TYPE;
        paramIN       BD.LOG.PARAMIN%TYPE;
        paramOUT      BD.LOG.PARAMOUT%TYPE;
        dateIN        BD.LOG.FXIN%TYPE;
        dateOUT       BD.LOG.FXOUT%TYPE;
        resultado     BD.LOG.RESULTADO%TYPE;
    BEGIN
    
        -- Asignamos valores entrada
        procedimiento := $$PLSQL_UNIT || '.GET_TRABAJO';
        paramIN       := 'pId = ' || pId;
        dateIN        := SYSDATE;
        resultado     := 'OK. El(Los) trabajo(s) se ha(n) consultado con xito';

        -- Realizamos la consulta en funcin del parmetro de entrada
        IF (pId IS NULL) THEN
            OPEN rs FOR
            SELECT * FROM TRABAJOS;
        ELSE
            OPEN rs FOR
            SELECT * FROM TRABAJOS WHERE idTrabajo = pId;
        END IF; 
        
        -- Asignamos valores salida
        paramOUT      := 'rs = Cursor de Trabajos.';
        dateOUT       := SYSDATE;
        
        -- Grabamos en el LOG
        CONTROL.GRABA_LOG( procedimiento
                         , paramIN
                         , paramOUT 
                         , dateIN
                         , dateOUT
                         , resultado );
         
         EXCEPTION
            WHEN OTHERS THEN
                CASE (SQLCODE)
                    WHEN 100 THEN
                        -- Registro no encontrado
                        resultado := 'ERROR: Registro(s) no encontrado(s).';
                    ELSE
                        -- Error no controlado
                        resultado := 'ERROR: ' ||  SQLERRM;
                END CASE;            

                -- Generamos un cursor vacio
                OPEN rs FOR
                SELECT * FROM DUAL WHERE DUMMY IS NULL;
                
                -- Asignamos valores salida
                paramOUT      := 'rs = Cursor vaco.';
                dateOUT       := SYSDATE;
                
                -- Mensaje de error
                DBMS_OUTPUT.put_line (resultado);

                -- Grabamos en el LOG
                CONTROL.GRABA_LOG( procedimiento
                                 , paramIN
                                 , paramOUT 
                                 , dateIN
                                 , dateOUT
                                 , resultado );
    END GET_TRABAJO;


    --RF63
    PROCEDURE CLOSE_PETICION( pIdPeticion IN BD.PETICIONES.IDPETICION%TYPE
                            , resultado OUT BD.LOG.RESULTADO%TYPE )
    IS
    /*
        RF63 - CASO DE USO: CERRAR PETICION
        PARAMETROS ENTRADA:
            pIdPeticion:    Identificador de la peticin
        PARAMETROS SALIDA:
            ninguno
    */
        procedimiento BD.LOG.PROCEDIMIENTO%TYPE;
        paramIN       BD.LOG.PARAMIN%TYPE;
        paramOUT      BD.LOG.PARAMOUT%TYPE;
        dateIN        BD.LOG.FXIN%TYPE;
        dateOUT       BD.LOG.FXOUT%TYPE;
        estado        BD.PETICIONES.IDESTADO%TYPE;
        numTrabajos   NUMBER;
    BEGIN
    
        -- Asignamos valores entrada
        procedimiento := $$PLSQL_UNIT || '.DES_TECNICO';
        paramIN       := 'pIdPeticion = ' || pIdPeticion;
        dateIN        := SYSDATE;
        resultado     := 'OK. El cierre se ha realizado con xito';

        -- Miramos que exista la peticin y est asignada
        SELECT idEstado INTO estado 
        FROM PETICIONES
        WHERE idPeticion = pIdPeticion;
                
        IF (estado!='T') THEN
            RAISE_APPLICATION_ERROR(-20000,'');        
        END IF;
        
        -- Miramos que tenga trabajos pendientes
        SELECT COUNT(*) INTO numTrabajos
        FROM TRABAJOS
        WHERE idPeticion = pIdPeticion;
        
        IF (numTrabajos=0) THEN
            RAISE_APPLICATION_ERROR(-20010,'');        
        END IF;
       
        --  Cerramos la peticin
        UPDATE PETICIONES
        SET fxCierre = SYSDATE
          , idEstado = 'C'
        WHERE idPeticion = pIdPeticion;
        
        -- Asignamos valores salida
        paramOUT      := 'resultado = ' || resultado;
        dateOUT       := SYSDATE;
        
        -- Grabamos en el LOG
        CONTROL.GRABA_LOG( procedimiento
                         , paramIN
                         , paramOUT 
                         , dateIN
                         , dateOUT
                         , resultado );
         
         EXCEPTION
            WHEN OTHERS THEN
                CASE (SQLCODE)
                    WHEN 100 THEN
                        -- Tcnico dado de baja
                        resultado := 'ERROR: El identificador de peticin no existe.';
                    WHEN -20000 THEN
                        -- Estado errneo
                        resultado := 'ERROR: El peticin no se encuentra en estado ASIGNADA.';
                    WHEN -20010 THEN
                        -- Tcnico dado de baja
                        resultado := 'ERROR: La peticin no tiene trabajos.';
                    ELSE
                        -- Error no controlado
                        resultado := 'ERROR: ' ||  SQLERRM;
                END CASE;            

                -- Asignamos valores salida
                paramOUT      := 'resultado = ' || resultado;
                dateOUT       := SYSDATE;
                
                -- Mensaje de error
                DBMS_OUTPUT.put_line (resultado);

                -- Grabamos en el LOG
                CONTROL.GRABA_LOG( procedimiento
                                 , paramIN
                                 , paramOUT 
                                 , dateIN
                                 , dateOUT
                                 , resultado );
    END CLOSE_PETICION;



/*
    ENCUESTA
*/     
    --RF64
    PROCEDURE INS_ENCUESTA( pIdPeticion IN BD.ENCUESTAS.IDPETICION%TYPE
                          , pIdTeleoperador IN BD.ENCUESTAS.IDTELEOPERADOR%TYPE
                          , pValoracion IN BD.ENCUESTAS.VALORACION%TYPE
                          , pComentario IN BD.ENCUESTAS.COMENTARIO%TYPE
                          , resultado OUT BD.LOG.RESULTADO%TYPE )
    IS
    /*
        RF64 - CASO DE USO: CREAR ENCUESTA
        PARAMETROS ENTRADA:
 	        pIdPeticion: identificador de la peticin sobre la que se realiza la encuesta.
 	        pIdTeleoperador: identificador del teleoperador que realiza la encuesta.
 	        pValoracin: valoracin de la satisfaccin del cliente (0..9).
 	        pComentario: comentario de mejora.
        PARAMETROS SALIDA:
            resultado: Informacin del resultado de la operacin.
    */
        procedimiento BD.LOG.PROCEDIMIENTO%TYPE;
        paramIN       BD.LOG.PARAMIN%TYPE;
        paramOUT      BD.LOG.PARAMOUT%TYPE;
        dateIN        BD.LOG.FXIN%TYPE;
        dateOUT       BD.LOG.FXOUT%TYPE;
        estado        BD.PETICIONES.IDESTADO%TYPE;
        teleoperador  BD.ENCUESTAS.IDTELEOPERADOR%TYPE;
        fechaBaja     BD.TELEOPERADORES.FXBAJA%TYPE;
    BEGIN
    
        -- Asignamos valores entrada
        procedimiento := $$PLSQL_UNIT|| '.INS_ENCUESTA';
        paramIN       := 'pIdPeticion = ' || pIdPeticion
                      || ' ,pIdTeleoperador = ' || pIdTeleoperador
                      || ' ,pValoracion = ' || pValoracion
                      || ' ,pComentario = ' || pComentario;
        dateIN        := SYSDATE;
        resultado     := 'OK. La encuesta se ha creado con xito';

        -- Miramos que exista la peticin y est cerrada
        BEGIN
            SELECT idEstado INTO estado 
            FROM PETICIONES
            WHERE idPeticion = pIdPeticion;
        EXCEPTION
            WHEN OTHERS THEN
                IF (SQLCODE = 100) THEN
                    -- Registro No encontrado
                    RAISE_APPLICATION_ERROR(-20000,'');
                END IF; 
        END;
        
        IF (estado!='C') THEN
            RAISE_APPLICATION_ERROR(-20001,'');        
        END IF;
        
        -- Miramos que exista le teleoperador y est de alta
        BEGIN
            SELECT fxBaja INTO fechaBaja 
            FROM TELEOPERADORES
            WHERE idTeleoperador = pIdTeleoperador;
        EXCEPTION
            WHEN OTHERS THEN
                IF (SQLCODE = 100) THEN
                    -- Registro No encontrado
                    RAISE_APPLICATION_ERROR(-20010,'');
                END IF; 
        END;
        
        IF (fechaBaja IS NOT NULL) THEN
            RAISE_APPLICATION_ERROR(-20011,'');        
        END IF;
        
        -- Insertamos la Encuesta
        INSERT INTO ENCUESTAS(idPeticion, idTeleoperador, valoracion, comentario) 
        VALUES(pIdPeticion, pIdTeleoperador, pValoracion, pComentario);
        
        -- Asignamos valores salida
        paramOUT      := 'resultado = ' || resultado;
        dateOUT       := SYSDATE;
        
        -- Grabamos en el LOG
        CONTROL.GRABA_LOG( procedimiento
                         , paramIN
                         , paramOUT 
                         , dateIN
                         , dateOUT
                         , resultado );
         
         EXCEPTION
            WHEN OTHERS THEN
                CASE (SQLCODE)
                    WHEN -1400 THEN
                        -- Valor nulo
                        resultado := 'ERROR: Se ha intentado almacenar un valor nulo.' || SUBSTR(SQLERRM,INSTR(SQLERRM,'('));
                    WHEN -20000 THEN
                            resultado := 'ERROR: El identificador de peticin no existe.';
                    WHEN -20001 THEN
                            resultado := 'ERROR: La peticin no se encuentra en estado CERRADA.';
                    WHEN -20010 THEN
                            resultado := 'ERROR: El identificador de teleoperador no existe.';
                    WHEN -20011 THEN
                            resultado := 'ERROR: El teleoperador est de baja.';
                    WHEN -2290 THEN
                            resultado := 'ERROR: El valor de valorcin debe ser entre 0 y 9.';
                    ELSE
                        -- Error no controlado
                        resultado := 'ERROR: ' || SQLCODE || ' - ' || SQLERRM;
                END CASE;            

                -- Asignamos valores salida
                paramOUT      := 'resultado = ' || resultado;
                dateOUT       := SYSDATE;
                
                -- Mensaje de error
                DBMS_OUTPUT.put_line (resultado);

                -- Grabamos en el LOG
                CONTROL.GRABA_LOG( procedimiento
                                 , paramIN
                                 , paramOUT 
                                 , dateIN
                                 , dateOUT
                                 , resultado );
    END INS_ENCUESTA;

    -- RF65
    PROCEDURE GET_ENCUESTA( pId IN BD.ENCUESTAS.IDENCUESTA%TYPE
                          , rs OUT SYS_REFCURSOR)
    IS
    /*
        RF65 - CASO DE USO: CONSULTAR ENCUESTA
        PARAMETROS ENTRADA:
            pId: identificador de la encuesta
                valor nulo devolver todos los registros.   
        PARAMETROS SALIDA:
            rs: cursor con el/los registro obtenido
    */
        procedimiento BD.LOG.PROCEDIMIENTO%TYPE;
        paramIN       BD.LOG.PARAMIN%TYPE;
        paramOUT      BD.LOG.PARAMOUT%TYPE;
        dateIN        BD.LOG.FXIN%TYPE;
        dateOUT       BD.LOG.FXOUT%TYPE;
        resultado     BD.LOG.RESULTADO%TYPE;
    BEGIN
    
        -- Asignamos valores entrada
        procedimiento := $$PLSQL_UNIT || '.GET_ENCUESTA';
        paramIN       := 'pId = ' || pId;
        dateIN        := SYSDATE;
        resultado     := 'OK. La(s) encuesta(s) se ha(n) consultado con xito';

        -- Realizamos la consulta en funcin del parmetro de entrada
        IF (pId IS NULL) THEN
            OPEN rs FOR
            SELECT * FROM ENCUESTAS;
        ELSE
            OPEN rs FOR
            SELECT * FROM ENCUESTAS WHERE idEncuesta = pId;
        END IF; 
        
        -- Asignamos valores salida
        paramOUT      := 'rs = Cursor de Encuestas.';
        dateOUT       := SYSDATE;
        
        -- Grabamos en el LOG
        CONTROL.GRABA_LOG( procedimiento
                         , paramIN
                         , paramOUT 
                         , dateIN
                         , dateOUT
                         , resultado );
         
         EXCEPTION
            WHEN OTHERS THEN
                CASE (SQLCODE)
                    WHEN 100 THEN
                        -- Registro no encontrado
                        resultado := 'ERROR: Registro(s) no encontrado(s).';
                    ELSE
                        -- Error no controlado
                        resultado := 'ERROR: ' ||  SQLERRM;
                END CASE;            

                -- Generamos un cursor vacio
                OPEN rs FOR
                SELECT * FROM DUAL WHERE DUMMY IS NULL;
                
                -- Asignamos valores salida
                paramOUT      := 'rs = Cursor vaco.';
                dateOUT       := SYSDATE;
                
                -- Mensaje de error
                DBMS_OUTPUT.put_line (resultado);

                -- Grabamos en el LOG
                CONTROL.GRABA_LOG( procedimiento
                                 , paramIN
                                 , paramOUT 
                                 , dateIN
                                 , dateOUT
                                 , resultado );
    END GET_ENCUESTA;

    
END ATENCION;

