/*
    PACKAGE HISTORIAL
    Fecha: 06/05/2011
    Autor: Francisco Amador Risquez Teva
    Proyecto: gPV
*/



/*
    ESPECIFICACIN DEL PAQUETE
    SUBSISTEMA HISTORIAL
*/
CREATE OR REPLACE PACKAGE BD.HISTORIAL AS

    /*
        HISTORICO
    */     
    --RF66
    PROCEDURE GET_HCLIENTE( pIdCliente IN BD.CLIENTES.IDCLIENTE%TYPE
                          , rs OUT SYS_REFCURSOR);
    
    --RF67
    PROCEDURE GET_HPRODUCTO( pIdProducto IN BD.PRODUCTOS.IDPRODUCTO%TYPE
                           , rs OUT SYS_REFCURSOR);

    --RF68
    PROCEDURE GET_HTECNICO( pIdTecnico IN BD.TECNICOS.IDTECNICO%TYPE
                           , rs OUT SYS_REFCURSOR);

 
END HISTORIAL;



/*
    CUERPO DEL PAQUETE
    SUBSISTEMA HISTORIAL
*/
CREATE OR REPLACE PACKAGE BODY BD.HISTORIAL AS

/*
       HISTORICO
*/     
    -- RF66
    PROCEDURE GET_HCLIENTE( pIdCliente IN BD.CLIENTES.IDCLIENTE%TYPE
                          , rs OUT SYS_REFCURSOR)
    IS
    /*
        RF66 - CASO DE USO: CONSULTAR HISTORIAL CLIENTE
        PARAMETROS ENTRADA:
            pIdCliente: identificador de cliente
        PARAMETROS SALIDA:
            rs: cursor con el/los registro obtenido
    */
        procedimiento BD.LOG.PROCEDIMIENTO%TYPE;
        paramIN       BD.LOG.PARAMIN%TYPE;
        paramOUT      BD.LOG.PARAMOUT%TYPE;
        dateIN        BD.LOG.FXIN%TYPE;
        dateOUT       BD.LOG.FXOUT%TYPE;
        resultado     BD.LOG.RESULTADO%TYPE;
    BEGIN
    
        -- Asignamos valores entrada
        procedimiento := $$PLSQL_UNIT || '.GET_HCLIENTE';
        paramIN       := 'pIdCliente = ' || pIdCliente;
        dateIN        := SYSDATE;
        resultado     := 'OK. La(s) peticin(es) se ha(n) consultado con xito';

        -- Realizamos la consulta
        OPEN rs FOR
        SELECT * 
        FROM PETICIONES PET INNER JOIN (PRODUCTOS PRO INNER JOIN CLIENTES CLI ON CLI.idCliente = PRO.idCliente) ON PET.idProducto = PRO.idProducto
        WHERE PRO.idCliente=pIdCliente;
        
        -- Asignamos valores salida
        paramOUT      := 'rs = Cursor de Peticiones.';
        dateOUT       := SYSDATE;
        
        -- Grabamos en el LOG
        CONTROL.GRABA_LOG( procedimiento
                         , paramIN
                         , paramOUT 
                         , dateIN
                         , dateOUT
                         , resultado );
         
         EXCEPTION
            WHEN OTHERS THEN
                CASE (SQLCODE)
                    WHEN 100 THEN
                        -- Registro no encontrado
                        resultado := 'ERROR: Registro(s) no encontrado(s).';
                    ELSE
                        -- Error no controlado
                        resultado := 'ERROR: ' ||  SQLERRM;
                END CASE;            

                -- Generamos un cursor vacio
                OPEN rs FOR
                SELECT * FROM DUAL WHERE DUMMY IS NULL;
                
                -- Asignamos valores salida
                paramOUT      := 'rs = Cursor vaco.';
                dateOUT       := SYSDATE;
                
                -- Mensaje de error
                DBMS_OUTPUT.put_line (resultado);

                -- Grabamos en el LOG
                CONTROL.GRABA_LOG( procedimiento
                                 , paramIN
                                 , paramOUT 
                                 , dateIN
                                 , dateOUT
                                 , resultado );
    END GET_HCLIENTE;


    -- RF67
    PROCEDURE GET_HPRODUCTO( pIdProducto IN BD.PRODUCTOS.IDPRODUCTO%TYPE
                           , rs OUT SYS_REFCURSOR)
    IS
    /*
        RF67 - CASO DE USO: CONSULTAR HISTORIAL PRODUCTO
        PARAMETROS ENTRADA:
            pIdProducto: identificador de producto
        PARAMETROS SALIDA:
            rs: cursor con el/los registro obtenido
    */
        procedimiento BD.LOG.PROCEDIMIENTO%TYPE;
        paramIN       BD.LOG.PARAMIN%TYPE;
        paramOUT      BD.LOG.PARAMOUT%TYPE;
        dateIN        BD.LOG.FXIN%TYPE;
        dateOUT       BD.LOG.FXOUT%TYPE;
        resultado     BD.LOG.RESULTADO%TYPE;
    BEGIN
    
        -- Asignamos valores entrada
        procedimiento := $$PLSQL_UNIT || '.GET_HPRODUCTO';
        paramIN       := 'pIdProducto = ' || pIdProducto;
        dateIN        := SYSDATE;
        resultado     := 'OK. La(s) peticin(es) se ha(n) consultado con xito';

        -- Realizamos la consulta
        OPEN rs FOR
        SELECT * 
        FROM PETICIONES PET INNER JOIN PRODUCTOS PRO ON PET.idProducto = PRO.idProducto
        WHERE PRO.idProducto=pIdProducto;
        
        -- Asignamos valores salida
        paramOUT      := 'rs = Cursor de Peticiones.';
        dateOUT       := SYSDATE;
        
        -- Grabamos en el LOG
        CONTROL.GRABA_LOG( procedimiento
                         , paramIN
                         , paramOUT 
                         , dateIN
                         , dateOUT
                         , resultado );
         
         EXCEPTION
            WHEN OTHERS THEN
                CASE (SQLCODE)
                    WHEN 100 THEN
                        -- Registro no encontrado
                        resultado := 'ERROR: Registro(s) no encontrado(s).';
                    ELSE
                        -- Error no controlado
                        resultado := 'ERROR: ' ||  SQLERRM;
                END CASE;            

                -- Generamos un cursor vacio
                OPEN rs FOR
                SELECT * FROM DUAL WHERE DUMMY IS NULL;
                
                -- Asignamos valores salida
                paramOUT      := 'rs = Cursor vaco.';
                dateOUT       := SYSDATE;
                
                -- Mensaje de error
                DBMS_OUTPUT.put_line (resultado);

                -- Grabamos en el LOG
                CONTROL.GRABA_LOG( procedimiento
                                 , paramIN
                                 , paramOUT 
                                 , dateIN
                                 , dateOUT
                                 , resultado );
    END GET_HPRODUCTO;


    -- RF68
    PROCEDURE GET_HTECNICO( pIdTecnico IN BD.TECNICOS.IDTECNICO%TYPE
                           , rs OUT SYS_REFCURSOR)
    IS
    /*
        RF68 - CASO DE USO: CONSULTAR HISTORIAL TECNICO
        PARAMETROS ENTRADA:
            pIdTcnico: identificador de tcnico
        PARAMETROS SALIDA:
            rs: cursor con el/los registro obtenido
    */
        procedimiento BD.LOG.PROCEDIMIENTO%TYPE;
        paramIN       BD.LOG.PARAMIN%TYPE;
        paramOUT      BD.LOG.PARAMOUT%TYPE;
        dateIN        BD.LOG.FXIN%TYPE;
        dateOUT       BD.LOG.FXOUT%TYPE;
        resultado     BD.LOG.RESULTADO%TYPE;
    BEGIN
    
        -- Asignamos valores entrada
        procedimiento := $$PLSQL_UNIT || '.GET_HTECNICO';
        paramIN       := 'pIdTecnico = ' || pIdTecnico;
        dateIN        := SYSDATE;
        resultado     := 'OK. Lo(s) trabajo(s) se ha(n) consultado con xito';

        -- Realizamos la consulta
        OPEN rs FOR
        SELECT * 
        FROM TRABAJOS TRA INNER JOIN (PETICIONES PET 
                                      INNER JOIN (PRODUCTOS PRO 
                                                  INNER JOIN CLIENTES CLI 
                                                  ON PRO.idCliente = CLI.idCliente )
                                      ON PET.idProducto = PRO.idProducto) 
                          ON TRA.idPeticion = PET.idPeticion
        WHERE TRA.idTecnico=pIdTecnico;
        
        -- Asignamos valores salida
        paramOUT      := 'rs = Cursor de Trabajos.';
        dateOUT       := SYSDATE;
        
        -- Grabamos en el LOG
        CONTROL.GRABA_LOG( procedimiento
                         , paramIN
                         , paramOUT 
                         , dateIN
                         , dateOUT
                         , resultado );
         
         EXCEPTION
            WHEN OTHERS THEN
                CASE (SQLCODE)
                    WHEN 100 THEN
                        -- Registro no encontrado
                        resultado := 'ERROR: Registro(s) no encontrado(s).';
                    ELSE
                        -- Error no controlado
                        resultado := 'ERROR: ' ||  SQLERRM;
                END CASE;            

                -- Generamos un cursor vacio
                OPEN rs FOR
                SELECT * FROM DUAL WHERE DUMMY IS NULL;
                
                -- Asignamos valores salida
                paramOUT      := 'rs = Cursor vaco.';
                dateOUT       := SYSDATE;
                
                -- Mensaje de error
                DBMS_OUTPUT.put_line (resultado);

                -- Grabamos en el LOG
                CONTROL.GRABA_LOG( procedimiento
                                 , paramIN
                                 , paramOUT 
                                 , dateIN
                                 , dateOUT
                                 , resultado );
    END GET_HTECNICO;
    
END HISTORIAL;

