/*
    PACKAGE ETL
    Fecha: 10/05/2011
    Autor: Francisco Amador Risquez Teva
    Proyecto: gPV
*/



/*
    ESPECIFICACIN DEL PAQUETE
    ETL
*/
CREATE OR REPLACE PACKAGE DW.ETL AS

    /*
        DIMENSIONES
    */
	PROCEDURE CARGA_D_ELECTRODOMESTICOS;

	PROCEDURE CARGA_D_LOCALIZACIONES;

	PROCEDURE CARGA_D_SERVICIOS;
    
    PROCEDURE CARGA_D_TECNICOS;

    PROCEDURE CARGA_D_TELEOPERADORES;

    PROCEDURE CARGA_D_TEMPORAL;


    /*
        HECHOS
    */
    PROCEDURE CARGA_H_PETICIONES;
    
    PROCEDURE CARGA_H_TECNICOS;
    
    PROCEDURE CARGA_H_TELEOPERADORES;
    
    
    /* 
        CARGAMOS DATOS EN EL ALMACEN
     */
    PROCEDURE CARGAS;

 

END ETL;



/*
    CUERPO DEL PAQUETE
    ETL
*/
CREATE OR REPLACE PACKAGE BODY DW.ETL AS

/*
    DIMENSIONES
*/


	PROCEDURE CARGA_D_ELECTRODOMESTICOS
    AS
    BEGIN
    
        -- Modificamos los datos de los electrodomsticos existentes excepto el ID 
        UPDATE D_ELECTRODOMESTICOS DE
        SET DE.dsElectrodomestico = (SELECT dsElectrodomestico 
                                     FROM BD.ELECTRODOMESTICOS E
                                     WHERE  DE.idElectrodomestico=E.idElectrodomestico);
                                     
        -- Insertamos los electrodomsticos no existentes
        INSERT INTO D_ELECTRODOMESTICOS
        SELECT idElectrodomestico, dsElectrodomestico
        FROM BD.ELECTRODOMESTICOS
        WHERE idElectrodomestico NOT IN (SELECT idElectrodomestico FROM D_ELECTRODOMESTICOS);
    
    END CARGA_D_ELECTRODOMESTICOS;



	PROCEDURE CARGA_D_LOCALIZACIONES
    AS
    BEGIN

        -- Insertamos las nuevas localizaciones
        /* 
            La PK estar formada por:
            identificador de Zona con 5 caracteres y mscara 00000
           codigoPostal de la Provincia
           cdigo de la Provincia
        */
        INSERT INTO D_LOCALIZACION
        SELECT LPAD(Z.idZona,5,'0')||PB.codPostal||PR.coProvincia as idLocalizacion, PB.dsPoblacion, PR.dsProvincia, Z.dsZona
        FROM BD.ZONASPOBLACIONES ZP 
             INNER JOIN BD.ZONAS Z ON Z.idZona = ZP.idZona
             INNER JOIN (BD.POBLACIONES PB INNER JOIN BD.PROVINCIAS PR ON PB.coProvincia = PR.coProvincia) ON PB.codPostal = ZP.codPostal
        WHERE LPAD(Z.idZona,5,'0')||PB.codPostal||PR.coProvincia NOT IN (SELECT idLocalizacion FROM D_LOCALIZACION);
        
    
    END CARGA_D_LOCALIZACIONES;



	PROCEDURE CARGA_D_SERVICIOS
    AS
    BEGIN
    
        -- Modificamos los datos de los servicios existentes excepto el ID 
        UPDATE D_SERVICIOS DS
        SET DS.dsServicio = (SELECT dsServicio 
                             FROM BD.SERVICIOS S
                             WHERE  DS.idServicio=S.idServicio);
                                     
        -- Insertamos los servicios no existentes
        INSERT INTO D_SERVICIOS
        SELECT idServicio, dsServicio
        FROM BD.SERVICIOS
        WHERE idServicio NOT IN (SELECT idServicio FROM D_SERVICIOS);
    
    END CARGA_D_SERVICIOS;
    
    
    
	PROCEDURE CARGA_D_TECNICOS
    AS
    BEGIN
    
        -- Modificamos los datos de los tecnicos existentes excepto el ID 
        UPDATE D_TECNICOS DT
        SET DT.nombre = (SELECT nombre || ' ' || apellidos  
                         FROM BD.TECNICOS T
                             WHERE  DT.idTecnico=T.idTecnico);
                                     
        -- Insertamos los tcnicos no existentes
        INSERT INTO D_TECNICOS
        SELECT idTecnico, nombre || ' ' || apellidos
        FROM BD.TECNICOS
        WHERE idTecnico NOT IN (SELECT idTecnico FROM D_TECNICOS);
    
    END CARGA_D_TECNICOS;



	PROCEDURE CARGA_D_TELEOPERADORES
    AS
    BEGIN
    
        -- Modificamos los datos de los teleoperadores existentes excepto el ID 
        UPDATE D_TELEOPERADORES DT
        SET DT.nombre = (SELECT nombre || ' ' || apellidos  
                         FROM BD.TELEOPERADORES T
                             WHERE  DT.idTeleoperador=T.idTeleoperador);
                                     
        -- Insertamos los teleoperadores no existentes
        INSERT INTO D_TELEOPERADORES
        SELECT idTeleoperador, nombre || ' ' || apellidos
        FROM BD.TELEOPERADORES
        WHERE idTeleoperador NOT IN (SELECT idTeleoperador FROM D_TELEOPERADORES);
    
    END CARGA_D_TELEOPERADORES;



    PROCEDURE CARGA_D_TEMPORAL
    AS
    BEGIN
    
        --Insertamos las fechas del sistema
        INSERT INTO D_TEMPORAL
        SELECT id , mes, ano
        FROM (
                SELECT TO_CHAR(fxAlta,'YYYYMM') AS id, TO_CHAR(fxAlta,'MM') AS mes, TO_CHAR(fxAlta, 'YYYY') AS ano FROM BD.CLIENTES
                UNION
                SELECT TO_CHAR(fxEncuesta,'YYYYMM'), TO_CHAR(fxEncuesta,'MM'), TO_CHAR(fxEncuesta, 'YYYY') FROM BD.ENCUESTAS
                UNION
                SELECT TO_CHAR(fxReclamacion,'YYYYMM'), TO_CHAR(fxReclamacion,'MM'), TO_CHAR(fxReclamacion, 'YYYY') FROM BD.PETICIONES
                UNION
                SELECT TO_CHAR(fxCompra,'YYYYMM'), TO_CHAR(fxCompra,'MM'), TO_CHAR(fxCompra, 'YYYY') FROM BD.PRODUCTOS
                UNION
                SELECT TO_CHAR(fxTrabajo,'YYYYMM'), TO_CHAR(fxTrabajo,'MM'), TO_CHAR(fxTrabajo, 'YYYY') FROM BD.TRABAJOS
        ) F
        WHERE F.mes NOT IN (SELECT idTemporal FROM D_TEMPORAL);
        
        
    END CARGA_D_TEMPORAL;




/*
    HECHOS
*/


    PROCEDURE CARGA_H_PETICIONES
    AS
    BEGIN
    
        --Insertamos las peticiones nuevas
        INSERT INTO H_PETICIONES
        SELECT idTemporal, idLocalizacion, idServicio, idElectrodomestico, numPeticiones, importe, impGarantia, valoracionMedia
        FROM (
                SELECT TO_CHAR(P.fxReclamacion,'YYYYMM') AS idTemporal
                     , LPAD(ZP.idZona,5,'0')||PD.codPostal||PB.coProvincia as idLocalizacion       
                     , P.idServicio AS idServicio
                     , PD.idElectrodomestico AS idElectrodomestico
                     , COUNT(P.idPeticion) AS numPeticiones
                     , SUM((SELECT importe FROM BD.TRABAJOS T WHERE T.idPeticion =  P.idPeticion)) AS importe
                     , SUM((SELECT (importe*garantia) FROM BD.TRABAJOS T WHERE T.idPeticion =  P.idPeticion)) AS impGarantia
                     , AVG((SELECT (valoracion) FROM BD.ENCUESTAS E WHERE E.idPeticion =  P.idPeticion)) AS valoracionMedia
                FROM BD.PETICIONES P INNER JOIN (BD.PRODUCTOS PD INNER JOIN BD.ZONASPOBLACIONES ZP ON PD.codPostal = ZP.codPostal 
                                                                 INNER JOIN BD.POBLACIONES PB ON PD.codPostal = PB.codPostal) 
                                                ON P.idProducto = PD.idProducto
                WHERE fxCierre IS NOT NULL
                GROUP BY TO_CHAR(fxReclamacion,'YYYYMM')
                       , LPAD(ZP.idZona,5,'0')||PD.codPostal||PB.coProvincia
                       , P.idServicio
                       , PD.idElectrodomestico
        ) H
        WHERE 0 = (SELECT COUNT(*) 
                   FROM H_PETICIONES 
                   WHERE idTemporal = H.idTemporal
                     AND idLocalizacion = H.idLocalizacion
                     AND idServicio = H.idServicio
                     AND idElectrodomestico = H.idElectrodomestico);
        
        
    END CARGA_H_PETICIONES;



    PROCEDURE CARGA_H_TECNICOS
    AS
    BEGIN
    
        --Insertamos las peticiones nuevas
        INSERT INTO H_TECNICOS
        SELECT idTemporal, idTecnico, idServicio, idElectrodomestico, numTrabajos, importe, impGarantia
        FROM (
                SELECT TO_CHAR(T.fxTrabajo,'YYYYMM') AS idTemporal
                     , T.idTecnico AS idTecnico
                     , P.idServicio AS idServicio
                     , PD.idElectrodomestico AS idElectrodomestico
                     , COUNT(T.idTrabajo) AS numTrabajos
                     , SUM((SELECT importe FROM BD.TRABAJOS T2 WHERE T2.idTrabajo =  T.idTrabajo)) AS importe
                     , SUM((SELECT (importe*garantia) FROM BD.TRABAJOS T2 WHERE T2.idTrabajo =  T.idTrabajo)) AS impGarantia
                FROM BD.TRABAJOS T INNER JOIN (BD.PETICIONES P INNER JOIN BD.PRODUCTOS PD ON P.idProducto = PD.idProducto)
                                   ON T.idPeticion = P.idPeticion
                GROUP BY TO_CHAR(T.fxTrabajo,'YYYYMM')
                       , T.idTecnico
                       , P.idServicio
                       , PD.idElectrodomestico
        ) H
        WHERE 0 = (SELECT COUNT(*) 
                   FROM H_PETICIONES 
                   WHERE idTemporal = H.idTemporal
                     AND idTecnico = H.idTecnico
                     AND idServicio = H.idServicio
                     AND idElectrodomestico = H.idElectrodomestico);
        
        
    END CARGA_H_TECNICOS;



    PROCEDURE CARGA_H_TELEOPERADORES
    AS
    BEGIN
    
        --Insertamos las peticiones nuevas
        INSERT INTO H_TELEOPERADORES 
        SELECT idTemporal, idTeleoperador, idServicio, idElectrodomestico, numPeticiones, numEncuestas
        FROM (
                SELECT TO_CHAR(P.fxReclamacion,'YYYYMM') AS idTemporal
                     , P.idTeleoperador AS idTeleoperador
                     , P.idServicio AS idServicio
                     , PD.idElectrodomestico AS idElectrodomestico
                     , COUNT(P.idPeticion) AS numPeticiones
                     , -1 as numEncuestas
                FROM BD.PETICIONES P INNER JOIN BD.PRODUCTOS PD ON P.idProducto = PD.idProducto
                GROUP BY TO_CHAR(P.fxReclamacion,'YYYYMM')
                       , P.idTeleoperador
                       , P.idServicio
                       , PD.idElectrodomestico
        ) H
        WHERE 0 = (SELECT COUNT(*) 
                   FROM H_TELEOPERADORES 
                   WHERE idTemporal = H.idTemporal
                     AND idTeleoperador = H.idTeleoperador
                     AND idServicio = H.idServicio
                     AND idElectrodomestico = H.idElectrodomestico);

    -- Modificamos encuestas 
    UPDATE H_TELEOPERADORES H
    SET numEncuestas = (SELECT COUNT(*)
                        FROM BD.ENCUESTAS E INNER JOIN (BD.PETICIONES P INNER JOIN BD.PRODUCTOS PD ON PD.idProducto=P.idProducto)
                                            ON E.idPeticion=P.idPeticion
                        WHERE TO_CHAR(E.fxEncuesta,'YYYYMM')=H.idTemporal
                        AND E.idTeleoperador = H.idTeleoperador
                        AND P.idServicio = H.idServicio
                        AND PD.idElectrodomestico = H.idElectrodomestico
                        ) 
    WHERE numEncuestas < 0; 
    
    --Insertamos encuestas en fechas no existentes
        INSERT INTO H_TELEOPERADORES 
        SELECT idTemporal, idTeleoperador, idServicio, idElectrodomestico, numPeticiones, numEncuestas
        FROM (
                SELECT TO_CHAR(E.fxEncuesta,'YYYYMM') AS idTemporal
                     , E.idTeleoperador AS idTeleoperador
                     , P.idServicio AS idServicio
                     , PD.idElectrodomestico AS idElectrodomestico
                     , 0 AS numPeticiones
                     , COUNT(*) AS numEncuestas
                FROM BD.ENCUESTAS E INNER JOIN (BD.PETICIONES P INNER JOIN BD.PRODUCTOS PD ON PD.idProducto=P.idProducto)
                                    ON P.idPeticion=E.idPeticion
                GROUP BY TO_CHAR(E.fxEncuesta,'YYYYMM') 
                       , E.idTeleoperador
                       , P.idServicio
                       , PD.idElectrodomestico
        ) H
        WHERE 0 = (SELECT COUNT(*) 
                   FROM H_TELEOPERADORES 
                   WHERE idTemporal = H.idTemporal
                     AND idTeleoperador = H.idTeleoperador
                     AND idServicio = H.idServicio
                     AND idElectrodomestico = H.idElectrodomestico);
        
    END CARGA_H_TELEOPERADORES;


    PROCEDURE CARGAS
    AS
    BEGIN
    
        -- Cargamos dimensin Electrodomsticos
        CARGA_D_ELECTRODOMESTICOS;
        
        -- Cargamos dimensin Localizacin
        CARGA_D_LOCALIZACIONES;
        
        -- Cargamos dimensin Servicios
        CARGA_D_SERVICIOS;
        
        -- Cargamos dimensin Tcnicos
        CARGA_D_TECNICOS;
        
        -- Cargamos dimensin Teleoperadores
        CARGA_D_TELEOPERADORES;

        -- Cargamos dimensin Temporal
        CARGA_D_TEMPORAL;
                
        -- Cargamos Hechos Peticiones
        CARGA_H_PETICIONES;
        
        -- Cargamos Hechos Tcnicos
        CARGA_H_TECNICOS;
        
        -- Cargamos Hechos Teleoperadores
        CARGA_H_TELEOPERADORES;
        
    END CARGAS;

    
END ETL;

