/*
    PACKAGE TEST_CONFIGURACION
    Fecha: 22/04/2011
    Autor: Francisco Amador Risquez Teva
    Proyecto: gPV
*/



/*
    ESPECIFICACIN DEL PAQUETE
    TEST_CONFIGURACION
*/
CREATE OR REPLACE PACKAGE BD.TEST_CONFIGURACION AS

    --TEST01
    PROCEDURE TEST_SERVICIOS;
    
    --TEST02
    PROCEDURE TEST_ELECTRODOMESTICOS;

    --TEST03
    PROCEDURE TEST_PROVINCIAS;

    --TEST04
    PROCEDURE TEST_POBLACIONES;

    --TEST05
    PROCEDURE TEST_ZONAS;


END TEST_CONFIGURACION;



/*
    CUERPO DEL PAQUETE
    TEST_CONFIGURACION
*/
CREATE OR REPLACE PACKAGE BODY BD.TEST_CONFIGURACION AS

    -- TEST01
    PROCEDURE TEST_SERVICIOS 
    IS
    resultado BD.LOG.RESULTADO%TYPE;
    BEGIN
        /*
            RF01 INS_SERVICIO
        */
        --Insertamos Servicio de forma correcta
        CONFIGURACION.INS_SERVICIO('SERVICIO_01', resultado);
        -- Resultado correcto
        DBMS_OUTPUT.put_line (resultado);

        --Insertamos Servicio de forma correcta
        CONFIGURACION.INS_SERVICIO('SERVICIO_02', resultado);
        -- Resultado correcto
        DBMS_OUTPUT.put_line (resultado);

        --Insertamos Servicio de forma correcta
        CONFIGURACION.INS_SERVICIO('SERVICIO_03', resultado);
        -- Resultado correcto
        DBMS_OUTPUT.put_line (resultado);

        --Insertamos Servicio con descripcin existente
        CONFIGURACION.INS_SERVICIO('SERVICIO_01', resultado);

        --Insertamos Servicio con descripcin nula
        CONFIGURACION.INS_SERVICIO(NULL, resultado);

        
        /*
            RF02 DEL_SERVICIO
        */
        --Damos de baja el Servicio de forma correcta
        CONFIGURACION.DEL_SERVICIO(2, resultado);
        -- Resultado correcto
        DBMS_OUTPUT.put_line (resultado);

        --Damos de baja el Servicio de forma correcta
        CONFIGURACION.DEL_SERVICIO(3, resultado);
        -- Resultado correcto
        DBMS_OUTPUT.put_line (resultado);

        --Damos de baja un Servicio que no existe
        CONFIGURACION.DEL_SERVICIO(8, resultado);
        
        --Damos de baja un Servicio que est de baja
        CONFIGURACION.DEL_SERVICIO(2, resultado);


        /*
            RF03 RES_SERVICIO
        */
        --Recuperamos el Servicio de forma correcta
        CONFIGURACION.RES_SERVICIO(2, resultado);
        -- Resultado correcto
        DBMS_OUTPUT.put_line (resultado);

        --Recuperamos un Servicio que no existe
        CONFIGURACION.RES_SERVICIO(8, resultado);
        
        --Recuperamos un Servicio que NO est de baja
        CONFIGURACION.RES_SERVICIO(1, resultado);


        /*
            RF04 UPD_SERVICIO
        */
        --Modificamos el Servicio de forma correcta
        CONFIGURACION.UPD_SERVICIO(2,'SERVICIO_02BIS', resultado);
        -- Resultado correcto
        DBMS_OUTPUT.put_line (resultado);

        --Modificamos un Servicio que no existe
        CONFIGURACION.UPD_SERVICIO(8, 'SERVICIO_MODIFICADO', resultado);
        
        --Modificamos un Servicio con una descripcin existente
        CONFIGURACION.UPD_SERVICIO(2, 'SERVICIO_01', resultado);
        
        --Modificamos un Servicio con una descripcin nula
        CONFIGURACION.UPD_SERVICIO(2, NULL, resultado);

        --Modificamos un Servicio que est de baja
        CONFIGURACION.UPD_SERVICIO(3, 'SERVICIO_03BIS', resultado);


    END TEST_SERVICIOS;


    -- TEST02
    PROCEDURE TEST_ELECTRODOMESTICOS 
    IS
    resultado BD.LOG.RESULTADO%TYPE;
    BEGIN
        /*
            RF06 INS_ELECTRODOMESTICO
        */
        --Insertamos Electrodomstico de forma correcta
        CONFIGURACION.INS_ELECTRODOMESTICO('ELECTRODOMESTICO_01', resultado);
        -- Resultado correcto
        DBMS_OUTPUT.put_line (resultado);

        --Insertamos Electrodomstico de forma correcta
        CONFIGURACION.INS_ELECTRODOMESTICO('ELECTRODOMESTICO_02', resultado);
        -- Resultado correcto
        DBMS_OUTPUT.put_line (resultado);

        --Insertamos Electrodomstico de forma correcta
        CONFIGURACION.INS_ELECTRODOMESTICO('ELECTRODOMESTICO_03', resultado);
        -- Resultado correcto
        DBMS_OUTPUT.put_line (resultado);

        --Insertamos Electrodomstico con descripcin existente
        CONFIGURACION.INS_ELECTRODOMESTICO('ELECTRODOMESTICO_01', resultado);

        --Insertamos Electrodomstico con descripcin nula
        CONFIGURACION.INS_ELECTRODOMESTICO(NULL, resultado);

        
        /*
            RF07 DEL_ELECTRODOMESTICO
        */
        --Damos de baja el Electrodomstico de forma correcta
        CONFIGURACION.DEL_ELECTRODOMESTICO(2, resultado);
        -- Resultado correcto
        DBMS_OUTPUT.put_line (resultado);

        --Damos de baja el Electrodomstico de forma correcta
        CONFIGURACION.DEL_ELECTRODOMESTICO(3, resultado);
        -- Resultado correcto
        DBMS_OUTPUT.put_line (resultado);

        --Damos de baja un Electrodomstico que no existe
        CONFIGURACION.DEL_ELECTRODOMESTICO(8, resultado);
        
        --Damos de baja un Electrodomstico que est de baja
        CONFIGURACION.DEL_ELECTRODOMESTICO(2, resultado);


        /*
            RF08 RES_ELECTRODOMESTICO
        */
        --Recuperamos el Electrodomstico de forma correcta
        CONFIGURACION.RES_ELECTRODOMESTICO(2, resultado);
        -- Resultado correcto
        DBMS_OUTPUT.put_line (resultado);

        --Recuperamos un Electrodomstico que no existe
        CONFIGURACION.RES_ELECTRODOMESTICO(8, resultado);
        
        --Recuperamos un Electrodomstico que NO est de baja
        CONFIGURACION.RES_ELECTRODOMESTICO(1, resultado);


        /*
            RF09 UPD_ELECTRODOMESTICO
        */
        --Modificamos el Electrodomstico de forma correcta
        CONFIGURACION.UPD_SERVICIO(2,'ELECTRODOMESTICO_02BIS', resultado);
        -- Resultado correcto
        DBMS_OUTPUT.put_line (resultado);

        --Modificamos un Electrodomstico que no existe
        CONFIGURACION.UPD_ELECTRODOMESTICO(8, 'ELECTRODOMESTICO_MODIFICADO', resultado);
        
        --Modificamos un Electrodomstico con una descripcin existente
        CONFIGURACION.UPD_ELECTRODOMESTICO(2, 'ELECTRODOMESTICO_01', resultado);
        
        --Modificamos un Electrodomstico con una descripcin nula
        CONFIGURACION.UPD_ELECTRODOMESTICO(2, NULL, resultado);

        --Modificamos un Electrodomstico que est de baja
        CONFIGURACION.UPD_ELECTRODOMESTICO(3, 'ELECTRODOMESTICO_03BIS', resultado);


    END TEST_ELECTRODOMESTICOS;

    
    -- TEST03
    PROCEDURE TEST_PROVINCIAS
    IS
    resultado BD.LOG.RESULTADO%TYPE;
    BEGIN
        /*
            RF11 INS_PROVINCIA
        */
        --Insertamos Provincia de forma correcta
        CONFIGURACION.INS_PROVINCIA('01', 'PROVINCIA_01', resultado);
        -- Resultado correcto
        DBMS_OUTPUT.put_line (resultado);

        --Insertamos Provincia de forma correcta
        CONFIGURACION.INS_PROVINCIA('02','PROVINCIA_02', resultado);
        -- Resultado correcto
        DBMS_OUTPUT.put_line (resultado);

        --Insertamos Provincia de forma correcta
        CONFIGURACION.INS_PROVINCIA('03','PROVINCIA_03', resultado);
        -- Resultado correcto
        DBMS_OUTPUT.put_line (resultado);

        --Insertamos Provincia con cdigo existente
        CONFIGURACION.INS_PROVINCIA('01', 'PROVINCIA_04', resultado);

        --Insertamos Provincia con descripcin existente
        CONFIGURACION.INS_PROVINCIA('04', 'PROVINCIA_01', resultado);

        --Insertamos Provincia con cdigo nulo
        CONFIGURACION.INS_PROVINCIA(NULL, 'PROVINCIA_04', resultado);

        --Insertamos Provincia con descripcin nula
        CONFIGURACION.INS_PROVINCIA('04',NULL, resultado);

        
        /*
            RF12 DEL_PROVINCIA
        */
        --Damos de baja la Provincia de forma correcta
        CONFIGURACION.DEL_PROVINCIA('02', resultado);
        -- Resultado correcto
        DBMS_OUTPUT.put_line (resultado);

        --Damos de baja una Provincia que no existe
        CONFIGURACION.DEL_PROVINCIA('00', resultado);
        
        --Damos de baja una Provincia con Poblaciones asignadas
            --Creamos primero una poblacin en una provincia
        CONFIGURACION.INS_POBLACION('03001','POBLACION_03001', '03', resultado);
        CONFIGURACION.DEL_PROVINCIA('03', resultado);


        /*
            RF13 UPD_PROVINCIA
        */
        --Modificamos la Provincia de forma correcta
        CONFIGURACION.UPD_PROVINCIA('01','PROVINCIA_01BIS', resultado);
        -- Resultado correcto
        DBMS_OUTPUT.put_line (resultado);

        --Modificamos una Provincia que no existe
        CONFIGURACION.UPD_PROVINCIA('00', 'PROVINCIA_MODIFICADA', resultado);
        
        --Modificamos una Provincia con una descripcin existente
        CONFIGURACION.UPD_PROVINCIA('01', 'PROVINCIA_03', resultado);
        
        --Modificamos una Provincia con una descripcin nula
        CONFIGURACION.UPD_PROVINCIA('01', NULL, resultado);



    END TEST_PROVINCIAS;


    -- TEST04
    PROCEDURE TEST_POBLACIONES
    IS
    resultado BD.LOG.RESULTADO%TYPE;
    BEGIN
        /*
            RF15 INS_POBLACION
        */
        --Insertamos Poblacin de forma correcta
        CONFIGURACION.INS_POBLACION('01001', 'POBLACION_01001', '01', resultado);
        -- Resultado correcto
        DBMS_OUTPUT.put_line (resultado);

        --Insertamos Poblacin de forma correcta
        CONFIGURACION.INS_POBLACION('01002','POBLACION_01002', '01', resultado);
        -- Resultado correcto
        DBMS_OUTPUT.put_line (resultado);

        --Insertamos Poblacin de forma correcta
        CONFIGURACION.INS_POBLACION('03002','POBLACION_03002', '03', resultado);
        -- Resultado correcto
        DBMS_OUTPUT.put_line (resultado);

        --Insertamos Poblacin de forma correcta
        CONFIGURACION.INS_POBLACION('03003','POBLACION_03003', '03', resultado);
        -- Resultado correcto
        DBMS_OUTPUT.put_line (resultado);

        --Insertamos Poblacin con cdigo existente
        CONFIGURACION.INS_POBLACION('01001', 'POBLACION_01004', '01', resultado);

        --Insertamos Poblacin con descripcin existente
        CONFIGURACION.INS_POBLACION('01003', 'POBLACION_01002', '01', resultado);

        --Insertamos Poblacin con Provincia no existente
        CONFIGURACION.INS_POBLACION('01004', 'POBLACION_01004', '00', resultado);

        --Insertamos Poblacin con cdigo nulo
        CONFIGURACION.INS_POBLACION(NULL, 'POBLACION_01004', '01', resultado);

        --Insertamos Provincia con descripcin nula
        CONFIGURACION.INS_POBLACION('01004',NULL, '01', resultado);

        --Insertamos Poblacin con Provincia nula
        CONFIGURACION.INS_POBLACION('01004', 'POBLACION_01004', NULL, resultado);

        
        /*
            RF16 DEL_POBLACION
        */
        --Damos de baja la Poblacin de forma correcta
        CONFIGURACION.DEL_POBLACION('03003', resultado);
        -- Resultado correcto
        DBMS_OUTPUT.put_line (resultado);

        --Damos de baja una Poblacin que no existe
        CONFIGURACION.DEL_POBLACION('00000', resultado);
        
        --Damos de baja una Poblacin asignada a una zona
            --Creamos primero una zona y asignamos una provincia a la zona
        CONFIGURACION.INS_ZONA('ZONA_01', resultado);
        CONFIGURACION.ASG_POBLACIONZONA('01001','01', resultado);
        CONFIGURACION.DEL_POBLACION('01001', resultado);


        /*
            RF17 UPD_POBLACION
        */
        --Modificamos la Poblacin de forma correcta
        CONFIGURACION.UPD_POBLACION('01002','POBLACION_01002BIS', '01', resultado);
        -- Resultado correcto
        DBMS_OUTPUT.put_line (resultado);

        --Modificamos una Poblacin que no existe
        CONFIGURACION.UPD_POBLACION('00000', 'POBLACION_MODIFICADA', '01', resultado);
        
        --Modificamos una Poblacin con una descripcin existente
        CONFIGURACION.UPD_POBLACION('01002', 'POBLACION_01001', '01', resultado);
        
        --Modificamos una Poblacin con una Provincia inexistente
        CONFIGURACION.UPD_POBLACION('01002', 'POBLACION_01002BIS', '00', resultado);

        --Modificamos una Poblacin con una descripcin nula
        CONFIGURACION.UPD_POBLACION('01002', NULL, '01', resultado);

        --Modificamos una Poblacin con una Provincia nula
        CONFIGURACION.UPD_POBLACION('01002', 'POBLACION_MODIFICADA', NULL, resultado);


    END TEST_POBLACIONES;


    -- TEST05
    PROCEDURE TEST_ZONAS
    IS
    resultado BD.LOG.RESULTADO%TYPE;
    BEGIN
        /*
            RF19 INS_ZONA
        */
        --Insertamos Zona de forma correcta
        CONFIGURACION.INS_ZONA('ZONA_02', resultado);
        -- Resultado correcto
        DBMS_OUTPUT.put_line (resultado);

        --Insertamos Zona de forma correcta
        CONFIGURACION.INS_ZONA('ZONA_03', resultado);
        -- Resultado correcto
        DBMS_OUTPUT.put_line (resultado);

        --Insertamos Zona con descripcin existente
        CONFIGURACION.INS_ZONA('ZONA_01', resultado);

         --Insertamos Zona con descripcin nula
        CONFIGURACION.INS_ZONA(NULL, resultado);

        
        /*
            RF20 DEL_ZONA
        */
        --Damos de baja la Zona de forma correcta
        CONFIGURACION.DEL_ZONA(3, resultado);
        -- Resultado correcto
        DBMS_OUTPUT.put_line (resultado);

        --Damos de baja una Zona que no existe
        CONFIGURACION.DEL_ZONA(88, resultado);
        
        --Damos de baja una Zona con Poblaciones asignadas
        CONFIGURACION.DEL_ZONA(1, resultado);


        /*
            RF21 UPD_ZONA
        */
        --Modificamos la Zona de forma correcta
        CONFIGURACION.UPD_ZONA(2,'ZONA_02BIS', resultado);
        -- Resultado correcto
        DBMS_OUTPUT.put_line (resultado);

        --Modificamos una Zona que no existe
        CONFIGURACION.UPD_ZONA(88, 'ZONA_MODIFICADA', resultado);
        
        --Modificamos una Zona con una descripcin existente
        CONFIGURACION.UPD_ZONA(2, 'ZONA_01', resultado);
        
        --Modificamos una Zona con una descripcin nula
        CONFIGURACION.UPD_ZONA(2, NULL, resultado);


        /*
            RF23 ASG_PROVINCIAZONA
        */
        --Asignamos una provincia a una zona de forma correcta
        CONFIGURACION.ASG_PROVINCIAZONA('01',2, resultado);
        -- Resultado correcto
        DBMS_OUTPUT.put_line (resultado);

        --Asignamos una Provincia que no existe a una Zona
        CONFIGURACION.ASG_PROVINCIAZONA('00', 2, resultado);
        
        --Asignamos una Provincia a una Zona que no existente
        CONFIGURACION.ASG_PROVINCIAZONA('01', 88, resultado);
        

        /*
            RF24 ASG_POBLACIONZONA
        */
        --Asignamos una poblacin a una zona de forma correcta
        CONFIGURACION.ASG_POBLACIONZONA('03002',2, resultado);
        -- Resultado correcto
        DBMS_OUTPUT.put_line (resultado);

        --Asignamos una Poblacin que no existe a una Zona
        CONFIGURACION.ASG_POBLACIONZONA('00000', 2, resultado);
        
        --Asignamos una Poblacin a una Zona no existente
        CONFIGURACION.ASG_POBLACIONZONA('03002', 88, resultado);

        --Asignamos una Poblacin ya asignada a la misma Zona
        CONFIGURACION.ASG_POBLACIONZONA('03002', 2, resultado);


        /*
            RF25 DSG_PROVINCIAZONA
        */
        --Desasignamos una provincia a una zona de forma correcta
        CONFIGURACION.DSG_PROVINCIAZONA('03',2, resultado);
        -- Resultado correcto
        DBMS_OUTPUT.put_line (resultado);

        --Desasignamos una Provincia que no existe a una Zona
        CONFIGURACION.DSG_PROVINCIAZONA('00', 1, resultado);
        
        --Desasignamos una Provincia a una Zona no existente
        CONFIGURACION.DSG_PROVINCIAZONA('01', 88, resultado);


        /*
            RF26 DSG_POBLACIONZONA
        */
        --Desasignamos una poblacin a una zona de forma correcta
        CONFIGURACION.DSG_POBLACIONZONA('01001',1, resultado);
        -- Resultado correcto
        DBMS_OUTPUT.put_line (resultado);

        --Designamos una Poblacin que no existe a una Zona
        CONFIGURACION.DSG_POBLACIONZONA('00000', 1, resultado);
        
        --Desasignamos una Poblacin a una Zona que no existente
        CONFIGURACION.DSG_POBLACIONZONA('01002', 88, resultado);

        --Desasignamos una Poblacin No asignada a una Zona
        CONFIGURACION.DSG_POBLACIONZONA('01002', 2, resultado);
        
    END TEST_ZONAS;




END TEST_CONFIGURACION;

