/*
    PACKAGE TEST_ATENCION
    Fecha: 04/05/2011
    Autor: Francisco Amador Risquez Teva
    Proyecto: gPV
*/



/*
    ESPECIFICACIN DEL PAQUETE
    TEST_ATENCION
*/
CREATE OR REPLACE PACKAGE BD.TEST_ATENCION AS

    --TEST11
    PROCEDURE TEST_ASIGNACIONES;
    
    --TEST12
    PROCEDURE TEST_TRABAJOS;

    --TEST13
    PROCEDURE TEST_ENCUESTAS;


END TEST_ATENCION;



/*
    CUERPO DEL PAQUETE
    TEST_ATENCION
*/
CREATE OR REPLACE PACKAGE BODY BD.TEST_ATENCION AS


    -- TEST11
    PROCEDURE TEST_ASIGNACIONES 
    IS
    resultado BD.LOG.RESULTADO%TYPE;
    BEGIN

        /*
            RF58 ASG_TECNICO
        */
        --Asignamos Peticin a Tcnico de forma correcta
        ATENCION.ASG_TECNICO(1, 1, resultado);
        -- Resultado correcto
        DBMS_OUTPUT.put_line (resultado);

        --Asignamos Peticin a Tcnico de forma correcta
        ATENCION.ASG_TECNICO(2, 2, resultado);
        -- Resultado correcto
        DBMS_OUTPUT.put_line (resultado);

        --Asignamos Peticin no existente a un Tcnico
        ATENCION.ASG_TECNICO(88, 1, resultado);

        --Asignamos Peticin no PENDIENTE a un Tcnico
        ATENCION.ASG_TECNICO(2, 1, resultado);

        --Asignamos Peticin a un Tcnico no existente 
        ATENCION.ASG_TECNICO(3, 88, resultado);

        --Asignamos Peticin a un Tcnico dado de baja 
        ATENCION.ASG_TECNICO(3, 3, resultado);

        --Asignamos Peticin a Tcnico de forma correcta
        ATENCION.ASG_TECNICO(3, 2, resultado);
        -- Resultado correcto
        DBMS_OUTPUT.put_line (resultado);


        /*
            RF59 DSG_TECNICO
        */
        --Desasignamos Peticin de forma correcta
        ATENCION.DSG_TECNICO(3, resultado);
        -- Resultado correcto
        DBMS_OUTPUT.put_line (resultado);

        --Desasignamos Peticin no existente
        ATENCION.DSG_TECNICO(88, resultado);

        --Desasignamos Peticin en estado no ASIGNADA
        ATENCION.DSG_TECNICO(3, resultado);


    END TEST_ASIGNACIONES;



    -- TEST12
    PROCEDURE TEST_TRABAJOS 
    IS
    resultado BD.LOG.RESULTADO%TYPE;
    BEGIN

        /*
            RF61 INS_TRABAJO
        */
        --Creamos Trabajo de forma correcta
        ATENCION.INS_TRABAJO(1, 1, 'TRABAJO_01', 1000, 0, resultado);
        -- Resultado correcto
        DBMS_OUTPUT.put_line (resultado);

        --Creamos Trabajo de forma correcta
        ATENCION.INS_TRABAJO(1, 1, 'TRABAJO_02', 100, 1, resultado);
        -- Resultado correcto
        DBMS_OUTPUT.put_line (resultado);

        --Creamos Trabajo de forma correcta
        ATENCION.INS_TRABAJO(2, 2, 'TRABAJO_03', 500, 1, resultado);
        -- Resultado correcto
        DBMS_OUTPUT.put_line (resultado);


        --Creamos Trabajo con valor nulo
        ATENCION.INS_TRABAJO(2, 2, NULL, 300, 1, resultado);
        ATENCION.INS_TRABAJO(2, 2, 'TRABAJO_04', NULL, 0, resultado);
        ATENCION.INS_TRABAJO(2, 2, 'TRABAJO_03', 300, NULL, resultado);
        
        --Creamos Trabajo con una Peticion no existente
        ATENCION.INS_TRABAJO(88, 2, 'TRABAJO_03', 500, 1, resultado);
            
        --Creamos Trabajo con una Peticin en estado no ASIGNADA
        ATENCION.INS_TRABAJO(3, 1, 'TRABAJO_03', 500, 1, resultado);
        
        --Creamos Trabajo con una Peticin asignada a otro tcnico
        ATENCION.INS_TRABAJO(1, 2, 'TRABAJO_03', 500, 1, resultado);

        --Creamos Trabajo con una Tcnico no existente
        ATENCION.INS_TRABAJO(1, 88, 'TRABAJO_03', 500, 1, resultado);

        --Creamos Trabajo con una Tcnico dado de baja
        ATENCION.INS_TRABAJO(1, 3, 'TRABAJO_03', 500, 1, resultado);

        --Creamos Trabajo con valor no vlido de importe
        ATENCION.INS_TRABAJO(1, 1, 'TRABAJO_03', -500, 1, resultado);

        --Creamos Trabajo con valor no vlido de garanta
        ATENCION.INS_TRABAJO(1, 1, 'TRABAJO_03', 500, 6, resultado);


        /*
            RF63 CLOSE_PETICION
        */
        --Cerramos Peticin de forma correcta
        ATENCION.CLOSE_PETICION(1, resultado);
        -- Resultado correcto
        DBMS_OUTPUT.put_line (resultado);

        --Cerramos Peticin no existente
        ATENCION.CLOSE_PETICION(88, resultado);

        --Cerramos Peticin en estado no ASIGNADA
        ATENCION.CLOSE_PETICION(1, resultado);

        --Cerramos Peticin sin trabajos asociados
        --Asignamos primero la peticn a un tcnico
        ATENCION.ASG_TECNICO(3, 2, resultado);
        ATENCION.CLOSE_PETICION(3, resultado);


    END TEST_TRABAJOS;



    -- TEST13
    PROCEDURE TEST_ENCUESTAS
    IS
    resultado BD.LOG.RESULTADO%TYPE;
    BEGIN

        /*
            RF64 INS_ENCUESTA
        */
        --Creamos Encuesta de forma correcta
        ATENCION.INS_ENCUESTA(1, 1, 8, 'Correcto', resultado);
        -- Resultado correcto
        DBMS_OUTPUT.put_line (resultado);

        --Creamos Encuesta con una Peticion no existente
        ATENCION.INS_ENCUESTA(88, 1, 8, 'Bien', resultado);
            
        --Creamos Trabajo con una Peticin en estado no CERRADA
        ATENCION.INS_ENCUESTA(2, 1, 8, 'Bien', resultado);
        
        --Creamos Trabajo con una Teleoperador no existente
        ATENCION.INS_ENCUESTA(1, 88, 7, 'Perfecto', resultado);

        --Creamos Trabajo con una Teleoperador dado de baja
        ATENCION.INS_ENCUESTA(1, 3, 7, 'Perfecto', resultado);

        --Creamos Teletrabajo con valoracin no vlida
        ATENCION.INS_ENCUESTA(1, 1, 16, 'Correcto', resultado);


     END TEST_ENCUESTAS;


END TEST_ATENCION;

