/*
    PACKAGE CARGA_INICIAL
    Fecha: 16/05/2011
    Autor: Francisco Amador Risquez Teva
    Proyecto: gPV
*/


/*
    ESPECIFICACIN DEL PAQUETE
    CARGA INICIAL
*/

CREATE OR REPLACE PACKAGE BD.CARGA_INICIAL AS

    -- Carga Servicios
    PROCEDURE CARGA_SERVICIOS(num NUMBER);

    -- Carga Electrodomsticos
    PROCEDURE CARGA_ELECTRODOMESTICOS(num NUMBER);

    -- Carga Provincias
    PROCEDURE CARGA_PROVINCIAS(num NUMBER);

    -- Carga Poblaciones
    PROCEDURE CARGA_POBLACIONES(num NUMBER);

    -- Carga y asigna Zonas (habr tantas Zonas como Provincias)
    PROCEDURE CARGA_ZONAS(num NUMBER);

    -- Carga Teleoperadores
    PROCEDURE CARGA_TELEOPERADORES(num NUMBER);

    -- Carga y asignaciones de Tcnicos
    PROCEDURE CARGA_TECNICOS(num NUMBER);

    -- Carga Clientes
    PROCEDURE CARGA_CLIENTES(num NUMBER);

    -- Carga Productos (entre 1 y 3 por Cliente)
    PROCEDURE CARGA_PRODUCTOS(num NUMBER);

    -- Carga y asignacin de Peticiones (entre 1 y 3 por Producto)
    PROCEDURE CARGA_PETICIONES(num NUMBER);

    -- Cargamos un Trabajo por cada peticin asignada y la cierra
    PROCEDURE CARGA_TRABAJOS;
    
    -- Cargamos una encuesta por cada peticin cerrada
    PROCEDURE CARGA_ENCUESTAS;
    
    -- Modificamos las fechas de forma aleatoria para realizar estadsticas
    PROCEDURE MODIFICA_FECHAS;

    -- Carga de datos en el sistema
    PROCEDURE CARGA;
    
    --Borra el contenido de todas las tablas
    PROCEDURE DEL_ALL;

 
END CARGA_INICIAL;


/*
    CUERPO DEL PAQUETE
    CARGA INICIAL
*/

CREATE OR REPLACE PACKAGE BODY BD.CARGA_INICIAL AS

    -- Cargamos servicios
    PROCEDURE CARGA_SERVICIOS(num NUMBER) IS
    /*
        PARAMETROS ENTRADA:
            num: nmero de servicios a insertar
        PARAMETROS SALIDA:
            ninguno
    */
      resultado VARCHAR2(255);
      contador NUMBER;
    BEGIN
    
        FOR contador IN 1..num
        LOOP
            --Insertamos Servicio
            BD.CONFIGURACION.INS_SERVICIO('SERVICIO_'||LPAD(contador,2,'0'), resultado);
        END LOOP;

    END CARGA_SERVICIOS;
    
    
    -- Cargamos Electrodomsticos
    PROCEDURE CARGA_ELECTRODOMESTICOS(num NUMBER) IS
    /*
        PARAMETROS ENTRADA:
            num: nmero de electrodomsticos a insertar
        PARAMETROS SALIDA:
            ninguno
    */
      resultado VARCHAR2(255);
      contador NUMBER;
    BEGIN
    
        FOR contador IN 1..num
        LOOP
            --Insertamos Electrodomstico
            BD.CONFIGURACION.INS_ELECTRODOMESTICO('ELECTRODOMESTICO_'||LPAD(contador,2,'0'), resultado);
        END LOOP;

    END CARGA_ELECTRODOMESTICOS;


    -- Cargamos Provincias
    PROCEDURE CARGA_PROVINCIAS(num NUMBER) IS
    /*
        PARAMETROS ENTRADA:
            num: nmero de provincias a insertar
        PARAMETROS SALIDA:
            ninguno
    */
      resultado VARCHAR2(255);
      contador NUMBER;
    BEGIN
    
        FOR contador IN 1..num
        LOOP
            --Insertamos Provincia
            BD.CONFIGURACION.INS_PROVINCIA(LPAD(contador,2,'0'),'PROVINCIA_'||LPAD(contador,2,'0'), resultado);
        END LOOP;

    END CARGA_PROVINCIAS;


    -- Cargamos Poblaciones por cada Provincia
    PROCEDURE CARGA_POBLACIONES(num NUMBER) IS
    /*
        PARAMETROS ENTRADA:
            num: nmero de poblaciones por provincia a insertar
        PARAMETROS SALIDA:
            ninguno
    */
      resultado VARCHAR2(255);
      contador NUMBER;
    BEGIN
    
        FOR provincias IN (SELECT coProvincia FROM BD.PROVINCIAS)
        LOOP
            FOR contador IN 1..num
            LOOP
                --Insertamos Poblacin
                BD.CONFIGURACION.INS_POBLACION(provincias.coProvincia||LPAD(contador,3,'0')
                                              ,'POBLACION_'||provincias.coProvincia||LPAD(contador,3,'0')
                                              , provincias.coProvincia
                                              , resultado);
            END LOOP;
        END LOOP; 

    END CARGA_POBLACIONES;


    -- Cargamos y asignamos ZONAS (habr tantas Zonas como provincias)
    PROCEDURE CARGA_ZONAS(num NUMBER) IS
    /*
        PARAMETROS ENTRADA:
            num: nmero de zonas a insertar
        PARAMETROS SALIDA:
            ninguno
    */
      resultado VARCHAR2(255);
      contador NUMBER;
      id1 NUMBER;
    BEGIN

        FOR contador IN 1..num
        LOOP
            --Insertamos Zona
            BD.CONFIGURACION.INS_ZONA('ZONA_'||LPAD(contador,2,'0'), resultado);
        END LOOP;

        --Asignamos cada provincia a una zona
        FOR provincias IN (SELECT coProvincia FROM BD.PROVINCIAS)
        LOOP
            SELECT idZona INTO id1 FROM BD.ZONAS WHERE dsZona='ZONA_'||provincias.coProvincia;
            BD.CONFIGURACION.ASG_PROVINCIAZONA(provincias.coProvincia, id1, resultado);
        END LOOP; 

    END CARGA_ZONAS;


    -- Cargamos Teleoperadores
    PROCEDURE CARGA_TELEOPERADORES(num NUMBER) IS
    /*
        PARAMETROS ENTRADA:
            num: nmero de teleoperadores a insertar
        PARAMETROS SALIDA:
            ninguno
    */
      resultado VARCHAR2(255);
      contador NUMBER;
    BEGIN
    
        FOR contador IN 1..num
        LOOP
            --Insertamos Teleoperador
            BD.PERSONAL.INS_TELEOPERADOR('TELE_'||LPAD(contador,2,'0')
                                        ,'OPERADOR_'||LPAD(contador,2,'0')
                                        , LPAD(contador,9,'0')
                                        , resultado);
        END LOOP;

    END CARGA_TELEOPERADORES;


    -- Cargamos Tecnicos
    PROCEDURE CARGA_TECNICOS(num NUMBER) IS
    /*
        PARAMETROS ENTRADA:
            num: nmero de tcnicos a insertar
        PARAMETROS SALIDA:
            ninguno
    */
      resultado VARCHAR2(255);
      contador NUMBER;
      veces NUMBER;
      id1 NUMBER;
    BEGIN
    
        FOR contador IN 1..num
        LOOP
            --Insertamos tcnico
            BD.PERSONAL.INS_TECNICO('TEC_'||LPAD(contador,2,'0')
                                   ,'NICO_'||LPAD(contador,2,'0')
                                   , LPAD(contador,9,'0')
                                   , '6'|| LPAD(contador,8,'0')
                                   , resultado);
        END LOOP;
        
        --Asignamos cada tcnico a un nmero aleatorio de servicios
        FOR tecnicos IN (SELECT idTecnico FROM BD.TECNICOS)
        LOOP
            -- Obtenemos un servicio de forma aleatoria
            SELECT TRUNC(DBMS_RANDOM.VALUE(1,6)) INTO veces FROM DUAL; 
            FOR contador IN 1..veces
            LOOP
                SELECT idServicio INTO id1
                FROM ( SELECT idServicio
                       FROM bd.servicios s
                       WHERE idServicio NOT IN (SELECT idServicio 
                                                FROM bd.tecnicosServicios 
                                                WHERE idTecnico=tecnicos.idTecnico)
                       ORDER BY dbms_random.value ) 
                WHERE rownum = 1;
                
                --Realizamos la asignacin
                BD.PERSONAL.ASG_SERVICIOTECNICO(id1, tecnicos.idTecnico, resultado);
            END LOOP;
        END LOOP; 
        
        --Asignamos cada tcnico a un nmero aleatorio de electrodomsticos
        FOR tecnicos IN (SELECT idTecnico FROM BD.TECNICOS)
        LOOP
            SELECT TRUNC(DBMS_RANDOM.VALUE(1,6)) INTO veces FROM DUAL; 
            FOR contador IN 1..veces
            LOOP
                -- Obtenemos un electrodomstico de manera aleatoria
                SELECT idElectrodomestico INTO id1
                FROM ( SELECT idElectrodomestico
                       FROM bd.electrodomesticos
                       WHERE idElectrodomestico NOT IN (SELECT idElectrodomestico 
                                                        FROM bd.tecnicosElectrodomesticos 
                                                        WHERE idTecnico=tecnicos.idTecnico)
                       ORDER BY dbms_random.value ) 
                WHERE rownum = 1;
                
                -- Realizamos la asignacin
                BD.PERSONAL.ASG_ELECTRODOMESTICOTECNICO(id1, tecnicos.idTecnico, resultado);
                
            END LOOP;
        END LOOP; 

        --Asignamos cada tcnico a un nmero aleatorio de zonas
        FOR tecnicos IN (SELECT idTecnico FROM BD.TECNICOS)
        LOOP
            SELECT TRUNC(DBMS_RANDOM.VALUE(1,6)) INTO veces FROM DUAL; 
            FOR contador IN 1..veces
            LOOP
        
                -- Obtenemos Zona aleatoria de las asignadas al tcnico
                SELECT idZona INTO id1
                FROM ( SELECT idZona
                       FROM bd.zonas
                       WHERE idZona NOT IN (SELECT idZona 
                                            FROM bd.tecnicosZonas 
                                            WHERE idTecnico=tecnicos.idTecnico)
                       ORDER BY dbms_random.value ) 
                WHERE rownum = 1;
                
                -- Realizamos la asignancin
                BD.PERSONAL.ASG_ZONATECNICO(id1, tecnicos.idTecnico, resultado);
        
            END LOOP;
        END LOOP; 
        

    END CARGA_TECNICOS;


    -- Cargamos Clientes
    PROCEDURE CARGA_CLIENTES(num NUMBER) IS
    /*
        PARAMETROS ENTRADA:
            num: nmero de clientes a insertar
        PARAMETROS SALIDA:
            ninguno
    */
      resultado VARCHAR2(255);
      contador NUMBER;
      cp BD.POBLACIONES.CODPOSTAL%TYPE;
    BEGIN
    
        FOR contador IN 1..num
        LOOP
            -- Obtenemos una poblacin aleatoria
            SELECT codpostal into cp 
            FROM ( SELECT codpostal FROM bd.poblaciones ORDER BY dbms_random.value ) 
            WHERE rownum = 1;
        
            -- Insertamos Cliente
            BD.PETICION.INS_CLIENTE('CLI_'||LPAD(contador,2,'0')
                                   ,'ENTE_'||LPAD(contador,2,'0')
                                   , LPAD(contador,9,'0')
                                   , 'DIRECCION_'||LPAD(contador,2,'0')
                                   , cp
                                   , '9'|| LPAD(contador,8,'0')
                                   , '6'|| LPAD(contador,8,'0')
                                   , 'cliente'|| LPAD(contador,8,'0')||'@hotmail.com'
                                   , resultado);
        END LOOP;

    END CARGA_CLIENTES;


    -- Cargamos Productos de 1 a 3 por cada cliente
    PROCEDURE CARGA_PRODUCTOS(num NUMBER) IS
    /*
        PARAMETROS ENTRADA:
            num: mxmio nmero de productos por cliente
        PARAMETROS SALIDA:
            ninguno
    */
      resultado VARCHAR2(255);
      contador NUMBER;
      veces NUMBER;
      id1 NUMBER;
      cp BD.POBLACIONES.CODPOSTAL%TYPE;
    BEGIN
    
        -- Insertamos de 1 a num productos por cada Cliente
        FOR clientes IN (SELECT idCliente FROM BD.CLIENTES)
        LOOP
            SELECT TRUNC(DBMS_RANDOM.VALUE(1,num+1)) INTO veces FROM DUAL; 
            FOR contador IN 1..veces
            LOOP
                --Obtenemos electrodomestico
                SELECT idElectrodomestico into id1 
                FROM (SELECT idElectrodomestico FROM bd.electrodomesticos ORDER BY dbms_random.value ) 
                WHERE rownum = 1;

                -- Obtenemos una poblacin aleatoria
                SELECT codpostal into cp 
                FROM ( SELECT codpostal FROM bd.poblaciones ORDER BY dbms_random.value ) 
                WHERE rownum = 1;

                -- Insertamos Producto
                BD.PETICION.INS_PRODUCTO( id1
                                        , 'MARCA_'||LPAD(clientes.idCliente,3,'0')||LPAD(contador,2,'0')
                                        , 'MODELO_'||LPAD(clientes.idCliente,3,'0')||LPAD(contador,2,'0')
                                        , 'NS_'||LPAD(clientes.idCliente,3,'0')||LPAD(contador,2,'0')
                                        , TRUNC(SYSDATE - DBMS_RANDOM.VALUE(750,3000))
                                        , 'DIRECCION_'||LPAD(clientes.idCliente,3,'0')||LPAD(contador,2,'0')
                                        , cp
                                        , clientes.idCliente
                                        , resultado);

            END LOOP;    
        END LOOP;

    END CARGA_PRODUCTOS;


    -- Cargamos Peticiones de 1 a 3 por cada producto
    PROCEDURE CARGA_PETICIONES(num NUMBER) IS
    /*
        PARAMETROS ENTRADA:
            num: mximo nmero de peticiones a insertar por producto
        PARAMETROS SALIDA:
            ninguno
    */
      resultado VARCHAR2(255);
      contador NUMBER;
      veces NUMBER;
      id1 NUMBER;
      id2 NUMBER;
      cp BD.POBLACIONES.CODPOSTAL%TYPE;
    BEGIN
    
        -- Insertamos de 1 a num peticiones por producto
        FOR productos IN (SELECT idProducto FROM BD.PRODUCTOS)
        LOOP
            SELECT TRUNC(DBMS_RANDOM.VALUE(1,num+1)) INTO veces FROM DUAL; 
            FOR contador IN 1..veces
            LOOP
                --Obtenemos servicio
                SELECT idServicio into id1 
                FROM (SELECT idServicio FROM bd.servicios ORDER BY dbms_random.value ) 
                WHERE rownum = 1;
        
                --Obtenemos teleoperador
                SELECT idTeleoperador into id2 
                FROM (SELECT idTeleoperador FROM bd.teleoperadores ORDER BY dbms_random.value ) 
                WHERE rownum = 1;

                -- Insertamos peticin
                BD.PETICION.INS_PETICION( productos.idProducto
                                        , id1
                                        , id2
                                        , resultado);

            END LOOP;
        END LOOP;


        -- Asignamos cada peticin a un tcnico
        FOR peticiones IN (SELECT pe.idPeticion, pe.idServicio, pr.idElectrodomestico, pr.codPostal 
                           FROM bd.peticiones pe INNER JOIN bd.productos pr ON pe.idProducto=pr.idProducto)
        LOOP
            BEGIN
                --Obtenemos un tcnico que pueda atender esa peticin
                SELECT idTecnico into id1 
                FROM (SELECT idTecnico FROM bd.tecnicosservicios ts 
                              INNER JOIN bd.tecnicoselectrodomesticos te ON te.idtecnico=ts.idtecnico
                              INNER JOIN bd.tecnicoszonas tz ON tz.idtecnico=ts.idtecnico
                      WHERE idServicio=1
                        AND idElectrodomestico=1
                        AND idZona=1
                      ORDER BY dbms_random.value ) 
                WHERE rownum = 1;
            
                --Asignamos al tcnico
                BD.ATENCION.ASG_TECNICO(peticiones.idPeticion, id1, resultado);
            EXCEPTION
                WHEN NO_DATA_FOUND THEN
                    --Ningn tcnico puede atender la peticin
                    NULL;
            END;
    
        END LOOP;

    END CARGA_PETICIONES;


    -- Cargamos un trabajo por cada peticin asignada y la cierra
    PROCEDURE CARGA_TRABAJOS IS
    /*
        PARAMETROS ENTRADA:
            ninguno
        PARAMETROS SALIDA:
            ninguno
    */
      resultado VARCHAR2(255);
      contador NUMBER;
      valor NUMBER;
      esGarantia NUMBER;
    BEGIN
    
        -- Insertamos un trabajo por cada peticin asignada
        FOR peticiones IN (SELECT idPeticion, idTecnico FROM BD.PETICIONES WHERE idEstado='T')
        LOOP
            --Obtenemos importe
            SELECT ROUND(DBMS_RANDOM.VALUE(0,3000),2) INTO valor FROM DUAL; 
            --Obtenemos garantia
            SELECT TRUNC(DBMS_RANDOM.VALUE(0,2)) INTO esGarantia FROM DUAL; 

            -- Insertamos Trabajo
            BD.ATENCION.INS_TRABAJO( peticiones.idPeticion
                                   , peticiones.idTecnico
                                   , 'TRABAJO_'||LPAD(peticiones.idPeticion,2,'0')||LPAD(peticiones.idTecnico,2,'0')
                                   , valor
                                   , esGarantia
                                   , resultado);

            -- Cerramos la peticin
            BD.ATENCION.CLOSE_PETICION( peticiones.idPeticion
                                      , resultado);
                                      
         END LOOP;

    END CARGA_TRABAJOS;


    -- Cargamos una encuesta por cada peticin cerrada
    PROCEDURE CARGA_ENCUESTAS IS
    /*
        PARAMETROS ENTRADA:
            ninguno
        PARAMETROS SALIDA:
            ninguno
    */
      resultado VARCHAR2(255);
      contador NUMBER;
      id1 NUMBER;
      valor NUMBER;
    BEGIN
    
        -- Insertamos una encuesta por cada peticin cerrada
        FOR peticiones IN (SELECT idPeticion, idTecnico FROM BD.PETICIONES WHERE idEstado='C')
        LOOP
            --Obtenemos teleoperador
            SELECT idTeleoperador into id1
            FROM (SELECT idTeleoperador FROM bd.teleoperadores ORDER BY dbms_random.value ) 
            WHERE rownum = 1;

            --Obtenemos valoracin
            SELECT TRUNC(DBMS_RANDOM.VALUE(0,10)) INTO valor FROM DUAL; 

            -- Insertamos Encuesta
            BD.ATENCION.INS_ENCUESTA( peticiones.idPeticion
                                    , id1
                                    , valor
                                    , 'COMENTARIO_'||LPAD(peticiones.idPeticion,4,'0')
                                    , resultado);
         END LOOP;

    END CARGA_ENCUESTAS;


    -- Modificamos las fechas de forma aleatoria para realizar estadsticas
    PROCEDURE MODIFICA_FECHAS IS
    /*
        PARAMETROS ENTRADA:
            ninguno
        PARAMETROS SALIDA:
            ninguno
    */
      fecha DATE;
    BEGIN
    
        --Modificamos la fecha de creacin
        UPDATE BD.PETICIONES P
        SET fxReclamacion = (SELECT TRUNC(fxCompra + DBMS_RANDOM.VALUE(1,400)) FROM BD.PRODUCTOS WHERE idProducto=P.idProducto); 
                    
        --Modificamos la fecha del trabajo 
        UPDATE BD.TRABAJOS T
        SET fxTrabajo = (SELECT TRUNC(fxReclamacion + DBMS_RANDOM.VALUE(1,15)) FROM BD.PETICIONES WHERE idPeticion=T.idPeticion);

        --Modificamos la fecha de cierre
        UPDATE BD.PETICIONES P
        SET fxCierre = (SELECT fxTrabajo FROM BD.TRABAJOS WHERE idPeticion=P.idPeticion);
            
        --Modificamos la fecha de la encuesta 
        UPDATE BD.ENCUESTAS E
        SET fxEncuesta = (SELECT TRUNC(fxCierre + DBMS_RANDOM.VALUE(1,15)) FROM BD.PETICIONES WHERE idPeticion=E.idPeticion);


    END MODIFICA_FECHAS;


    -- Carga de datos
    PROCEDURE CARGA IS
    /*
        PARAMETROS ENTRADA:
            ninguno
        PARAMETROS SALIDA:
            ninguno
    */
        nServicios NUMBER:=5;
        nElectrodomesticos NUMBER:=5;
        nProvincias NUMBER:=5;
        nPoblaciones NUMBER:=50;
        nTeleoperadores NUMBER:=5;
        nTecnicos NUMBER:=20;
        nClientes NUMBER:=300;
        maxProductos NUMBER:=3;
        maxPeticiones NUMBER:=5;
    BEGIN
        -- Borramos el contenido de las tablas
        DEL_ALL;
        
        --Cargamos Servicios
        CARGA_SERVICIOS(nServicios);
        
        --Cargamos Electrodomsticos
        CARGA_ELECTRODOMESTICOS(nElectrodomesticos);

        --Cargamos Provincias
        CARGA_PROVINCIAS(nProvincias);

        --Cargamos Poblaciones por Provincias
        CARGA_POBLACIONES(nPoblaciones);
        
        --Cargamos Zonas (Cada provincia pertenecer a una zona)
        CARGA_ZONAS(nProvincias);

        --Cargamos Teleoperadores
        CARGA_TELEOPERADORES(nTeleoperadores);

        --Cargamos Tecnicos
        CARGA_TECNICOS(nTecnicos);

        --Cargamos Clientes
        CARGA_CLIENTES(nClientes);

        --Cargamos Productos
        CARGA_PRODUCTOS(maxProductos);

        --Cargamos Peticiones
        CARGA_PETICIONES(maxPeticiones);
        
        --Cargamos Trabajos y cerramos peticiones
        CARGA_TRABAJOS;

        --Cargamos Encuestas
        CARGA_ENCUESTAS;
        
        --Modificamos fechas para poder realizar estadsticas
        MODIFICA_FECHAS;

    END CARGA;    
    
    -- Borramos el contenido de todas las tablas
    PROCEDURE DEL_ALL IS
    /*
        PARAMETROS ENTRADA:
            ninguno
        PARAMETROS SALIDA:
            ninguno
    */
    BEGIN
        DELETE FROM bd.encuestas;
        DELETE FROM bd.trabajos;
        DELETE FROM bd.peticiones;
        DELETE FROM bd.productos;
        DELETE FROM bd.clientes;
        DELETE FROM bd.tecnicoszonas;
        DELETE FROM bd.tecnicoselectrodomesticos;
        DELETE FROM bd.tecnicosservicios;
        DELETE FROM bd.tecnicos;
        DELETE FROM bd.teleoperadores;
        DELETE FROM bd.zonaspoblaciones;
        DELETE FROM bd.poblaciones;
        DELETE FROM bd.zonas;
        DELETE FROM bd.poblaciones;
        DELETE FROM bd.provincias;
        DELETE FROM bd.electrodomesticos;
        DELETE FROM bd.servicios; 
        DELETE FROM bd.log;   
    END DEL_ALL;
    
    
    
END CARGA_INICIAL;

